@@ -1,6 +1,6 @@
 /*
  * blocklist.c
- * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * Build, read and destroy a list of Word text blocks
@@ -34,7 +34,7 @@ static list_mem_type	*pFootBlockCurrent = NULL;
 /* Variable needed to read the endnote block list */
 static list_mem_type	*pEndBlockCurrent = NULL;
 /* Last block read */
-static unsigned char	aucBlock[BIG_BLOCK_SIZE];
+static UCHAR		aucBlock[BIG_BLOCK_SIZE];
 
 
 /*
@@ -87,22 +87,22 @@ bAdd2TextBlockList(text_block_type *pTextBlock)
 	list_mem_type	*pListMember;
 
 	fail(pTextBlock == NULL);
-	fail(pTextBlock->lFileOffset < 0);
-	fail(pTextBlock->lTextOffset < 0);
-	fail(pTextBlock->lLength <= 0);
-	fail(pTextBlock->bUsesUnicode && odd(pTextBlock->lLength));
+	fail(pTextBlock->ulFileOffset == FC_INVALID);
+	fail(pTextBlock->ulCharPos == CP_INVALID);
+	fail(pTextBlock->ulLength == 0);
+	fail(pTextBlock->bUsesUnicode && odd(pTextBlock->ulLength));
 
 	NO_DBG_MSG("bAdd2TextBlockList");
-	NO_DBG_HEX(pTextBlock->lFileOffset);
-	NO_DBG_HEX(pTextBlock->lTextOffset);
-	NO_DBG_HEX(pTextBlock->lLength);
+	NO_DBG_HEX(pTextBlock->ulFileOffset);
+	NO_DBG_HEX(pTextBlock->ulCharPos);
+	NO_DBG_HEX(pTextBlock->ulLength);
 	NO_DBG_DEC(pTextBlock->bUsesUnicode);
 	NO_DBG_DEC(pTextBlock->usPropMod);
 
-	if (pTextBlock->lFileOffset < 0 ||
-	    pTextBlock->lTextOffset < 0 ||
-	    pTextBlock->lLength <= 0 ||
-	    (pTextBlock->bUsesUnicode && odd(pTextBlock->lLength))) {
+	if (pTextBlock->ulFileOffset == FC_INVALID ||
+	    pTextBlock->ulCharPos == CP_INVALID ||
+	    pTextBlock->ulLength == 0 ||
+	    (pTextBlock->bUsesUnicode && odd(pTextBlock->ulLength))) {
 		werr(0, "Software (textblock) error");
 		return FALSE;
 	}
@@ -111,14 +111,14 @@ bAdd2TextBlockList(text_block_type *pTextBlock)
 	 * the same properties modifier
 	 */
 	if (pBlockLast != NULL &&
-	    pBlockLast->tInfo.lFileOffset +
-	     pBlockLast->tInfo.lLength == pTextBlock->lFileOffset &&
-	    pBlockLast->tInfo.lTextOffset +
-	     pBlockLast->tInfo.lLength == pTextBlock->lTextOffset &&
+	    pBlockLast->tInfo.ulFileOffset +
+	     pBlockLast->tInfo.ulLength == pTextBlock->ulFileOffset &&
+	    pBlockLast->tInfo.ulCharPos +
+	     pBlockLast->tInfo.ulLength == pTextBlock->ulCharPos &&
 	    pBlockLast->tInfo.bUsesUnicode == pTextBlock->bUsesUnicode &&
 	    pBlockLast->tInfo.usPropMod == pTextBlock->usPropMod) {
 		/* These are continous blocks */
-		pBlockLast->tInfo.lLength += pTextBlock->lLength;
+		pBlockLast->tInfo.ulLength += pTextBlock->ulLength;
 		return TRUE;
 	}
 	/* Make a new block */
@@ -148,39 +148,45 @@ bAdd2TextBlockList(text_block_type *pTextBlock)
  * length of the blocks are in bytes.
  */
 void
-vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
-	long lEndnoteLen, long lUnused2Len, BOOL bMustExtend)
+vSplitBlockList(ULONG ulTextLen, ULONG ulFootnoteLen, ULONG ulUnused1Len,
+	ULONG ulEndnoteLen, ULONG ulUnused2Len, BOOL bMustExtend)
 {
 	list_mem_type	*apAnchors[5];
 	list_mem_type	*pGarbageAnchor, *pCurr, *pNext;
 	long		lCharsToGo, lBytesTooFar;
 	int		iIndex;
 #if defined(DEBUG)
-	long		lTotal;
+	ULONG		ulTotal;
 #endif /* DEBUG */
 
 	DBG_MSG("vSplitBlockList");
 
+	fail(ulTextLen > (ULONG)LONG_MAX);
+	fail(ulFootnoteLen > (ULONG)LONG_MAX);
+	fail(ulUnused1Len > (ULONG)LONG_MAX);
+	fail(ulEndnoteLen > (ULONG)LONG_MAX);
+	fail(ulUnused2Len > (ULONG)LONG_MAX);
+
 /* Text block list */
 	pCurr = NULL;
-	lCharsToGo = lTextLen;
+	lCharsToGo = (long)ulTextLen;
 	lBytesTooFar = -1;
-	if (lTextLen > 0) {
+	if (ulTextLen != 0) {
 		DBG_MSG("Text block list");
-		DBG_DEC(lTextLen);
+		DBG_DEC(ulTextLen);
 		for (pCurr = pTextAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			NO_DBG_DEC(pCurr->tInfo.lLength);
-			fail(pCurr->tInfo.lLength <= 0);
+			NO_DBG_DEC(pCurr->tInfo.ulLength);
+			fail(pCurr->tInfo.ulLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.lLength));
-				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				fail(odd(pCurr->tInfo.ulLength));
+				lCharsToGo -= pCurr->tInfo.ulLength / 2;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				lCharsToGo -= pCurr->tInfo.lLength;
+				lCharsToGo -= pCurr->tInfo.ulLength;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -lCharsToGo;
 				}
@@ -191,7 +197,7 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		}
 	}
 /* Split the list */
-	if (lTextLen <= 0) {
+	if (ulTextLen == 0) {
 		/* Empty text blocks list */
 		pFootAnchor = pTextAnchor;
 		pTextAnchor = NULL;
@@ -207,20 +213,20 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		DBG_DEC(lBytesTooFar);
 		fail(lBytesTooFar <= 0);
 		pFootAnchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		pFootAnchor->tInfo.lFileOffset =
-				pCurr->tInfo.lFileOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		pFootAnchor->tInfo.ulFileOffset =
+				pCurr->tInfo.ulFileOffset +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pFootAnchor->tInfo.lFileOffset);
-		DBG_HEX(pCurr->tInfo.lTextOffset);
-		pFootAnchor->tInfo.lTextOffset =
-				pCurr->tInfo.lTextOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pFootAnchor->tInfo.ulFileOffset);
+		DBG_HEX(pCurr->tInfo.ulCharPos);
+		pFootAnchor->tInfo.ulCharPos =
+				pCurr->tInfo.ulCharPos +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pFootAnchor->tInfo.lTextOffset);
-		pFootAnchor->tInfo.lLength = lBytesTooFar;
-		pCurr->tInfo.lLength -= lBytesTooFar;
+		DBG_HEX(pFootAnchor->tInfo.ulCharPos);
+		pFootAnchor->tInfo.ulLength = (ULONG)lBytesTooFar;
+		pCurr->tInfo.ulLength -= lBytesTooFar;
 		pFootAnchor->tInfo.bUsesUnicode = pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of footnote blocks */
 		pFootAnchor->pNext = pCurr->pNext;
@@ -228,24 +234,24 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 	}
 /* Footnote block list */
 	pCurr = NULL;
-	lCharsToGo = lFootnoteLen;
+	lCharsToGo = (long)ulFootnoteLen;
 	lBytesTooFar = -1;
-	if (lFootnoteLen > 0) {
+	if (ulFootnoteLen != 0) {
 		DBG_MSG("Footnote block list");
-		DBG_DEC(lFootnoteLen);
+		DBG_DEC(ulFootnoteLen);
 		for (pCurr = pFootAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.lLength);
-			fail(pCurr->tInfo.lLength <= 0);
+			DBG_DEC(pCurr->tInfo.ulLength);
+			fail(pCurr->tInfo.ulLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.lLength));
-				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				fail(odd(pCurr->tInfo.ulLength));
+				lCharsToGo -= pCurr->tInfo.ulLength / 2;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				lCharsToGo -= pCurr->tInfo.lLength;
+				lCharsToGo -= pCurr->tInfo.ulLength;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -lCharsToGo;
 				}
@@ -256,7 +262,7 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		}
 	}
 /* Split the list */
-	if (lFootnoteLen <= 0) {
+	if (ulFootnoteLen == 0) {
 		/* Empty footnote list */
 		pUnused1Anchor = pFootAnchor;
 		pFootAnchor = NULL;
@@ -268,24 +274,24 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		pUnused1Anchor = pCurr->pNext;
 		pCurr->pNext = NULL;
 	} else {
-	  	/* Split the part-footnote block, part-unused1 block */
+		/* Split the part-footnote block, part-unused1 block */
 		DBG_DEC(lBytesTooFar);
 		fail(lBytesTooFar <= 0);
 		pUnused1Anchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		pUnused1Anchor->tInfo.lFileOffset =
-				pCurr->tInfo.lFileOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		pUnused1Anchor->tInfo.ulFileOffset =
+				pCurr->tInfo.ulFileOffset +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pUnused1Anchor->tInfo.lFileOffset);
-		DBG_HEX(pCurr->tInfo.lTextOffset);
-		pUnused1Anchor->tInfo.lTextOffset =
-				pCurr->tInfo.lTextOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pUnused1Anchor->tInfo.ulFileOffset);
+		DBG_HEX(pCurr->tInfo.ulCharPos);
+		pUnused1Anchor->tInfo.ulCharPos =
+				pCurr->tInfo.ulCharPos +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pUnused1Anchor->tInfo.lTextOffset);
-		pUnused1Anchor->tInfo.lLength = lBytesTooFar;
-		pCurr->tInfo.lLength -= lBytesTooFar;
+		DBG_HEX(pUnused1Anchor->tInfo.ulCharPos);
+		pUnused1Anchor->tInfo.ulLength = (ULONG)lBytesTooFar;
+		pCurr->tInfo.ulLength -= lBytesTooFar;
 		pUnused1Anchor->tInfo.bUsesUnicode =
 				pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of unused1 blocks */
@@ -294,24 +300,24 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 	}
 /* Unused1 block list */
 	pCurr = NULL;
-	lCharsToGo = lUnused1Len;
+	lCharsToGo = (long)ulUnused1Len;
 	lBytesTooFar = -1;
-	if (lUnused1Len > 0) {
+	if (ulUnused1Len != 0) {
 		DBG_MSG("Unused1 block list");
-		DBG_DEC(lUnused1Len);
+		DBG_DEC(ulUnused1Len);
 		for (pCurr = pUnused1Anchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.lLength);
-			fail(pCurr->tInfo.lLength <= 0);
+			DBG_DEC(pCurr->tInfo.ulLength);
+			fail(pCurr->tInfo.ulLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.lLength));
-				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				fail(odd(pCurr->tInfo.ulLength));
+				lCharsToGo -= pCurr->tInfo.ulLength / 2;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				lCharsToGo -= pCurr->tInfo.lLength;
+				lCharsToGo -= pCurr->tInfo.ulLength;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -lCharsToGo;
 				}
@@ -322,7 +328,7 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		}
 	}
 /* Split the list */
-	if (lUnused1Len <= 0) {
+	if (ulUnused1Len == 0) {
 		/* Empty unused1 list */
 		pEndAnchor = pUnused1Anchor;
 		pUnused1Anchor = NULL;
@@ -338,20 +344,20 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		DBG_DEC(lBytesTooFar);
 		fail(lBytesTooFar <= 0);
 		pEndAnchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		pEndAnchor->tInfo.lFileOffset =
-				pCurr->tInfo.lFileOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		pEndAnchor->tInfo.ulFileOffset =
+				pCurr->tInfo.ulFileOffset +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pEndAnchor->tInfo.lFileOffset);
-		DBG_HEX(pCurr->tInfo.lTextOffset);
-		pEndAnchor->tInfo.lTextOffset =
-				pCurr->tInfo.lTextOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pEndAnchor->tInfo.ulFileOffset);
+		DBG_HEX(pCurr->tInfo.ulCharPos);
+		pEndAnchor->tInfo.ulCharPos =
+				pCurr->tInfo.ulCharPos +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pEndAnchor->tInfo.lTextOffset);
-		pEndAnchor->tInfo.lLength = lBytesTooFar;
-		pCurr->tInfo.lLength -= lBytesTooFar;
+		DBG_HEX(pEndAnchor->tInfo.ulCharPos);
+		pEndAnchor->tInfo.ulLength = (ULONG)lBytesTooFar;
+		pCurr->tInfo.ulLength -= lBytesTooFar;
 		pEndAnchor->tInfo.bUsesUnicode = pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of endnote blocks */
 		pEndAnchor->pNext = pCurr->pNext;
@@ -359,24 +365,24 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 	}
 /* Endnote block list */
 	pCurr = NULL;
-	lCharsToGo = lEndnoteLen;
+	lCharsToGo = (long)ulEndnoteLen;
 	lBytesTooFar = -1;
-	if (lEndnoteLen > 0) {
+	if (ulEndnoteLen != 0) {
 		DBG_MSG("Endnote block list");
-		DBG_DEC(lEndnoteLen);
+		DBG_DEC(ulEndnoteLen);
 		for (pCurr = pEndAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.lLength);
-			fail(pCurr->tInfo.lLength <= 0);
+			DBG_DEC(pCurr->tInfo.ulLength);
+			fail(pCurr->tInfo.ulLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.lLength));
-				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				fail(odd(pCurr->tInfo.ulLength));
+				lCharsToGo -= pCurr->tInfo.ulLength / 2;
 				if (lCharsToGo <= 0) {
 					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				lCharsToGo -= pCurr->tInfo.lLength;
+				lCharsToGo -= pCurr->tInfo.ulLength;
 				if (lCharsToGo <= 0) {
 					lBytesTooFar = -lCharsToGo;
 				}
@@ -387,7 +393,7 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		}
 	}
 /* Split the list */
-	if (lEndnoteLen <= 0) {
+	if (ulEndnoteLen == 0) {
 		/* Empty endnote list */
 		pUnused2Anchor = pEndAnchor;
 		pEndAnchor = NULL;
@@ -403,20 +409,20 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		DBG_DEC(lBytesTooFar);
 		fail(lBytesTooFar <= 0);
 		pUnused2Anchor = xmalloc(sizeof(list_mem_type));
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		pUnused2Anchor->tInfo.lFileOffset =
-				pCurr->tInfo.lFileOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		pUnused2Anchor->tInfo.ulFileOffset =
+				pCurr->tInfo.ulFileOffset +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pUnused2Anchor->tInfo.lFileOffset);
-		DBG_HEX(pCurr->tInfo.lTextOffset);
-		pUnused2Anchor->tInfo.lTextOffset =
-				pCurr->tInfo.lTextOffset +
-				pCurr->tInfo.lLength -
+		DBG_HEX(pUnused2Anchor->tInfo.ulFileOffset);
+		DBG_HEX(pCurr->tInfo.ulCharPos);
+		pUnused2Anchor->tInfo.ulCharPos =
+				pCurr->tInfo.ulCharPos +
+				pCurr->tInfo.ulLength -
 				lBytesTooFar;
-		DBG_HEX(pUnused2Anchor->tInfo.lTextOffset);
-		pUnused2Anchor->tInfo.lLength = lBytesTooFar;
-		pCurr->tInfo.lLength -= lBytesTooFar;
+		DBG_HEX(pUnused2Anchor->tInfo.ulCharPos);
+		pUnused2Anchor->tInfo.ulLength = (ULONG)lBytesTooFar;
+		pCurr->tInfo.ulLength -= lBytesTooFar;
 		pUnused2Anchor->tInfo.bUsesUnicode =
 				pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of unused2 blocks */
@@ -425,24 +431,24 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 	}
 /* Unused2 block list */
 	pCurr = NULL;
-	lCharsToGo = lUnused2Len;
+	lCharsToGo = (long)ulUnused2Len;
 	lBytesTooFar = -1;
-	if (lUnused2Len > 0) {
+	if (ulUnused2Len != 0) {
 		DBG_MSG("Unused2 block list");
-		DBG_DEC(lUnused2Len);
+		DBG_DEC(ulUnused2Len);
 		for (pCurr = pUnused2Anchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.lLength);
-			fail(pCurr->tInfo.lLength <= 0);
+			DBG_DEC(pCurr->tInfo.ulLength);
+			fail(pCurr->tInfo.ulLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.lLength));
-				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				fail(odd(pCurr->tInfo.ulLength));
+				lCharsToGo -= pCurr->tInfo.ulLength / 2;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				lCharsToGo -= pCurr->tInfo.lLength;
+				lCharsToGo -= pCurr->tInfo.ulLength;
 				if (lCharsToGo < 0) {
 					lBytesTooFar = -lCharsToGo;
 				}
@@ -453,7 +459,7 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		}
 	}
 /* Split the list */
-	if (lUnused2Len <= 0) {
+	if (ulUnused2Len == 0) {
 		/* Empty unused2 list */
 		pGarbageAnchor = pUnused2Anchor;
 		pUnused2Anchor = NULL;
@@ -468,7 +474,7 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		/* Reduce the part-unused2 block */
 		DBG_DEC(lBytesTooFar);
 		fail(lBytesTooFar <= 0);
-		pCurr->tInfo.lLength -= lBytesTooFar;
+		pCurr->tInfo.ulLength -= lBytesTooFar;
 		/* Move the integral number of garbage blocks */
 		pGarbageAnchor = pCurr->pNext;
 		pCurr->pNext = NULL;
@@ -477,8 +483,8 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 	pCurr = pGarbageAnchor;
 	while (pCurr != NULL) {
 		DBG_FIXME();
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		DBG_DEC(pCurr->tInfo.lLength);
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		DBG_DEC(pCurr->tInfo.ulLength);
 		pNext = pCurr->pNext;
 		pCurr = xfree(pCurr);
 		pCurr = pNext;
@@ -486,94 +492,94 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 
 #if defined(DEBUG)
 	/* Check the number of bytes in the block lists */
-	lTotal = 0;
+	ulTotal = 0;
 	for (pCurr = pTextAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		NO_DBG_HEX(pCurr->tInfo.lFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		NO_DBG_DEC(pCurr->tInfo.lLength);
-		fail(pCurr->tInfo.lLength <= 0);
+		NO_DBG_HEX(pCurr->tInfo.ulFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.ulCharPos);
+		NO_DBG_DEC(pCurr->tInfo.ulLength);
+		fail(pCurr->tInfo.ulLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.lLength));
-			lTotal += pCurr->tInfo.lLength / 2;
+			fail(odd(pCurr->tInfo.ulLength));
+			ulTotal += pCurr->tInfo.ulLength / 2;
 		} else {
-			lTotal += pCurr->tInfo.lLength;
+			ulTotal += pCurr->tInfo.ulLength;
 		}
 	}
-	DBG_DEC(lTotal);
-	if (lTotal != lTextLen) {
-		DBG_DEC(lTextLen);
+	DBG_DEC(ulTotal);
+	if (ulTotal != ulTextLen) {
+		DBG_DEC(ulTextLen);
 		werr(1, "Software error (Text)");
 	}
-	lTotal = 0;
+	ulTotal = 0;
 	for (pCurr = pFootAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.lLength);
-		fail(pCurr->tInfo.lLength <= 0);
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.ulCharPos);
+		DBG_DEC(pCurr->tInfo.ulLength);
+		fail(pCurr->tInfo.ulLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.lLength));
-			lTotal += pCurr->tInfo.lLength / 2;
+			fail(odd(pCurr->tInfo.ulLength));
+			ulTotal += pCurr->tInfo.ulLength / 2;
 		} else {
-			lTotal += pCurr->tInfo.lLength;
+			ulTotal += pCurr->tInfo.ulLength;
 		}
 	}
-	DBG_DEC(lTotal);
-	if (lTotal != lFootnoteLen) {
-		DBG_DEC(lFootnoteLen);
+	DBG_DEC(ulTotal);
+	if (ulTotal != ulFootnoteLen) {
+		DBG_DEC(ulFootnoteLen);
 		werr(1, "Software error (Footnotes)");
 	}
-	lTotal = 0;
+	ulTotal = 0;
 	for (pCurr = pUnused1Anchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.lLength);
-		fail(pCurr->tInfo.lLength <= 0);
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.ulCharPos);
+		DBG_DEC(pCurr->tInfo.ulLength);
+		fail(pCurr->tInfo.ulLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.lLength));
-			lTotal += pCurr->tInfo.lLength / 2;
+			fail(odd(pCurr->tInfo.ulLength));
+			ulTotal += pCurr->tInfo.ulLength / 2;
 		} else {
-			lTotal += pCurr->tInfo.lLength;
+			ulTotal += pCurr->tInfo.ulLength;
 		}
 	}
-	DBG_DEC(lTotal);
-	if (lTotal != lUnused1Len) {
-		DBG_DEC(lUnused1Len);
+	DBG_DEC(ulTotal);
+	if (ulTotal != ulUnused1Len) {
+		DBG_DEC(ulUnused1Len);
 		werr(1, "Software error (Unused1-list)");
 	}
-	lTotal = 0;
+	ulTotal = 0;
 	for (pCurr = pEndAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.lLength);
-		fail(pCurr->tInfo.lLength <= 0);
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.ulCharPos);
+		DBG_DEC(pCurr->tInfo.ulLength);
+		fail(pCurr->tInfo.ulLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.lLength));
-			lTotal += pCurr->tInfo.lLength / 2;
+			fail(odd(pCurr->tInfo.ulLength));
+			ulTotal += pCurr->tInfo.ulLength / 2;
 		} else {
-			lTotal += pCurr->tInfo.lLength;
+			ulTotal += pCurr->tInfo.ulLength;
 		}
 	}
-	DBG_DEC(lTotal);
-	if (lTotal != lEndnoteLen) {
-		DBG_DEC(lEndnoteLen);
+	DBG_DEC(ulTotal);
+	if (ulTotal != ulEndnoteLen) {
+		DBG_DEC(ulEndnoteLen);
 		werr(1, "Software error (Endnotes)");
 	}
-	lTotal = 0;
+	ulTotal = 0;
 	for (pCurr = pUnused2Anchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		DBG_HEX(pCurr->tInfo.lFileOffset);
-		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.lLength);
-		fail(pCurr->tInfo.lLength <= 0);
+		DBG_HEX(pCurr->tInfo.ulFileOffset);
+		NO_DBG_HEX(pCurr->tInfo.ulCharPos);
+		DBG_DEC(pCurr->tInfo.ulLength);
+		fail(pCurr->tInfo.ulLength == 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.lLength));
-			lTotal += pCurr->tInfo.lLength / 2;
+			fail(odd(pCurr->tInfo.ulLength));
+			ulTotal += pCurr->tInfo.ulLength / 2;
 		} else {
-			lTotal += pCurr->tInfo.lLength;
+			ulTotal += pCurr->tInfo.ulLength;
 		}
 	}
-	DBG_DEC(lTotal);
-	if (lTotal != lUnused2Len) {
-		DBG_DEC(lUnused2Len);
+	DBG_DEC(ulTotal);
+	if (ulTotal != ulUnused2Len) {
+		DBG_DEC(ulUnused2Len);
 		werr(1, "Software error (Unused2-list)");
 	}
 #endif /* DEBUG */
@@ -597,14 +603,14 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
 			if (pCurr->pNext != NULL &&
-			    pCurr->tInfo.lLength % BIG_BLOCK_SIZE != 0) {
-				DBG_HEX(pCurr->tInfo.lFileOffset);
-				DBG_HEX(pCurr->tInfo.lTextOffset);
-				DBG_DEC(pCurr->tInfo.lLength);
-				pCurr->tInfo.lLength /= BIG_BLOCK_SIZE;
-				pCurr->tInfo.lLength++;
-				pCurr->tInfo.lLength *= BIG_BLOCK_SIZE;
-				DBG_DEC(pCurr->tInfo.lLength);
+			    pCurr->tInfo.ulLength % BIG_BLOCK_SIZE != 0) {
+				DBG_HEX(pCurr->tInfo.ulFileOffset);
+				DBG_HEX(pCurr->tInfo.ulCharPos);
+				DBG_DEC(pCurr->tInfo.ulLength);
+				pCurr->tInfo.ulLength /= BIG_BLOCK_SIZE;
+				pCurr->tInfo.ulLength++;
+				pCurr->tInfo.ulLength *= BIG_BLOCK_SIZE;
+				DBG_DEC(pCurr->tInfo.ulLength);
 			}
 		}
 	}
@@ -616,236 +622,236 @@ vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
  *
  * returns: The total number of characters
  */
-long
-lGetDocumentLength(void)
+ULONG
+ulGetDocumentLength(void)
 {
 	list_mem_type	*apAnchors[3];
 	list_mem_type	*pCurr;
-	long		lTotal;
+	long		ulTotal;
 	int		iIndex;
 
-	DBG_MSG("uiGetDocumentLength");
+	DBG_MSG("ulGetDocumentLength");
 
 	apAnchors[0] = pTextAnchor;
 	apAnchors[1] = pFootAnchor;
 	apAnchors[2] = pEndAnchor;
 
-	lTotal = 0;
+	ulTotal = 0;
 	for (iIndex = 0; iIndex < 3; iIndex++) {
 		for (pCurr = apAnchors[iIndex];
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			fail(pCurr->tInfo.lLength <= 0);
+			fail(pCurr->tInfo.ulLength == 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.lLength));
-				lTotal += pCurr->tInfo.lLength / 2;
+				fail(odd(pCurr->tInfo.ulLength));
+				ulTotal += pCurr->tInfo.ulLength / 2;
 			} else {
-				lTotal += pCurr->tInfo.lLength;
+				ulTotal += pCurr->tInfo.ulLength;
 			}
 		}
 	}
-	DBG_DEC(lTotal);
-	return lTotal;
-} /* end of tGetDocumentLength */
+	DBG_DEC(ulTotal);
+	return ulTotal;
+} /* end of ulGetDocumentLength */
 #endif /* __riscos */
 
 /*
  * usNextTextByte - get the next byte from the text block list
  */
-static unsigned short
+static USHORT
 usNextTextByte(FILE *pFile,
-	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
+	ULONG *pulFileOffset, ULONG *pulCharPos, USHORT *pusPropMod)
 {
-	static long	lBlockOffset = 0;
+	static ULONG	ulBlockOffset = 0;
 	static size_t	tByteNext = 0;
-	long	lReadOff;
+	ULONG	ulReadOff;
 	size_t	tReadLen;
 
 	if (pTextBlockCurrent == NULL ||
 	    tByteNext >= sizeof(aucBlock) ||
-	    lBlockOffset + (long)tByteNext >= pTextBlockCurrent->tInfo.lLength) {
+	    ulBlockOffset + tByteNext >= pTextBlockCurrent->tInfo.ulLength) {
 		if (pTextBlockCurrent == NULL) {
 			/* First block, first part */
 			pTextBlockCurrent = pTextAnchor;
-			lBlockOffset = 0;
-		} else if (lBlockOffset + (long)sizeof(aucBlock) <
-				pTextBlockCurrent->tInfo.lLength) {
+			ulBlockOffset = 0;
+		} else if (ulBlockOffset + sizeof(aucBlock) <
+				pTextBlockCurrent->tInfo.ulLength) {
 			/* Same block, next part */
-			lBlockOffset += (long)sizeof(aucBlock);
+			ulBlockOffset += sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pTextBlockCurrent = pTextBlockCurrent->pNext;
-			lBlockOffset = 0;
+			ulBlockOffset = 0;
 		}
 		if (pTextBlockCurrent == NULL) {
 			/* Past the last part of the last block */
-			return (unsigned short)EOF;
+			return (USHORT)EOF;
 		}
 		tReadLen = (size_t)
-			(pTextBlockCurrent->tInfo.lLength - lBlockOffset);
+			(pTextBlockCurrent->tInfo.ulLength - ulBlockOffset);
 		if (tReadLen > sizeof(aucBlock)) {
 			tReadLen = sizeof(aucBlock);
 		}
-		lReadOff = pTextBlockCurrent->tInfo.lFileOffset +
-				lBlockOffset;
-		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
-			return (unsigned short)EOF;
+		ulReadOff = pTextBlockCurrent->tInfo.ulFileOffset +
+				ulBlockOffset;
+		if (!bReadBytes(aucBlock, tReadLen, ulReadOff, pFile)) {
+			return (USHORT)EOF;
 		}
 		tByteNext = 0;
 	}
-	if (plFileOffset != NULL) {
-		*plFileOffset = pTextBlockCurrent->tInfo.lFileOffset +
-			lBlockOffset + (long)tByteNext;
+	if (pulFileOffset != NULL) {
+		*pulFileOffset = pTextBlockCurrent->tInfo.ulFileOffset +
+			ulBlockOffset + tByteNext;
 	}
-	if (plTextOffset != NULL) {
-		*plTextOffset = pTextBlockCurrent->tInfo.lTextOffset +
-			lBlockOffset + (long)tByteNext;
+	if (pulCharPos != NULL) {
+		*pulCharPos = pTextBlockCurrent->tInfo.ulCharPos +
+			ulBlockOffset + tByteNext;
 	}
 	if (pusPropMod != NULL) {
 		*pusPropMod = pTextBlockCurrent->tInfo.usPropMod;
 	}
-	return aucBlock[tByteNext++];
+	return (USHORT)aucBlock[tByteNext++];
 } /* end of usNextTextByte */
 
 /*
  * usNextFootByte - get the next byte from the footnote block list
  */
-static unsigned short
+static USHORT
 usNextFootByte(FILE *pFile,
-	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
+	ULONG *pulFileOffset, ULONG *pulCharPos, USHORT *pusPropMod)
 {
-	static long	lBlockOffset = 0;
+	static ULONG	ulBlockOffset = 0;
 	static size_t	tByteNext = 0;
-	long	lReadOff;
+	ULONG	ulReadOff;
 	size_t	tReadLen;
 
 	if (pFootBlockCurrent == NULL ||
 	    tByteNext >= sizeof(aucBlock) ||
-	    lBlockOffset + (long)tByteNext >= pFootBlockCurrent->tInfo.lLength) {
+	    ulBlockOffset + tByteNext >= pFootBlockCurrent->tInfo.ulLength) {
 		if (pFootBlockCurrent == NULL) {
 			/* First block, first part */
 			pFootBlockCurrent = pFootAnchor;
-			lBlockOffset = 0;
-		} else if (lBlockOffset + (long)sizeof(aucBlock) <
-				pFootBlockCurrent->tInfo.lLength) {
+			ulBlockOffset = 0;
+		} else if (ulBlockOffset + sizeof(aucBlock) <
+				pFootBlockCurrent->tInfo.ulLength) {
 			/* Same block, next part */
-			lBlockOffset += (long)sizeof(aucBlock);
+			ulBlockOffset += sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pFootBlockCurrent = pFootBlockCurrent->pNext;
-			lBlockOffset = 0;
+			ulBlockOffset = 0;
 		}
 		if (pFootBlockCurrent == NULL) {
 			/* Past the last part of the last block */
-			return (unsigned short)EOF;
+			return (USHORT)EOF;
 		}
 		tReadLen = (size_t)
-			(pFootBlockCurrent->tInfo.lLength - lBlockOffset);
+			(pFootBlockCurrent->tInfo.ulLength - ulBlockOffset);
 		if (tReadLen > sizeof(aucBlock)) {
 			tReadLen = sizeof(aucBlock);
 		}
-		lReadOff = pFootBlockCurrent->tInfo.lFileOffset +
-				lBlockOffset;
-		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
-			return (unsigned short)EOF;
+		ulReadOff = pFootBlockCurrent->tInfo.ulFileOffset +
+				ulBlockOffset;
+		if (!bReadBytes(aucBlock, tReadLen, ulReadOff, pFile)) {
+			return (USHORT)EOF;
 		}
 		tByteNext = 0;
 	}
-	if (plFileOffset != NULL) {
-		*plFileOffset = pFootBlockCurrent->tInfo.lFileOffset +
-			lBlockOffset + (long)tByteNext;
+	if (pulFileOffset != NULL) {
+		*pulFileOffset = pFootBlockCurrent->tInfo.ulFileOffset +
+			ulBlockOffset + tByteNext;
 	}
-	if (plTextOffset != NULL) {
-		*plTextOffset = pFootBlockCurrent->tInfo.lTextOffset +
-			lBlockOffset + (long)tByteNext;
+	if (pulCharPos != NULL) {
+		*pulCharPos = pFootBlockCurrent->tInfo.ulCharPos +
+			ulBlockOffset + tByteNext;
 	}
 	if (pusPropMod != NULL) {
 		*pusPropMod = pFootBlockCurrent->tInfo.usPropMod;
 	}
-	return aucBlock[tByteNext++];
+	return (USHORT)aucBlock[tByteNext++];
 } /* end of usNextFootByte */
 
 /*
  * usNextEndByte - get the next byte from the endnote block list
  */
-static unsigned short
+static USHORT
 usNextEndByte(FILE *pFile,
-	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
+	ULONG *pulFileOffset, ULONG *pulCharPos, USHORT *pusPropMod)
 {
-	static long	lBlockOffset = 0;
+	static ULONG	ulBlockOffset = 0;
 	static size_t	tByteNext = 0;
-	long	lReadOff;
+	ULONG	ulReadOff;
 	size_t	tReadLen;
 
 	if (pEndBlockCurrent == NULL ||
 	    tByteNext >= sizeof(aucBlock) ||
-	    lBlockOffset + (long)tByteNext >= pEndBlockCurrent->tInfo.lLength) {
+	    ulBlockOffset + tByteNext >= pEndBlockCurrent->tInfo.ulLength) {
 		if (pEndBlockCurrent == NULL) {
 			/* First block, first part */
 			pEndBlockCurrent = pEndAnchor;
-			lBlockOffset = 0;
-		} else if (lBlockOffset + (long)sizeof(aucBlock) <
-				pEndBlockCurrent->tInfo.lLength) {
+			ulBlockOffset = 0;
+		} else if (ulBlockOffset + sizeof(aucBlock) <
+				pEndBlockCurrent->tInfo.ulLength) {
 			/* Same block, next part */
-			lBlockOffset += sizeof(aucBlock);
+			ulBlockOffset += sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pEndBlockCurrent = pEndBlockCurrent->pNext;
-			lBlockOffset = 0;
+			ulBlockOffset = 0;
 		}
 		if (pEndBlockCurrent == NULL) {
 			/* Past the last part of the last block */
-			return (unsigned short)EOF;
+			return (USHORT)EOF;
 		}
 		tReadLen = (size_t)
-			(pEndBlockCurrent->tInfo.lLength - lBlockOffset);
+			(pEndBlockCurrent->tInfo.ulLength - ulBlockOffset);
 		if (tReadLen > sizeof(aucBlock)) {
 			tReadLen = sizeof(aucBlock);
 		}
-		lReadOff = pEndBlockCurrent->tInfo.lFileOffset +
-				lBlockOffset;
-		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
-			return (unsigned short)EOF;
+		ulReadOff = pEndBlockCurrent->tInfo.ulFileOffset +
+				ulBlockOffset;
+		if (!bReadBytes(aucBlock, tReadLen, ulReadOff, pFile)) {
+			return (USHORT)EOF;
 		}
 		tByteNext = 0;
 	}
-	if (plFileOffset != NULL) {
-		*plFileOffset = pEndBlockCurrent->tInfo.lFileOffset +
-			lBlockOffset + (long)tByteNext;
+	if (pulFileOffset != NULL) {
+		*pulFileOffset = pEndBlockCurrent->tInfo.ulFileOffset +
+			ulBlockOffset + tByteNext;
 	}
-	if (plTextOffset != NULL) {
-		*plTextOffset = pEndBlockCurrent->tInfo.lTextOffset +
-			lBlockOffset + (long)tByteNext;
+	if (pulCharPos != NULL) {
+		*pulCharPos = pEndBlockCurrent->tInfo.ulCharPos +
+			ulBlockOffset + tByteNext;
 	}
 	if (pusPropMod != NULL) {
 		*pusPropMod = pEndBlockCurrent->tInfo.usPropMod;
 	}
-	return aucBlock[tByteNext++];
+	return (USHORT)aucBlock[tByteNext++];
 } /* end of usNextEndByte */
 
 /*
  * usNextTextChar - get the next character from the text block list
  */
-static unsigned short
+static USHORT
 usNextTextChar(FILE *pFile,
-	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
+	ULONG *pulFileOffset, ULONG *pulCharPos, USHORT *pusPropMod)
 {
-	unsigned short	usLSB, usMSB;
+	USHORT	usLSB, usMSB;
 
-	usLSB = usNextTextByte(pFile, plFileOffset, plTextOffset, pusPropMod);
-	if (usLSB == (unsigned short)EOF) {
-		return (unsigned short)EOF;
+	usLSB = usNextTextByte(pFile, pulFileOffset, pulCharPos, pusPropMod);
+	if (usLSB == (USHORT)EOF) {
+		return (USHORT)EOF;
 	}
 	if (pTextBlockCurrent->tInfo.bUsesUnicode) {
 		usMSB = usNextTextByte(pFile, NULL, NULL, NULL);
 	} else {
 		usMSB = 0x00;
 	}
-	if (usMSB == (unsigned short)EOF) {
+	if (usMSB == (USHORT)EOF) {
 		DBG_MSG("usNextTextChar: Unexpected EOF");
-		DBG_HEX_C(plFileOffset != NULL, *plFileOffset);
-		DBG_HEX_C(plTextOffset != NULL, *plTextOffset);
-		return (unsigned short)EOF;
+		DBG_HEX_C(pulFileOffset != NULL, *pulFileOffset);
+		DBG_HEX_C(pulCharPos != NULL, *pulCharPos);
+		return (USHORT)EOF;
 	}
 	return (usMSB << 8) | usLSB;
 } /* end of usNextTextChar */
@@ -853,26 +859,26 @@ usNextTextChar(FILE *pFile,
 /*
  * usNextFootChar - get the next character from the footnote block list
  */
-static unsigned short
+static USHORT
 usNextFootChar(FILE *pFile,
-	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
+	ULONG *pulFileOffset, ULONG *pulCharPos, USHORT *pusPropMod)
 {
-	unsigned short	usLSB, usMSB;
+	USHORT	usLSB, usMSB;
 
-	usLSB = usNextFootByte(pFile, plFileOffset, plTextOffset, pusPropMod);
-	if (usLSB == (unsigned short)EOF) {
-		return (unsigned short)EOF;
+	usLSB = usNextFootByte(pFile, pulFileOffset, pulCharPos, pusPropMod);
+	if (usLSB == (USHORT)EOF) {
+		return (USHORT)EOF;
 	}
 	if (pFootBlockCurrent->tInfo.bUsesUnicode) {
 		usMSB = usNextFootByte(pFile, NULL, NULL, NULL);
 	} else {
 		usMSB = 0x00;
 	}
-	if (usMSB == (unsigned short)EOF) {
+	if (usMSB == (USHORT)EOF) {
 		DBG_MSG("usNextFootChar: Unexpected EOF");
-		DBG_HEX_C(plFileOffset != NULL, *plFileOffset);
-		DBG_HEX_C(plTextOffset != NULL, *plTextOffset);
-		return (unsigned short)EOF;
+		DBG_HEX_C(pulFileOffset != NULL, *pulFileOffset);
+		DBG_HEX_C(pulCharPos != NULL, *pulCharPos);
+		return (USHORT)EOF;
 	}
 	return (usMSB << 8) | usLSB;
 } /* end of usNextFootChar */
@@ -880,26 +886,26 @@ usNextFootChar(FILE *pFile,
 /*
  * usNextEndChar - get the next character from the endnote block list
  */
-static unsigned short
+static USHORT
 usNextEndChar(FILE *pFile,
-	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
+	ULONG *pulFileOffset, ULONG *pulCharPos, USHORT *pusPropMod)
 {
-	unsigned short	usLSB, usMSB;
+	USHORT	usLSB, usMSB;
 
-	usLSB = usNextEndByte(pFile, plFileOffset, plTextOffset, pusPropMod);
-	if (usLSB == (unsigned short)EOF) {
-		return (unsigned short)EOF;
+	usLSB = usNextEndByte(pFile, pulFileOffset, pulCharPos, pusPropMod);
+	if (usLSB == (USHORT)EOF) {
+		return (USHORT)EOF;
 	}
 	if (pEndBlockCurrent->tInfo.bUsesUnicode) {
 		usMSB = usNextEndByte(pFile, NULL, NULL, NULL);
 	} else {
 		usMSB = 0x00;
 	}
-	if (usMSB == (unsigned short)EOF) {
+	if (usMSB == (USHORT)EOF) {
 		DBG_MSG("usNextEndChar: Unexpected EOF");
-		DBG_HEX_C(plFileOffset != NULL, *plFileOffset);
-		DBG_HEX_C(plTextOffset != NULL, *plTextOffset);
-		return (unsigned short)EOF;
+		DBG_HEX_C(pulFileOffset != NULL, *pulFileOffset);
+		DBG_HEX_C(pulCharPos != NULL, *pulCharPos);
+		return (USHORT)EOF;
 	}
 	return (usMSB << 8) | usLSB;
 } /* end of usNextEndChar */
@@ -907,141 +913,114 @@ usNextEndChar(FILE *pFile,
 /*
  * usNextChar - get the next character from the given block list
  */
-unsigned short
+USHORT
 usNextChar(FILE *pFile, list_id_enum eListID,
-	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
+	ULONG *pulFileOffset, ULONG *pulCharPos, USHORT *pusPropMod)
 {
 	fail(pFile == NULL);
 
 	switch (eListID) {
 	case text_list:
 		return usNextTextChar(pFile,
-				plFileOffset, plTextOffset, pusPropMod);
+				pulFileOffset, pulCharPos, pusPropMod);
 	case footnote_list:
 		return usNextFootChar(pFile,
-				plFileOffset, plTextOffset, pusPropMod);
+				pulFileOffset, pulCharPos, pusPropMod);
 	case endnote_list:
 		return usNextEndChar(pFile,
-				plFileOffset, plTextOffset, pusPropMod);
+				pulFileOffset, pulCharPos, pusPropMod);
 	default:
-		if (plFileOffset != NULL) {
-			*plFileOffset = -1;
+		if (pulFileOffset != NULL) {
+			*pulFileOffset = FC_INVALID;
 		}
-		if (plTextOffset != NULL) {
-			*plTextOffset = -1;
+		if (pulCharPos != NULL) {
+			*pulCharPos = CP_INVALID;
 		}
 		if (pusPropMod != NULL) {
 			*pusPropMod = IGNORE_PROPMOD;
 		}
-		return (unsigned short)EOF;
+		return (USHORT)EOF;
 	}
 } /* end of usNextChar */
 
-#if 0
 /*
- * Translate the start from the begin of the text to the block that contains
- * this offset.
- * Logical offset to block pointer.
- *
- * Returns: block pointer or NULL
- */
-static text_block_type *
-pTextOffset2Block(long lTextOffset)
-{
-	list_mem_type	*apAnchors[3];
-	list_mem_type	*pCurr;
-	int		iIndex;
-
-	apAnchors[0] = pTextAnchor;
-	apAnchors[1] = pFootAnchor;
-	apAnchors[2] = pEndAnchor;
-
-	for (iIndex = 0; iIndex < 3; iIndex++) {
-		for (pCurr = apAnchors[iIndex];
-		     pCurr != NULL;
-		     pCurr = pCurr->pNext) {
-			if (lTextOffset >= pCurr->tInfo.lTextOffset &&
-			    lTextOffset < pCurr->tInfo.lTextOffset +
-			     pCurr->tInfo.lLength) {
-				/* The textoffset is in the current block */
-				return &pCurr->tInfo;
-			}
-		}
-	}
-	/* Passed beyond the end of the last list */
-	return NULL;
-} /* end of pTextOffset2Block */
-#endif
-
-/*
- * Translate the start from the begin of the text to an offset in the file.
+ * Translate a character position to an offset in the file.
  * Logical to physical offset.
  *
- * Returns:	 -1: in case of error
- *		>=0: the computed file offset
+ * Returns:	FC_INVALID: in case of error
+ *		otherwise: the computed file offset
  */
-long
-lTextOffset2FileOffset(long lTextOffset)
+ULONG
+ulCharPos2FileOffset(ULONG ulCharPos)
 {
 	list_mem_type	*apAnchors[3];
 	list_mem_type	*pCurr;
-	long		lBestGuess;
+	ULONG		ulBestGuess;
 	int		iIndex;
 
 	apAnchors[0] = pTextAnchor;
 	apAnchors[1] = pFootAnchor;
 	apAnchors[2] = pEndAnchor;
 
-	lBestGuess = -1;	/* Best guess is "fileoffset not found" */
+	ulBestGuess = FC_INVALID; /* Best guess is "fileoffset not found" */
 
 	for (iIndex = 0; iIndex < 3; iIndex++) {
 		for (pCurr = apAnchors[iIndex];
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			if (lTextOffset == pCurr->tInfo.lTextOffset +
-			     pCurr->tInfo.lLength &&
+			if (ulCharPos == pCurr->tInfo.ulCharPos +
+			     pCurr->tInfo.ulLength &&
 			    pCurr->pNext != NULL) {
 				/*
 				 * The textoffset is one beyond this block,
 				 * so we guess it's the first byte of the next
 				 * block (if there is a next block)
 				 */
-				lBestGuess = pCurr->pNext->tInfo.lFileOffset;
+				ulBestGuess = pCurr->pNext->tInfo.ulFileOffset;
 			}
 
-			if (lTextOffset < pCurr->tInfo.lTextOffset ||
-			    lTextOffset >= pCurr->tInfo.lTextOffset +
-			     pCurr->tInfo.lLength) {
+			if (ulCharPos < pCurr->tInfo.ulCharPos ||
+			    ulCharPos >= pCurr->tInfo.ulCharPos +
+			     pCurr->tInfo.ulLength) {
 				/* The textoffset is not in this block */
 				continue;
 			}
 
 			/* The textoffset is in the current block */
-			return pCurr->tInfo.lFileOffset +
-				lTextOffset - pCurr->tInfo.lTextOffset;
+			return pCurr->tInfo.ulFileOffset +
+				ulCharPos - pCurr->tInfo.ulCharPos;
 		}
 	}
 	/* Passed beyond the end of the last list */
-	NO_DBG_HEX(lTextOffset);
-	NO_DBG_HEX(lBestGuess);
-	return lBestGuess;
-} /* end of lTextOffset2FileOffset */
+	NO_DBG_HEX(ulCharPos);
+	NO_DBG_HEX(ulBestGuess);
+	return ulBestGuess;
+} /* end of ulCharPos2FileOffset */
 
-#if 0
 /*
- * Get the properties modifier for the given logical offset
+ * Get the sequence number beloning to the given file offset
  *
- * Returns: the properties modifier or IGNORE_PROPMOD
+ * Returns the sequence number
  */
-unsigned short
-pTextOffset2PropMod(long lTextOffset)
+ULONG
+ulGetSeqNumber(ULONG ulFileOffset)
 {
-	text_block_type	*pTextBlock;
+	list_mem_type	*pCurr;
+	ULONG		ulSeq;
 
-	pTextBlock = pTextOffset2Block(lTextOffset);
-	if (pTextBlock == NULL) {
-		return IGNORE_PROPMOD;
+	if (ulFileOffset == FC_INVALID) {
+		return FC_INVALID;
 	}
-	return pTextBlock->usPropMod;
-} /* end of pTextOffset2PropMod */
-#endif
+
+	ulSeq = 0;
+	for (pCurr = pTextAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
+		if (ulFileOffset >= pCurr->tInfo.ulFileOffset &&
+		    ulFileOffset < pCurr->tInfo.ulFileOffset +
+		     pCurr->tInfo.ulLength) {
+			/* The file offset is within the current textblock */
+			return ulSeq + ulFileOffset - pCurr->tInfo.ulFileOffset;
+		}
+		ulSeq += pCurr->tInfo.ulLength;
+	}
+	return FC_INVALID;
+} /* end of ulGetSeqNumber */
