@@ -445,14 +445,14 @@ vDecodeDIB(FILE *pInFile, FILE *pOutFile, const imagedata_type *pImg)
  * vCopy2File
  */
 static void
-vCopy2File(FILE *pInFile, long lFileOffset, int iPictureLen)
+vCopy2File(FILE *pInFile, ULONG ulFileOffset, int iPictureLen)
 {
 	static int	iPicCounter = 0;
 	FILE	*pOutFile;
 	int	iIndex, iTmp;
 	char	szFilename[30];
 
-	if (!bSetDataOffset(pInFile, lFileOffset)) {
+	if (!bSetDataOffset(pInFile, ulFileOffset)) {
 		return;
 	}
 
@@ -488,14 +488,14 @@ vCopy2File(FILE *pInFile, long lFileOffset, int iPictureLen)
  */
 BOOL
 bTranslateDIB(diagram_type *pDiag, FILE *pInFile,
-		long lFileOffset, const imagedata_type *pImg)
+		ULONG ulFileOffset, const imagedata_type *pImg)
 {
 #if defined(DEBUG)
-	vCopy2File(pInFile, lFileOffset, pImg->iLength - pImg->iPosition);
+	vCopy2File(pInFile, ulFileOffset, pImg->iLength - pImg->iPosition);
 #endif /* DEBUG */
 
 	/* Seek to start position of DIB data */
-	if (!bSetDataOffset(pInFile, lFileOffset)) {
+	if (!bSetDataOffset(pInFile, ulFileOffset)) {
 		return FALSE;
 	}
 
