@@ -1,6 +1,6 @@
 /*
  * fonts.c
- * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * Functions to deal with fonts (generic)
@@ -10,6 +10,22 @@
 #include <string.h>
 #include "antiword.h"
 
+/* Maximum line lenth in the font file */
+#define FONT_LINE_LENGTH	81
+
+/* Pitch */
+#define PITCH_UNKNOWN		0
+#define PITCH_FIXED		1
+#define PITCH_VARIABLE		2
+
+/* Font Family */
+#define FAMILY_UNKNOWN		0
+#define FAMILY_ROMAN		1
+#define FAMILY_SWISS		2
+#define FAMILY_MODERN		3
+#define FAMILY_SCRIPT		4
+#define FAMILY_DECORATIVE	5
+
 /* Font Translation Table */
 static size_t		tFontTableRecords = 0;
 static font_table_type	*pFontTable = NULL;
@@ -20,23 +36,24 @@ static font_table_type	*pFontTable = NULL;
  * returns the index into the FontTable, -1 if not found
  */
 int
-iGetFontByNumber(int iWordFontnumber, unsigned char ucFontstyle)
+iGetFontByNumber(UCHAR ucWordFontnumber, UCHAR ucFontstyle)
 {
 	int	iIndex;
 
 	for (iIndex = 0; iIndex < (int)tFontTableRecords; iIndex++) {
-		if (iWordFontnumber ==
-				(int)pFontTable[iIndex].ucWordFontnumber &&
+		if (ucWordFontnumber == pFontTable[iIndex].ucWordFontnumber &&
 		    ucFontstyle == pFontTable[iIndex].ucFontstyle &&
 		    pFontTable[iIndex].szOurFontname[0] != '\0') {
 			return iIndex;
 		}
 	}
+	DBG_DEC(ucWordFontnumber);
+	DBG_HEX(ucFontstyle);
 	return -1;
 } /* end of iGetFontByNumber */
 
 /*
- * szGetOurFontName - Get our font name
+ * szGetOurFontname - Get our font name
  *
  * return our font name from the given index, NULL if not found
  */
@@ -55,7 +72,7 @@ szGetOurFontname(int iIndex)
  * returns the Word font number, -1 if not found
  */
 int
-iFontname2Fontnumber(const char *szOurFontname, unsigned char ucFontstyle)
+iFontname2Fontnumber(const char *szOurFontname, UCHAR ucFontstyle)
 {
 	int	iIndex;
 
@@ -71,28 +88,124 @@ iFontname2Fontnumber(const char *szOurFontname, unsigned char ucFontstyle)
 /*
  * See if the fontname from the Word file matches the fontname from the
  * font translation file.
- * If iBytesPerChar is one than szWord is in ISO-8859-x (Word 6/7),
- * if iBytesPerChar is two than szWord is in Unicode (Word 8/97).
+ * If iBytesPerChar is one than szWord is in ISO-8859-x (Word 2/6/7),
+ * if iBytesPerChar is two than szWord is in Unicode (Word 8/9/10).
  */
 static BOOL
-bFontEqual(const char *szWord, const char *szTable, int iBytesPerChar)
+bFontEqual(const UCHAR *szWord, const char *szTable, int iBytesPerChar)
 {
-	const char	*pcTmp1, *pcTmp2;
+	const UCHAR	*pucTmp;
+	const char	*pcTmp;
 
 	fail(szWord == NULL || szTable == NULL);
 	fail(iBytesPerChar != 1 && iBytesPerChar != 2);
 
-	for (pcTmp1 = szWord, pcTmp2 = szTable;
-	     *pcTmp1 != '\0';
-	     pcTmp1 += iBytesPerChar, pcTmp2++) {
-		if (ulToUpper((unsigned long)*pcTmp1) !=
-		    ulToUpper((unsigned long)*pcTmp2)) {
+	for (pucTmp = szWord, pcTmp = szTable;
+	     *pucTmp != 0;
+	     pucTmp += iBytesPerChar, pcTmp++) {
+		if (ulToUpper((ULONG)*pucTmp) !=
+		    ulToUpper((ULONG)*pcTmp)) {
 			return FALSE;
 		}
 	}
-	return *pcTmp2 == '\0';
+	return *pcTmp == '\0';
 } /* end of bFontEqual */
 
+/*
+ *
+ */
+static void
+vFontname2Table(const UCHAR *aucFont, const UCHAR *aucAltFont,
+	int iBytesPerChar, int iEmphasis, UCHAR ucFFN,
+	const char *szWordFont, const char *szOurFont,
+	font_table_type *pFontTableRecord)
+{
+	BOOL	bMatchFound;
+	UCHAR	ucPrq, ucFf;
+
+	fail(aucFont == NULL || aucFont[0] == 0);
+	fail(aucAltFont != NULL && aucAltFont[0] == 0);
+	fail(iBytesPerChar != 1 && iBytesPerChar != 2);
+	fail(iEmphasis < 0 || iEmphasis > 3);
+	fail(szWordFont == NULL || szWordFont[0] == '\0');
+	fail(szOurFont == NULL || szOurFont[0] == '\0');
+	fail(pFontTableRecord == NULL);
+
+	bMatchFound = bFontEqual(aucFont, szWordFont, iBytesPerChar);
+
+	if (!bMatchFound && aucAltFont != NULL) {
+		bMatchFound = bFontEqual(aucAltFont, szWordFont, iBytesPerChar);
+	}
+
+	if (!bMatchFound &&
+	    pFontTableRecord->szWordFontname[0] == '\0' &&
+	    szWordFont[0] == '*' &&
+	    szWordFont[1] == '\0') {
+		/*
+		 * szWordFont contains a "*", so szOurFont will contain the
+		 * "default default" font. See if we can do better than that.
+		 */
+		ucPrq = ucFFN & 0x03;
+		ucFf = (ucFFN & 0x70) >> 4;
+		NO_DBG_DEC(ucPrq);
+		NO_DBG_DEC(ucFf);
+		if (ucPrq == PITCH_FIXED) {
+			/* Set to the default monospaced font */
+			switch (iEmphasis) {
+			case 0: szOurFont = FONT_MONOSPACED_PLAIN; break;
+			case 1: szOurFont = FONT_MONOSPACED_BOLD; break;
+			case 2: szOurFont = FONT_MONOSPACED_ITALIC; break;
+			case 3: szOurFont = FONT_MONOSPACED_BOLDITALIC; break;
+			default: break;
+			}
+		} else if (ucFf == FAMILY_ROMAN) {
+			/* Set to the default serif font */
+			switch (iEmphasis) {
+			case 0: szOurFont = FONT_SERIF_PLAIN; break;
+			case 1: szOurFont = FONT_SERIF_BOLD; break;
+			case 2: szOurFont = FONT_SERIF_ITALIC; break;
+			case 3: szOurFont = FONT_SERIF_BOLDITALIC; break;
+			default: break;
+			}
+		} else if (ucFf == FAMILY_SWISS) {
+			/* Set to the default sans serif font */
+			switch (iEmphasis) {
+			case 0: szOurFont = FONT_SANS_SERIF_PLAIN; break;
+			case 1: szOurFont = FONT_SANS_SERIF_BOLD; break;
+			case 2: szOurFont = FONT_SANS_SERIF_ITALIC; break;
+			case 3: szOurFont = FONT_SANS_SERIF_BOLDITALIC; break;
+			default: break;
+			}
+		}
+		bMatchFound = TRUE;
+	}
+
+	if (bMatchFound) {
+		switch (iBytesPerChar) {
+		case 1:
+			(void)memcpy(pFontTableRecord->szWordFontname, aucFont,
+				sizeof(pFontTableRecord->szWordFontname) - 1);
+			break;
+		case 2:
+			(void)unincpy(pFontTableRecord->szWordFontname, aucFont,
+				sizeof(pFontTableRecord->szWordFontname) - 1);
+			break;
+		default:
+			DBG_FIXME();
+			pFontTableRecord->szWordFontname[0] = '\0';
+			break;
+		}
+		pFontTableRecord->szWordFontname[
+			sizeof(pFontTableRecord->szWordFontname) - 1] = '\0';
+		(void)strncpy(pFontTableRecord->szOurFontname, szOurFont,
+			sizeof(pFontTableRecord->szOurFontname) - 1);
+		pFontTableRecord->szOurFontname[
+			sizeof(pFontTableRecord->szOurFontname) - 1] = '\0';
+		NO_DBG_MSG(pFontTableRecord->szWordFontname);
+		NO_DBG_MSG(pFontTableRecord->szOurFontname);
+	}
+} /* end of vFontname2Table */
+
 /*
  * vCreateFontTable - Create and initialize the internal font table
  */
@@ -104,19 +217,20 @@ vCreateFontTable(void)
 	int	iNbr;
 
 	if (tFontTableRecords == 0) {
-		pFontTable = NULL;
+		pFontTable = xfree(pFontTable);
 		return;
 	}
 
 	/* Create the font table */
 	tSize = tFontTableRecords * sizeof(*pFontTable);
 	pFontTable = xmalloc(tSize);
+
 	/* Initialize the font table */
 	(void)memset(pFontTable, 0, tSize);
 	for (iNbr = 0, pTmp = pFontTable;
 	     pTmp < pFontTable + tFontTableRecords;
 	     iNbr++, pTmp++) {
-		pTmp->ucWordFontnumber = (unsigned char)(iNbr / 4);
+		pTmp->ucWordFontnumber = (UCHAR)(iNbr / 4);
 		switch (iNbr % 4) {
 		case 0:
 			pTmp->ucFontstyle = FONT_REGULAR;
@@ -143,6 +257,8 @@ vCreateFontTable(void)
 static void
 vMinimizeFontTable(void)
 {
+	font_block_type		tFontNext;
+	const style_block_type	*pStyle;
 	const font_block_type	*pFont;
 	font_table_type		*pTmp;
 	int	iUnUsed;
@@ -150,7 +266,8 @@ vMinimizeFontTable(void)
 
 	NO_DBG_MSG("vMinimizeFontTable");
 
-	if (pFontTable == NULL || tFontTableRecords == 0) {
+	if (tFontTableRecords == 0) {
+		pFontTable = xfree(pFontTable);
 		return;
 	}
 
@@ -172,6 +289,8 @@ vMinimizeFontTable(void)
 
 	/* See which fonts/styles are really being used */
 	bMustAddTableFont = TRUE;
+
+	/* The fonts/styles that will be used */
 	pFont = NULL;
 	while((pFont = pGetNextFontInfoListItem(pFont)) != NULL) {
 		pTmp = pFontTable + 4 * (int)pFont->ucFontnumber;
@@ -191,6 +310,28 @@ vMinimizeFontTable(void)
 		pTmp->ucInUse = 1;
 	}
 
+	/* The fonts/styles that might be used */
+	pStyle = NULL;
+	while((pStyle = pGetNextStyleInfoListItem(pStyle)) != NULL) {
+		vFillFontFromStylesheet(pStyle->usIstdNext, &tFontNext);
+		vCorrectFontValues(&tFontNext);
+		pTmp = pFontTable + 4 * (int)tFontNext.ucFontnumber;
+		if (bIsBold(tFontNext.ucFontstyle)) {
+			pTmp++;
+		}
+		if (bIsItalic(tFontNext.ucFontstyle)) {
+			pTmp += 2;
+		}
+		if (pTmp >= pFontTable + tFontTableRecords) {
+			continue;
+		}
+		if (STREQ(pTmp->szOurFontname, TABLE_FONT)) {
+			/* The table font is already present */
+			bMustAddTableFont = FALSE;
+		}
+		pTmp->ucInUse = 1;
+	}
+
 	/* Remove the unused font entries from the font table */
 	iUnUsed = 0;
 	for (pTmp = pFontTable;
@@ -208,7 +349,7 @@ vMinimizeFontTable(void)
 	tFontTableRecords -= (size_t)iUnUsed;
 	if (bMustAddTableFont) {
 		pTmp = pFontTable + tFontTableRecords;
-	  	pTmp->ucWordFontnumber = (pTmp - 1)->ucWordFontnumber + 1;
+		pTmp->ucWordFontnumber = (pTmp - 1)->ucWordFontnumber + 1;
 		pTmp->ucFontstyle = FONT_REGULAR;
 		pTmp->ucInUse = 1;
 		strcpy(pTmp->szWordFontname, "Extra Table Font");
@@ -221,18 +362,6 @@ vMinimizeFontTable(void)
 		pFontTable = xrealloc(pFontTable,
 				tFontTableRecords * sizeof(*pFontTable));
 	}
-#if 0
-	/* Report the used but untranslated fontnames */
-	for (pTmp = pFontTable;
-	     pTmp < pFontTable + tFontTableRecords;
-	     pTmp++) {
-		if (pTmp->szOurFontname[0] == '\0') {
-			DBG_MSG(pTmp->szWordFontname);
-			werr(0, "%s is not in the FontNames file",
-				pTmp->szWordFontname);
-		}
-	}
-#endif /* 0 */
 #if defined(DEBUG)
 	DBG_MSG("After");
 	DBG_DEC(tFontTableRecords);
@@ -258,7 +387,7 @@ bReadFontFile(FILE *pFontTableFile, char *szWordFont,
 {
 	char	*pcTmp;
 	int	iFields;
-	char	szLine[81];
+	char	szLine[FONT_LINE_LENGTH];
 
 	fail(szWordFont == NULL || szOurFont == NULL);
 	fail(piItalic == NULL || piBold == NULL || piSpecial == NULL);
@@ -270,8 +399,8 @@ bReadFontFile(FILE *pFontTableFile, char *szWordFont,
 			continue;
 		}
 		iFields = sscanf(szLine, "%[^,],%d,%d,%1s%[^,],%d",
-		    	szWordFont, piItalic, piBold,
-		    	&szOurFont[0], &szOurFont[1], piSpecial);
+			szWordFont, piItalic, piBold,
+			&szOurFont[0], &szOurFont[1], piSpecial);
 		if (iFields != 6) {
 			pcTmp = strchr(szLine, '\r');
 			if (pcTmp != NULL) {
@@ -286,9 +415,13 @@ bReadFontFile(FILE *pFontTableFile, char *szWordFont,
 			continue;
 		}
 		if (strlen(szWordFont) >=
-		    		sizeof(pFontTable[0].szWordFontname) ||
-		    strlen(szOurFont) >=
-		    		sizeof(pFontTable[0].szOurFontname)) {
+				sizeof(pFontTable[0].szWordFontname)) {
+			werr(0, "Word fontname too long: '%s'", szWordFont);
+			continue;
+		}
+		if (strlen(szOurFont) >=
+				sizeof(pFontTable[0].szOurFontname)) {
+			werr(0, "Local fontname too long: '%s'", szOurFont);
 			continue;
 		}
 		/* The current line passed all the tests */
@@ -298,26 +431,26 @@ bReadFontFile(FILE *pFontTableFile, char *szWordFont,
 } /* end of bReadFontFile */
 
 /*
- * vCreate6FontTable - create a font table from Word 6/7
+ * vCreate2FontTable - create a font table from WinWord 1/2
  */
 void
-vCreate6FontTable(FILE *pFile, long lStartBlock,
-	const long *alBBD, size_t tBBDLen,
-	const unsigned char *aucHeader)
+vCreate2FontTable(FILE *pFile, const UCHAR *aucHeader)
 {
 	FILE	*pFontTableFile;
 	font_table_type	*pTmp;
-	char	*szFont, *szAltFont;
-	unsigned char	*aucBuffer;
-	long	lBeginFontInfo;
+	UCHAR	*aucFont;
+	UCHAR	*aucBuffer;
+	ULONG	ulBeginFontInfo;
 	size_t	tFontInfoLen;
-	int	iPos, iRecLen, iOffsetAltName;
-	int	iBold, iItalic, iSpecial;
-	char	szWordFont[81], szOurFont[81];
+	int	iPos, iRecLen;
+	int	iBold, iItalic, iSpecial, iEmphasis;
+	UCHAR	ucFFN;
+	char	szWordFont[FONT_LINE_LENGTH], szOurFont[FONT_LINE_LENGTH];
 
 	fail(pFile == NULL || aucHeader == NULL);
-	fail(lStartBlock < 0);
-	fail(alBBD == NULL);
+
+	tFontTableRecords = 0;
+	pFontTable = xfree(pFontTable);
 
 	pFontTableFile = pOpenFontTableFile();
 	if (pFontTableFile == NULL) {
@@ -325,16 +458,111 @@ vCreate6FontTable(FILE *pFile, long lStartBlock,
 		return;
 	}
 
-	lBeginFontInfo = (long)ulGetLong(0xd0, aucHeader);
-	DBG_HEX(lBeginFontInfo);
-	tFontInfoLen = (size_t)ulGetLong(0xd4, aucHeader);
+	ulBeginFontInfo = ulGetLong(0xb2, aucHeader); /* fcSttbfffn */
+	DBG_HEX(ulBeginFontInfo);
+	tFontInfoLen = (size_t)usGetWord(0xb6, aucHeader); /* cbSttbfffn */
+	DBG_DEC(tFontInfoLen);
+	fail(tFontInfoLen < 6);
+
+	if (ulBeginFontInfo > (ULONG)LONG_MAX) {
+		/* Don't ask me why this is needed */
+		DBG_HEX(ulBeginFontInfo);
+		(void)fclose(pFontTableFile);
+		return;
+	}
+
+	aucBuffer = xmalloc(tFontInfoLen);
+	if (!bReadBytes(aucBuffer, tFontInfoLen, ulBeginFontInfo, pFile)) {
+		aucBuffer = xfree(aucBuffer);
+		(void)fclose(pFontTableFile);
+		return;
+	}
+	NO_DBG_PRINT_BLOCK(aucBuffer, tFontInfoLen);
+	DBG_DEC(usGetWord(0, aucBuffer));
+
+	/* Compute the maximum number of entries in the font table */
+	tFontTableRecords = 0;
+	iPos = 2;
+	while (iPos + 3 < (int)tFontInfoLen) {
+		iRecLen = (int)ucGetByte(iPos, aucBuffer);
+		NO_DBG_DEC(iRecLen);
+		NO_DBG_MSG(aucBuffer + iPos + 3);
+		iPos += iRecLen + 1;
+		tFontTableRecords++;
+	}
+	tFontTableRecords *= 4;	/* Plain, Bold, Italic and Bold/italic */
+	tFontTableRecords++;	/* One extra for the table-font */
+	vCreateFontTable();
+
+	/* Read the font translation file */
+	while (bReadFontFile(pFontTableFile, szWordFont,
+			&iItalic, &iBold, szOurFont, &iSpecial)) {
+		iEmphasis = 0;
+		if (iBold != 0) {
+			iEmphasis++;
+		}
+		if (iItalic != 0) {
+			iEmphasis += 2;
+		}
+		pTmp = pFontTable + iEmphasis;
+		iPos = 2;
+		while (iPos + 3 < (int)tFontInfoLen) {
+			iRecLen = (int)ucGetByte(iPos, aucBuffer);
+			ucFFN = ucGetByte(iPos + 1, aucBuffer);
+			aucFont = aucBuffer + iPos + 3;
+			vFontname2Table(aucFont, NULL, 1, iEmphasis,
+					ucFFN, szWordFont, szOurFont, pTmp);
+			pTmp += 4;
+			iPos += iRecLen + 1;
+		}
+	}
+	(void)fclose(pFontTableFile);
+	aucBuffer = xfree(aucBuffer);
+	vMinimizeFontTable();
+} /* end of vCreate2FontTable */
+
+/*
+ * vCreate6FontTable - create a font table from Word 6/7
+ */
+void
+vCreate6FontTable(FILE *pFile, ULONG ulStartBlock,
+	const ULONG *aulBBD, size_t tBBDLen,
+	const UCHAR *aucHeader)
+{
+	FILE	*pFontTableFile;
+	font_table_type	*pTmp;
+	UCHAR	*aucFont, *aucAltFont;
+	UCHAR	*aucBuffer;
+	ULONG	ulBeginFontInfo;
+	size_t	tFontInfoLen;
+	int	iPos, iRecLen, iOffsetAltName;
+	int	iBold, iItalic, iSpecial, iEmphasis;
+	UCHAR	ucFFN;
+	char	szWordFont[FONT_LINE_LENGTH], szOurFont[FONT_LINE_LENGTH];
+
+	fail(pFile == NULL || aucHeader == NULL);
+	fail(ulStartBlock > MAX_BLOCKNUMBER && ulStartBlock != END_OF_CHAIN);
+	fail(aulBBD == NULL);
+
+	tFontTableRecords = 0;
+	pFontTable = xfree(pFontTable);
+
+	pFontTableFile = pOpenFontTableFile();
+	if (pFontTableFile == NULL) {
+		/* No translation table file, no translation table */
+		return;
+	}
+
+	ulBeginFontInfo = ulGetLong(0xd0, aucHeader); /* fcSttbfffn */
+	DBG_HEX(ulBeginFontInfo);
+	tFontInfoLen = (size_t)ulGetLong(0xd4, aucHeader); /* lcbSttbfffn */
 	DBG_DEC(tFontInfoLen);
 	fail(tFontInfoLen < 9);
 
 	aucBuffer = xmalloc(tFontInfoLen);
-	if (!bReadBuffer(pFile, lStartBlock,
-			alBBD, tBBDLen, BIG_BLOCK_SIZE,
-			aucBuffer, lBeginFontInfo, tFontInfoLen)) {
+	if (!bReadBuffer(pFile, ulStartBlock,
+			aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+			aucBuffer, ulBeginFontInfo, tFontInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		(void)fclose(pFontTableFile);
 		return;
@@ -354,75 +582,73 @@ vCreate6FontTable(FILE *pFile, long lStartBlock,
 		iPos += iRecLen + 1;
 		tFontTableRecords++;
 	}
-	tFontTableRecords *= 4;	/* Regular, Bold, Italic and Bold/italic */
+	tFontTableRecords *= 4;	/* Plain, Bold, Italic and Bold/italic */
 	tFontTableRecords++;	/* One extra for the table-font */
 	vCreateFontTable();
 
 	/* Read the font translation file */
 	while (bReadFontFile(pFontTableFile, szWordFont,
 			&iItalic, &iBold, szOurFont, &iSpecial)) {
-		pTmp = pFontTable;
+		iEmphasis = 0;
 		if (iBold != 0) {
-			pTmp++;
+			iEmphasis++;
 		}
 		if (iItalic != 0) {
-			pTmp += 2;
+			iEmphasis += 2;
 		}
+		pTmp = pFontTable + iEmphasis;
 		iPos = 2;
 		while (iPos + 6 < (int)tFontInfoLen) {
 			iRecLen = (int)ucGetByte(iPos, aucBuffer);
-			szFont = (char *)aucBuffer + iPos + 6;
+			ucFFN = ucGetByte(iPos + 1, aucBuffer);
+			aucFont = aucBuffer + iPos + 6;
 			iOffsetAltName = (int)ucGetByte(iPos + 5, aucBuffer);
 			if (iOffsetAltName <= 0) {
-				szAltFont = NULL;
+				aucAltFont = NULL;
 			} else {
-				szAltFont = szFont + iOffsetAltName;
-				NO_DBG_MSG(szFont);
-				NO_DBG_MSG(szAltFont);
-			}
-			if (bFontEqual(szFont, szWordFont, 1) ||
-			    (szAltFont != NULL &&
-			     bFontEqual(szAltFont, szWordFont, 1)) ||
-			    (pTmp->szWordFontname[0] == '\0' &&
-			     szWordFont[0] == '*' &&
-			     szWordFont[1] == '\0')) {
-			  	strncpy(pTmp->szWordFontname, szFont,
-			  		sizeof(pTmp->szWordFontname) - 1);
-			  	pTmp->szWordFontname[sizeof(
-			  		pTmp->szWordFontname) - 1] = '\0';
-				strcpy(pTmp->szOurFontname, szOurFont);
+				aucAltFont = aucFont + iOffsetAltName;
+				NO_DBG_MSG(aucFont);
+				NO_DBG_MSG(aucAltFont);
 			}
+			vFontname2Table(aucFont, aucAltFont, 1, iEmphasis,
+					ucFFN, szWordFont, szOurFont, pTmp);
 			pTmp += 4;
 			iPos += iRecLen + 1;
 		}
 	}
-	vMinimizeFontTable();
-	aucBuffer = xfree(aucBuffer);
 	(void)fclose(pFontTableFile);
+	aucBuffer = xfree(aucBuffer);
+	vMinimizeFontTable();
 } /* end of vCreate6FontTable */
 
 /*
- * vCreate8FontTable - create a font table from Word 8/97
+ * vCreate8FontTable - create a font table from Word 8/9/10
  */
 void
 vCreate8FontTable(FILE *pFile, const pps_info_type *pPPS,
-	const long *alBBD, size_t tBBDLen, const long *alSBD, size_t tSBDLen,
-	const unsigned char *aucHeader)
+	const ULONG *aulBBD, size_t tBBDLen,
+	const ULONG *aulSBD, size_t tSBDLen,
+	const UCHAR *aucHeader)
 {
 	FILE	*pFontTableFile;
 	font_table_type	*pTmp;
-	const long	*alBlockDepot;
-	char		*szFont, *szAltFont;
-	unsigned char	*aucBuffer;
-	long	lTableSize, lTableStartBlock, lBeginFontInfo;
+	const ULONG	*aulBlockDepot;
+	UCHAR	*aucFont, *aucAltFont;
+	UCHAR	*aucBuffer;
+	ULONG	ulBeginFontInfo;
+	ULONG	ulTableSize, ulTableStartBlock;
 	size_t	tFontInfoLen, tBlockDepotLen, tBlockSize;
 	int	iPos, iRecLen, iOffsetAltName;
-	int	iBold, iItalic, iSpecial;
-	unsigned short	usDocStatus;
-	char	szWordFont[81], szOurFont[81];
+	int	iBold, iItalic, iSpecial, iEmphasis;
+	USHORT	usDocStatus;
+	UCHAR	ucFFN;
+	char	szWordFont[FONT_LINE_LENGTH], szOurFont[FONT_LINE_LENGTH];
 
 	fail(pFile == NULL || pPPS == NULL || aucHeader == NULL);
-	fail(alBBD == NULL || alSBD == NULL);
+	fail(aulBBD == NULL || aulSBD == NULL);
+
+	tFontTableRecords = 0;
+	pFontTable = xfree(pFontTable);
 
 	pFontTableFile = pOpenFontTableFile();
 	if (pFontTableFile == NULL) {
@@ -430,43 +656,44 @@ vCreate8FontTable(FILE *pFile, const pps_info_type *pPPS,
 		return;
 	}
 
-	lBeginFontInfo = (long)ulGetLong(0x112, aucHeader);
-	DBG_HEX(lBeginFontInfo);
-	tFontInfoLen = (size_t)ulGetLong(0x116, aucHeader);
+	ulBeginFontInfo = ulGetLong(0x112, aucHeader); /* fcSttbfffn */
+	DBG_HEX(ulBeginFontInfo);
+	tFontInfoLen = (size_t)ulGetLong(0x116, aucHeader); /* lcbSttbfffn */
 	DBG_DEC(tFontInfoLen);
 	fail(tFontInfoLen < 46);
 
 	/* Use 0Table or 1Table? */
 	usDocStatus = usGetWord(0x0a, aucHeader);
 	if (usDocStatus & BIT(9)) {
-		lTableStartBlock = pPPS->t1Table.lSb;
-		lTableSize = pPPS->t1Table.lSize;
+		ulTableStartBlock = pPPS->t1Table.ulSB;
+		ulTableSize = pPPS->t1Table.ulSize;
 	} else {
-		lTableStartBlock = pPPS->t0Table.lSb;
-		lTableSize = pPPS->t0Table.lSize;
+		ulTableStartBlock = pPPS->t0Table.ulSB;
+		ulTableSize = pPPS->t0Table.ulSize;
 	}
-	DBG_DEC(lTableStartBlock);
-	if (lTableStartBlock < 0) {
-		DBG_DEC(lTableStartBlock);
+	DBG_DEC(ulTableStartBlock);
+	if (ulTableStartBlock == 0) {
+		DBG_DEC(ulTableStartBlock);
 		DBG_MSG("No fontname table");
+		(void)fclose(pFontTableFile);
 		return;
 	}
-	DBG_HEX(lTableSize);
-	if (lTableSize < MIN_SIZE_FOR_BBD_USE) {
-	  	/* Use the Small Block Depot */
-		alBlockDepot = alSBD;
+	DBG_HEX(ulTableSize);
+	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
+		/* Use the Small Block Depot */
+		aulBlockDepot = aulSBD;
 		tBlockDepotLen = tSBDLen;
 		tBlockSize = SMALL_BLOCK_SIZE;
 	} else {
-	  	/* Use the Big Block Depot */
-		alBlockDepot = alBBD;
+		/* Use the Big Block Depot */
+		aulBlockDepot = aulBBD;
 		tBlockDepotLen = tBBDLen;
 		tBlockSize = BIG_BLOCK_SIZE;
 	}
 	aucBuffer = xmalloc(tFontInfoLen);
-	if (!bReadBuffer(pFile, lTableStartBlock,
-			alBlockDepot, tBlockDepotLen, tBlockSize,
-			aucBuffer, lBeginFontInfo, tFontInfoLen)) {
+	if (!bReadBuffer(pFile, ulTableStartBlock,
+			aulBlockDepot, tBlockDepotLen, tBlockSize,
+			aucBuffer, ulBeginFontInfo, tFontInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		(void)fclose(pFontTableFile);
 		return;
@@ -475,60 +702,55 @@ vCreate8FontTable(FILE *pFile, const pps_info_type *pPPS,
 
 	/* Get the maximum number of entries in the font table */
 	tFontTableRecords = (size_t)usGetWord(0, aucBuffer);
-	tFontTableRecords *= 4;	/* Regular, Bold, Italic and Bold/italic */
+	tFontTableRecords *= 4;	/* Plain, Bold, Italic and Bold/italic */
 	tFontTableRecords++;	/* One extra for the table-font */
 	vCreateFontTable();
 
 	/* Read the font translation file */
 	while (bReadFontFile(pFontTableFile, szWordFont,
 			&iItalic, &iBold, szOurFont, &iSpecial)) {
-		pTmp = pFontTable;
+		iEmphasis = 0;
 		if (iBold != 0) {
-			pTmp++;
+			iEmphasis++;
 		}
 		if (iItalic != 0) {
-			pTmp += 2;
+			iEmphasis += 2;
 		}
+		pTmp = pFontTable + iEmphasis;
 		iPos = 4;
 		while (iPos + 40 < (int)tFontInfoLen) {
 			iRecLen = (int)ucGetByte(iPos, aucBuffer);
-			szFont = (char *)aucBuffer + iPos + 40;
-			iOffsetAltName = (int)unilen(szFont);
+			ucFFN = ucGetByte(iPos + 1, aucBuffer);
+			aucFont = aucBuffer + iPos + 40;
+			iOffsetAltName = (int)unilen(aucFont);
 			if (iPos + 40 + iOffsetAltName + 4 >= iRecLen) {
-				szAltFont = NULL;
+				aucAltFont = NULL;
 			} else {
-				szAltFont = szFont + iOffsetAltName + 2;
-				NO_DBG_UNICODE(szFont);
-				NO_DBG_UNICODE(szAltFont);
-			}
-			if (bFontEqual(szFont, szWordFont, 2) ||
-			    (szAltFont != NULL &&
-			     bFontEqual(szAltFont, szWordFont, 2)) ||
-			    (pTmp->szWordFontname[0] == '\0' &&
-			     szWordFont[0] == '*' &&
-			     szWordFont[1] == '\0')) {
-				(void)unincpy(pTmp->szWordFontname, szFont,
-					sizeof(pTmp->szWordFontname) - 1);
-				pTmp->szWordFontname[sizeof(
-					pTmp->szWordFontname) - 1] = '\0';
-				(void)strcpy(pTmp->szOurFontname, szOurFont);
+				aucAltFont = aucFont + iOffsetAltName + 2;
+				NO_DBG_UNICODE(aucFont);
+				NO_DBG_UNICODE(aucAltFont);
 			}
+			vFontname2Table(aucFont, aucAltFont, 2, iEmphasis,
+					ucFFN, szWordFont, szOurFont, pTmp);
 			pTmp += 4;
 			iPos += iRecLen + 1;
 		}
 	}
-	vMinimizeFontTable();
-	aucBuffer = xfree(aucBuffer);
 	(void)fclose(pFontTableFile);
+	aucBuffer = xfree(aucBuffer);
+	vMinimizeFontTable();
 } /* end of vCreate8FontTable */
 
+/*
+ * Destroy the internal font table by freeing its memory
+ */
 void
 vDestroyFontTable(void)
 {
 	DBG_MSG("vDestroyFontTable");
 
-	pFontTable = xfree(pFontTable);
 	tFontTableRecords = 0;
+	pFontTable = xfree(pFontTable);
 } /* end of vDestroyFontTable */
 
 /*
@@ -548,16 +770,17 @@ pGetNextFontTableRecord(const font_table_type *pRecordCurr)
 
 	iIndexCurr = pRecordCurr - pFontTable;
 	if (iIndexCurr + 1 < (int)tFontTableRecords) {
-		/* There is a next record */
+		/* There is a next record, so return it */
 		return &pFontTable[iIndexCurr + 1];
 	}
+	/* There is no next record */
 	return NULL;
 } /* end of pGetNextFontTableRecord */
 
 /*
  * tGetFontTableLength
  *
- * returns the number of records in the font table
+ * returns the number of records in the internal font table
  */
 size_t
 tGetFontTableLength(void)
