@@ -126,9 +126,9 @@ bFillPaletteDIB(FILE *pFile, imagedata_type *pImg, BOOL bNewFormat)
 	bIsColorPalette = FALSE;
 	for (iIndex = 0; iIndex < pImg->iColorsUsed; iIndex++) {
 		/* From BGR order to RGB order */
-		pImg->aucPalette[iIndex][2] = (unsigned char)iNextByte(pFile);
-		pImg->aucPalette[iIndex][1] = (unsigned char)iNextByte(pFile);
-		pImg->aucPalette[iIndex][0] = (unsigned char)iNextByte(pFile);
+		pImg->aucPalette[iIndex][2] = (UCHAR)iNextByte(pFile);
+		pImg->aucPalette[iIndex][1] = (UCHAR)iNextByte(pFile);
+		pImg->aucPalette[iIndex][0] = (UCHAR)iNextByte(pFile);
 		if (bNewFormat) {
 			(void)iNextByte(pFile);
 		}
@@ -426,9 +426,9 @@ bFillPalettePNG(FILE *pFile, imagedata_type *pImg, size_t tLength)
 	}
 
 	for (iIndex = 0; iIndex < iEntries; iIndex++) {
-		pImg->aucPalette[iIndex][0] = (unsigned char)iNextByte(pFile);
-		pImg->aucPalette[iIndex][1] = (unsigned char)iNextByte(pFile);
-		pImg->aucPalette[iIndex][2] = (unsigned char)iNextByte(pFile);
+		pImg->aucPalette[iIndex][0] = (UCHAR)iNextByte(pFile);
+		pImg->aucPalette[iIndex][1] = (UCHAR)iNextByte(pFile);
+		pImg->aucPalette[iIndex][2] = (UCHAR)iNextByte(pFile);
 		NO_DBG_PRINT_BLOCK(pImg->aucPalette[iIndex], 3);
 	}
 	for (;iIndex < pImg->iColorsUsed; iIndex++) {
@@ -449,17 +449,17 @@ static BOOL
 bExaminePNG(FILE *pFile, imagedata_type *pImg)
 {
 	size_t		tLength;
-	unsigned long	ulLong1, ulLong2, ulName;
+	ULONG		ulLong1, ulLong2, ulName;
 	int		iIndex, iTmp;
 	int		iCompressionMethod, iFilterMethod, iInterlaceMethod;
 	int		iColor, iIncrement;
 	BOOL		bHasPalette, bHasAlpha;
-	unsigned char	aucBuf[4];
+	UCHAR	aucBuf[4];
 
 	/* Check signature */
 	ulLong1 = ulNextLongBE(pFile);
 	ulLong2 = ulNextLongBE(pFile);
-	if (ulLong1 != 0x89504e47 || ulLong2 != 0x0d0a1a0a) {
+	if (ulLong1 != 0x89504e47UL || ulLong2 != 0x0d0a1a0aUL) {
 		DBG_HEX(ulLong1);
 		DBG_HEX(ulLong2);
 		return FALSE;
@@ -473,7 +473,7 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 		tLength = (size_t)ulNextLongBE(pFile);
 		ulName = 0x00;
 		for (iIndex = 0; iIndex < (int)elementsof(aucBuf); iIndex++) {
-			aucBuf[iIndex] = (unsigned char)iNextByte(pFile);
+			aucBuf[iIndex] = (UCHAR)iNextByte(pFile);
 			if (!isalpha(aucBuf[iIndex])) {
 				DBG_HEX(aucBuf[iIndex]);
 				return FALSE;
@@ -581,9 +581,9 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 		for (iIndex = 0, iColor = 0x00;
 		     iIndex < pImg->iColorsUsed;
 		     iIndex++, iColor += iIncrement) {
-			pImg->aucPalette[iIndex][0] = (unsigned char)iColor;
-			pImg->aucPalette[iIndex][1] = (unsigned char)iColor;
-			pImg->aucPalette[iIndex][2] = (unsigned char)iColor;
+			pImg->aucPalette[iIndex][0] = (UCHAR)iColor;
+			pImg->aucPalette[iIndex][1] = (UCHAR)iColor;
+			pImg->aucPalette[iIndex][2] = (UCHAR)iColor;
 		}
 		/* Just to be sure */
 		pImg->bColorImage = FALSE;
@@ -602,12 +602,12 @@ bExaminePNG(FILE *pFile, imagedata_type *pImg)
 static BOOL
 bExamineWMF(FILE *pFile, imagedata_type *pImg)
 {
-	unsigned long	ulFileSize, ulMaxRecord, ulMagic;
-	unsigned short	usType, usHeaderSize, usVersion, usNoObjects;
+	ULONG	ulFileSize, ulMaxRecord, ulMagic;
+	USHORT	usType, usHeaderSize, usVersion, usNoObjects;
 
 	usType = usNextWord(pFile);
 	usHeaderSize = usNextWord(pFile);
-	ulMagic = ((unsigned long)usHeaderSize << 16) | (unsigned long)usType;
+	ulMagic = ((ULONG)usHeaderSize << 16) | (ULONG)usType;
 	usVersion = usNextWord(pFile);
 	ulFileSize = ulNextLong(pFile);
 	usNoObjects = usNextWord(pFile);
@@ -684,9 +684,9 @@ vImage2Papersize(imagedata_type *pImg)
 static int
 iFind6Image(FILE *pFile, int iPos, int iLen, imagetype_enum *peImageType)
 {
-	unsigned long	ulMarker;
-	int		iElementLen, iToSkip;
-	unsigned short	usMarker;
+	ULONG	ulMarker;
+	int	iElementLen, iToSkip;
+	USHORT	usMarker;
 
 	fail(pFile == NULL);
 	fail(iPos < 0);
@@ -779,7 +779,7 @@ static int
 iFind8Image(FILE *pFile, int iPos, int iLen, imagetype_enum *peImageType)
 {
 	int	iElementTag, iElementLen;
-	unsigned short usID;
+	USHORT	usID;
 
 	fail(pFile == NULL);
 	fail(iPos < 0);
@@ -873,20 +873,20 @@ iFind8Image(FILE *pFile, int iPos, int iLen, imagetype_enum *peImageType)
  * Returns an indication of the amount of information found
  */
 image_info_enum
-eExamineImage(FILE *pFile, long lFileOffsetImage, imagedata_type *pImg)
+eExamineImage(FILE *pFile, ULONG ulFileOffsetImage, imagedata_type *pImg)
 {
 	long	lTmp;
 	size_t	tWordHeaderLen;
 	int	iLen, iPos;
 	int	iType, iHorSize, iVerSize;
-	unsigned short	usHorScalingFactor, usVerScalingFactor;
+	USHORT	usHorScalingFactor, usVerScalingFactor;
 
-	if (lFileOffsetImage < 0) {
+	if (ulFileOffsetImage == FC_INVALID) {
 		return image_no_information;
 	}
-	DBG_HEX(lFileOffsetImage);
+	DBG_HEX(ulFileOffsetImage);
 
-	if (!bSetDataOffset(pFile, lFileOffsetImage)) {
+	if (!bSetDataOffset(pFile, ulFileOffsetImage)) {
 		return image_no_information;
 	}
 
@@ -958,7 +958,7 @@ eExamineImage(FILE *pFile, long lFileOffsetImage, imagedata_type *pImg)
 		break;
 	case  94:	/* Word 6/7, no image just a pathname */
 		pImg->eImageType = imagetype_is_external;
-		DBG_HEX(lFileOffsetImage + iPos);
+		DBG_HEX(ulFileOffsetImage + iPos);
 		break;
 	case 100:
 		iPos = iFind8Image(pFile, iPos, iLen, &pImg->eImageType);
@@ -970,11 +970,11 @@ eExamineImage(FILE *pFile, long lFileOffsetImage, imagedata_type *pImg)
 		break;
 	case 102:	/* Word 8/97, no image just a pathname or URL */
 		pImg->eImageType = imagetype_is_external;
-		DBG_HEX(lFileOffsetImage + iPos);
+		DBG_HEX(ulFileOffsetImage + iPos);
 		break;
 	default:
 		DBG_DEC(iType);
-		DBG_HEX(lFileOffsetImage + iPos);
+		DBG_HEX(ulFileOffsetImage + iPos);
 		DBG_FIXME();
 		return image_no_information;
 	}
