@@ -37,9 +37,9 @@ static options_type	tOptionsCurr;
 static options_type	tOptionsTemp;
 #else
 typedef struct papersize_tag {
-	char		szName[16];	/* Papersize name */
-	unsigned short	usWidth;	/* In points */
-	unsigned short	usHeight;	/* In points */
+	char	szName[16];	/* Papersize name */
+	USHORT	usWidth;	/* In points */
+	USHORT	usHeight;	/* In points */
 } papersize_type;
 
 static const papersize_type atPaperSizes[] = {
@@ -188,28 +188,11 @@ iReadOptions(int argc, char **argv)
 	}
 /* Command line */
 	bPostScript = FALSE;
-	while ((iChar = getopt(argc, argv, "LX:hi:m:p:stw:")) != -1) {
+	while ((iChar = getopt(argc, argv, "Lhi:m:p:stw:")) != -1) {
 		switch (iChar) {
 		case 'L':
 			tOptionsCurr.bUseLandscape = TRUE;
 			break;
-		case 'X':
-			iTmp = (int)strtol(optarg, &pcChar, 10);
-			switch (iTmp) {
-			case 1:
-				tOptionsCurr.eEncoding = encoding_iso_8859_1;
-				strcpy(szLeafname, MAPPING_FILE_DEFAULT_1);
-				break;
-			case 2:
-				tOptionsCurr.eEncoding = encoding_iso_8859_2;
-				strcpy(szLeafname, MAPPING_FILE_DEFAULT_2);
-				break;
-			default:
-				/* Ignore */
-				break;
-			}
-			DBG_MSG(szLeafname);
-			break;
 		case 'h':
 			return 0;
 		case 'i':
@@ -343,7 +326,9 @@ iReadOptions(int argc, char **argv)
 	} else {
 		werr(0, "Global mappingfilename too long, ignored");
 	}
-	werr(0, "I can't open your mapping file (%s)", szLeafname);
+	werr(0, "I can't open your mapping file (%s)\n"
+		"It is not in '%s" FILE_SEPARATOR ANTIWORD_DIR "' nor in '"
+		GLOBAL_ANTIWORD_DIR "'.", szLeafname, szHome);
 	return -1;
 #endif /* __riscos */
 } /* end of iReadOptions */
