@@ -1,6 +1,6 @@
 /*
  * postscript.c
- * Copyright (C) 1999-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1999-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * Functions to deal with the PostScript format
@@ -37,14 +37,16 @@ static const char	*szCreationDate = NULL;
 static const char	*szCreator = NULL;
 /* Current font information */
 static draw_fontref	tFontRefCurr = (draw_fontref)-1;
-static int		iFontsizeCurr = -1;
+static short		sFontsizeCurr = -1;
 static int		iColorCurr = -1;
 /* Current vertical position information */
 static long		lYtopCurr = -1;
 /* PostScript page counter */
-static int	iPageCount = 0;
+static int		iPageCount = 0;
 /* Image counter */
-static int	iImageCount = 0;
+static int		iImageCount = 0;
+/* Local representation of the non-breaking space */
+static UCHAR		ucNbsp = 0;
 
 static char iso_8859_1_data[] = { "\
 /newcodes	% ISO-8859-1 character encodings\n\
@@ -233,7 +235,7 @@ vMoveToPS(diagram_type *pDiag, long lLastVerticalMovement)
 } /* end of vMoveToPS */
 
 /*
- * vPrologue - perform the PostScript initialization
+ * vPrologue - set options and perform the PostScript initialization
  */
 static void
 vPrologue(FILE *pOutFile, const char *szTask, const char *szFilename)
@@ -246,14 +248,18 @@ vPrologue(FILE *pOutFile, const char *szTask, const char *szFilename)
 	fail(szTask == NULL || szTask[0] == '\0');
 
 	vGetOptions(&tOptions);
-	lPageHeight = lPoints2DrawUnits(tOptions.iPageHeight);
+	if (tOptions.iPageHeight == INT_MAX) {
+		lPageHeight = LONG_MAX;
+	} else {
+		lPageHeight = lPoints2DrawUnits(tOptions.iPageHeight);
+	}
 	DBG_DEC(lPageHeight);
 	bUsePostScript = tOptions.bUseOutlineFonts;
 	bUseLandscape = tOptions.bUseLandscape;
 	eEncoding = tOptions.eEncoding;
 	eImageLevel = tOptions.eImageLevel;
 	tFontRefCurr = (draw_fontref)-1;
-	iFontsizeCurr = -1;
+	sFontsizeCurr = -1;
 	iColorCurr = -1;
 	lYtopCurr = -1;
 	iImageCount = 0;
@@ -274,7 +280,7 @@ vPrologue(FILE *pOutFile, const char *szTask, const char *szFilename)
 			szTmp = "unknown";
 		}
 	}
-	fprintf(pOutFile, "%%%%For: %s\n", szTmp);
+	fprintf(pOutFile, "%%%%For: %.50s\n", szTmp);
 	errno = 0;
 	tTime = time(NULL);
 	if (tTime == (time_t)-1 && errno != 0) {
@@ -333,11 +339,12 @@ vPrintPalette(FILE *pOutFile, const imagedata_type *pImg)
 		pImg->bColorImage ? "RGB" : "Gray", pImg->iColorsUsed - 1);
 	fprintf(pOutFile, "<");
 	for (iIndex = 0; iIndex < pImg->iColorsUsed; iIndex++) {
-		fprintf(pOutFile, "%02x", pImg->aucPalette[iIndex][0]);
+		fprintf(pOutFile, "%02x",
+				(unsigned int)pImg->aucPalette[iIndex][0]);
 		if (pImg->bColorImage) {
 			fprintf(pOutFile, "%02x%02x",
-				pImg->aucPalette[iIndex][1],
-				pImg->aucPalette[iIndex][2]);
+				(unsigned int)pImg->aucPalette[iIndex][1],
+				(unsigned int)pImg->aucPalette[iIndex][2]);
 		}
 		if (iIndex % 8 == 7) {
 			fprintf(pOutFile, "\n");
@@ -756,10 +763,10 @@ vAddFonts2Diagram(diagram_type *pDiag)
  */
 static void
 vPrintPS(FILE *pFile, const char *szString, int iStringLength,
-		unsigned char ucFontstyle)
+		UCHAR ucFontstyle)
 {
-	int iCount;
-	const unsigned char *ucBytes;
+	const UCHAR	*ucBytes;
+	int		iCount;
 
 	fail(szString == NULL || iStringLength < 0);
 
@@ -767,7 +774,7 @@ vPrintPS(FILE *pFile, const char *szString, int iStringLength,
 		return;
 	}
 
-	ucBytes = (unsigned char *)szString;
+	ucBytes = (UCHAR *)szString;
 	(void)putc('(', pFile);
 	for (iCount = 0; iCount < iStringLength ; iCount++) {
 		switch (ucBytes[iCount]) {
@@ -778,13 +785,14 @@ vPrintPS(FILE *pFile, const char *szString, int iStringLength,
 			(void)putc(szString[iCount], pFile);
 			break;
 		default:
-			if (ucBytes[iCount] < 0x20 ||
-			    (ucBytes[iCount] >= 0x7f &&
-			     ucBytes[iCount] < 0xa0)) {
+			if ((int)ucBytes[iCount] < 0x20 ||
+			    ((int)ucBytes[iCount] >= 0x7f &&
+			     (int)ucBytes[iCount] < 0xa0)) {
 				DBG_HEX(ucBytes[iCount]);
 				(void)putc(' ', pFile);
-			} else if (ucBytes[iCount] >= 0x80) {
-				fprintf(pFile, "\\%03o", ucBytes[iCount]);
+			} else if ((int)ucBytes[iCount] >= 0x80) {
+				fprintf(pFile, "\\%03o",
+						(unsigned int)ucBytes[iCount]);
 			} else {
 				(void)putc(szString[iCount], pFile);
 			}
@@ -792,41 +800,77 @@ vPrintPS(FILE *pFile, const char *szString, int iStringLength,
 		}
 	}
 	fprintf(pFile, ") ");
-	if (bIsStrike(ucFontstyle) && iFontsizeCurr > 0) {
+	if ((bIsStrike(ucFontstyle) || bIsMarkDel(ucFontstyle)) &&
+			sFontsizeCurr > 0) {
 		fprintf(pFile, "%.2f %.2f LineShow\n",
-			iFontsizeCurr * 0.02, iFontsizeCurr * 0.12);
-	} else if (bIsUnderline(ucFontstyle) && iFontsizeCurr > 0) {
+			sFontsizeCurr * 0.02, sFontsizeCurr * 0.12);
+	} else if (bIsUnderline(ucFontstyle) && sFontsizeCurr > 0) {
 		fprintf(pFile, "%.2f %.2f LineShow\n",
-			iFontsizeCurr * 0.02, iFontsizeCurr * -0.06);
+			sFontsizeCurr * 0.02, sFontsizeCurr * -0.06);
 	} else {
 		fprintf(pFile, "show\n");
 	}
 } /* end of vPrintPS */
 
+/*
+ * vPrintTXT - print a Text string
+ */
+static void
+vPrintTXT(FILE *pFile, const char *szString, int iStringLength)
+{
+	const UCHAR	*ucBytes;
+	int		iCount;
+
+	fail(szString == NULL || iStringLength < 0);
+
+	if (szString == NULL || szString[0] == '\0' || iStringLength <= 0) {
+		return;
+	}
+
+	if (eEncoding == encoding_utf8) {
+		fprintf(pFile, "%.*s", iStringLength, szString);
+		return;
+	}
+
+	if (ucNbsp == 0) {
+		ucNbsp = ucGetNbspValue();
+		DBG_HEX_C(ucNbsp != 0xa0, ucNbsp);
+	}
+
+	ucBytes = (UCHAR *)szString;
+	for (iCount = 0; iCount < iStringLength ; iCount++) {
+		if (ucBytes[iCount] == ucNbsp) {
+			(void)putc(' ', pFile);
+		} else {
+			(void)putc(szString[iCount], pFile);
+		}
+	}
+} /* end of vPrintTXT */
+
 /*
  * vSetColor - move to the given color
  */
 static void
 vSetColor(FILE *pFile, int iColor)
 {
-	unsigned int	uiTmp, uiRed, uiGreen, uiBlue;
+	ULONG	ulTmp, ulRed, ulGreen, ulBlue;
 
-	uiTmp = uiColor2Color(iColor);
-	uiRed   = (uiTmp & 0x0000ff00) >> 8;
-	uiGreen = (uiTmp & 0x00ff0000) >> 16;
-	uiBlue  = (uiTmp & 0xff000000) >> 24;
+	ulTmp = ulColor2Color(iColor);
+	ulRed   = (ulTmp & 0x0000ff00) >> 8;
+	ulGreen = (ulTmp & 0x00ff0000) >> 16;
+	ulBlue  = (ulTmp & 0xff000000) >> 24;
 	fprintf(pFile, "%.3f %.3f %.3f setrgbcolor\n",
-				uiRed / 255.0, uiGreen / 255.0, uiBlue / 255.0);
+				ulRed / 255.0, ulGreen / 255.0, ulBlue / 255.0);
 } /* end of vSetColor */
 
 /*
- * vMoveToASCII - move to the given X,Y coordinates (ASCII)
+ * vMoveToTXT - move to the given X,Y coordinates (Text)
  *
  * Move the current position of the given diagram to its X,Y coordinates,
  * start on a new page if needed
  */
 static void
-vMoveToASCII(diagram_type *pDiag)
+vMoveToTXT(diagram_type *pDiag)
 {
 	int	iCount, iNbr;
 
@@ -836,23 +880,23 @@ vMoveToASCII(diagram_type *pDiag)
 	if (pDiag->lYtop != lYtopCurr) {
 		iNbr = iDrawUnits2Char(pDiag->lXleft);
 		for (iCount = 0; iCount < iNbr; iCount++) {
-			fprintf(pDiag->pOutFile, "%c", FILLER_CHAR);
+			(void)putc(FILLER_CHAR, pDiag->pOutFile);
 		}
 		lYtopCurr = pDiag->lYtop;
 	}
-} /* end of vMoveToASCII */
+} /* end of vMoveToTXT */
 
 /*
  * vMove2NextLine - move to the next line
  */
 void
-vMove2NextLine(diagram_type *pDiag, draw_fontref tFontRef, int iFontsize)
+vMove2NextLine(diagram_type *pDiag, draw_fontref tFontRef, short sFontsize)
 {
 	fail(pDiag == NULL);
 	fail(pDiag->pOutFile == NULL);
-	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
+	fail(sFontsize < MIN_FONT_SIZE || sFontsize > MAX_FONT_SIZE);
 
-	pDiag->lYtop -= lComputeLeading(iFontsize);
+	pDiag->lYtop -= lComputeLeading(sFontsize);
 	if (!bUsePostScript) {
 		(void)fprintf(pDiag->pOutFile, "\n");
 	}
@@ -864,8 +908,8 @@ vMove2NextLine(diagram_type *pDiag, draw_fontref tFontRef, int iFontsize)
 void
 vSubstring2Diagram(diagram_type *pDiag,
 	char *szString, int iStringLength, long lStringWidth,
-	int iColor, unsigned char ucFontstyle, draw_fontref tFontRef,
-	int iFontsize, int iMaxFontsize)
+	int iColor, UCHAR ucFontstyle, draw_fontref tFontRef,
+	short sFontsize, short sMaxFontsize)
 {
 	const char	*szOurFontname;
 
@@ -874,59 +918,86 @@ vSubstring2Diagram(diagram_type *pDiag,
 	fail(pDiag->lXleft < 0);
 	fail(iStringLength < 0);
 	fail((size_t)iStringLength != strlen(szString));
-	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
-	fail(iMaxFontsize < MIN_FONT_SIZE || iMaxFontsize > MAX_FONT_SIZE);
-	fail(iFontsize > iMaxFontsize);
+	fail(sFontsize < MIN_FONT_SIZE || sFontsize > MAX_FONT_SIZE);
+	fail(sMaxFontsize < MIN_FONT_SIZE || sMaxFontsize > MAX_FONT_SIZE);
+	fail(sFontsize > sMaxFontsize);
 
 	if (szString[0] == '\0' || iStringLength <= 0) {
 		return;
 	}
 
 	if (bUsePostScript) {
-		if (tFontRef != tFontRefCurr || iFontsize != iFontsizeCurr) {
+		if (tFontRef != tFontRefCurr || sFontsize != sFontsizeCurr) {
 			szOurFontname = szGetFontname(tFontRef);
 			fail(szOurFontname == NULL);
 			fprintf(pDiag->pOutFile,
 				"%.1f /%s /%s-ISO-8859-x ChgFnt\n",
-				(double)iFontsize / 2.0,
+				(double)sFontsize / 2.0,
 				szOurFontname, szOurFontname);
 			tFontRefCurr = tFontRef;
-			iFontsizeCurr = iFontsize;
+			sFontsizeCurr = sFontsize;
 		}
 		if (iColor != iColorCurr) {
 			vSetColor(pDiag->pOutFile, iColor);
 			iColorCurr = iColor;
 		}
-		vMoveToPS(pDiag, lComputeLeading(iMaxFontsize));
+		vMoveToPS(pDiag, lComputeLeading(sMaxFontsize));
 		vPrintPS(pDiag->pOutFile, szString, iStringLength, ucFontstyle);
 	} else {
-		vMoveToASCII(pDiag);
-		fprintf(pDiag->pOutFile, "%.*s", iStringLength, szString);
+		vMoveToTXT(pDiag);
+		vPrintTXT(pDiag->pOutFile, szString, iStringLength);
 	}
 	pDiag->lXleft += lStringWidth;
 } /* end of vSubstring2Diagram */
 
 /*
- * Create an end of paragraph by moving the y-high mark 1/3 line down
+ * Create an start of paragraph by moving the y-top mark
  */
 void
-vEndOfParagraph2Diagram(diagram_type *pDiag,
-			draw_fontref tFontRef, int iFontsize)
+vStartOfParagraph2Diagram(diagram_type *pDiag,
+	draw_fontref tFontRef, short sFontsize, long lBeforeIndentation)
 {
 	fail(pDiag == NULL);
 	fail(pDiag->pOutFile == NULL);
-	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
+	fail(sFontsize < MIN_FONT_SIZE || sFontsize > MAX_FONT_SIZE);
+	fail(lBeforeIndentation < 0);
+
+	if (bUsePostScript) {
+		pDiag->lXleft = 0;
+		pDiag->lYtop -= lMilliPoints2DrawUnits(lBeforeIndentation);
+	} else {
+		if (lBeforeIndentation >= lTwips2MilliPoints(HEADING_GAP)) {
+			/* A large gap is replaced by an empty line */
+			vMove2NextLine(pDiag, tFontRef, sFontsize);
+		}
+	}
+} /* end of vStartOfParagraph2Diagram */
+
+/*
+ * Create an end of paragraph by moving the y-top mark
+ */
+void
+vEndOfParagraph2Diagram(diagram_type *pDiag,
+	draw_fontref tFontRef, short sFontsize, long lAfterIndentation)
+{
+	fail(pDiag == NULL);
+	fail(pDiag->pOutFile == NULL);
+	fail(sFontsize < MIN_FONT_SIZE || sFontsize > MAX_FONT_SIZE);
+	fail(lAfterIndentation < 0);
 
 	if (pDiag->lXleft > 0) {
 		/* To the start of the line */
-		vMove2NextLine(pDiag, tFontRef, iFontsize);
+		vMove2NextLine(pDiag, tFontRef, sFontsize);
 	}
-	/* Empty line */
+
 	if (bUsePostScript) {
 		pDiag->lXleft = 0;
-		pDiag->lYtop -= lComputeLeading(iFontsize) / 3;
+		pDiag->lYtop -= lMilliPoints2DrawUnits(lAfterIndentation);
 	} else {
-		vMove2NextLine(pDiag, tFontRef, iFontsize);
+		if (lAfterIndentation >= lTwips2MilliPoints(HEADING_GAP)) {
+			/* A large gap is replaced by an empty line */
+			vMove2NextLine(pDiag, tFontRef, sFontsize);
+		}
 	}
 } /* end of vEndOfParagraph2Diagram */
 
@@ -935,11 +1006,14 @@ vEndOfParagraph2Diagram(diagram_type *pDiag,
  */
 void
 vEndOfPage2Diagram(diagram_type *pDiag,
-			draw_fontref tFontRef, int iFontsize)
+	draw_fontref tFontRef, short sFontsize, long lAfterIndentation)
 {
 	if (bUsePostScript) {
 		vMove2NextPage(pDiag);
 	} else {
-		vEndOfParagraph2Diagram(pDiag, tFontRef, iFontsize);
+		vEndOfParagraph2Diagram(pDiag,
+					tFontRef,
+					sFontsize,
+					lAfterIndentation);
 	}
 } /* end of vEndOfPage2Diagram */
