@@ -1,6 +1,6 @@
 /*
  * prop6.c
- * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * Read the property information from a MS Word 6 or 7 file
@@ -15,57 +15,208 @@
  * iGet6InfoLength - the length of the information for Word 6/7 files
  */
 static int
-iGet6InfoLength(int iByteNbr, const unsigned char *aucFpage)
+iGet6InfoLength(int iByteNbr, const UCHAR *aucGrpprl)
 {
 	int	iTmp, iDel, iAdd;
 
-	switch (ucGetByte(iByteNbr, aucFpage)) {
-	case 0x02: case 0x10: case 0x11: case 0x13:
-	case 0x15: case 0x16: case 0x1b: case 0x1c:
-	case 0x26: case 0x27: case 0x28: case 0x29:
-	case 0x2a: case 0x2b: case 0x2d: case 0x2e:
-	case 0x2f: case 0x30: case 0x31: case 0x50:
-	case 0x5d: case 0x60: case 0x61: case 0x63:
-	case 0x65: case 0x69: case 0x6a: case 0x6b:
-	case 0x90: case 0xa4: case 0xa5: case 0xb6:
-	case 0xb8: case 0xbd: case 0xc3: case 0xc5:
-	case 0xc6:
-		return 3;
-	case 0x03: case 0x0c: case 0x0f: case 0x51:
-	case 0x67: case 0x6c: case 0xbc: case 0xbe:
-	case 0xbf:
-		return 2 + (int)ucGetByte(iByteNbr + 1, aucFpage);
-	case 0x14: case 0x49: case 0x4a: case 0xc0:
-	case 0xc2: case 0xc4: case 0xc8:
-		return 5;
-	case 0x17:
-		iTmp = (int)ucGetByte(iByteNbr + 1, aucFpage);
+	switch (ucGetByte(iByteNbr, aucGrpprl)) {
+	case   2: case  16: case  17: case  18: case  19: case  21: case  22:
+	case  26: case  27: case  28: case  30: case  31: case  32: case  33:
+	case  34: case  35: case  36: case  38: case  39: case  40: case  41:
+	case  42: case  43: case  45: case  46: case  47: case  48: case  49:
+	case  69: case  72: case  80: case  93: case  96: case  97: case  99:
+	case 101: case 105: case 106: case 107: case 109: case 110: case 121:
+	case 122: case 123: case 124: case 140: case 141: case 144: case 145:
+	case 148: case 149: case 154: case 155: case 156: case 157: case 160:
+	case 161: case 164: case 165: case 166: case 167: case 168: case 169:
+	case 170: case 171: case 182: case 183: case 184: case 189: case 195:
+	case 197: case 198:
+		return 1 + 2;
+	case   3: case  12: case  15: case  81: case 103: case 108: case 188:
+	case 190: case 191:
+		return 2 + (int)ucGetByte(iByteNbr + 1, aucGrpprl);
+	case  20: case  70: case  74: case 192: case 194: case 196: case 200:
+		return 1 + 4;
+	case  23:
+		iTmp = (int)ucGetByte(iByteNbr + 1, aucGrpprl);
 		if (iTmp == 255) {
-			iDel = (int)ucGetByte(iByteNbr + 2, aucFpage);
+			iDel = (int)ucGetByte(iByteNbr + 2, aucGrpprl);
 			iAdd = (int)ucGetByte(
-					iByteNbr + 3 + iDel * 4, aucFpage);
+					iByteNbr + 3 + iDel * 4, aucGrpprl);
 			iTmp = 2 + iDel * 4 + iAdd * 3;
 		}
 		return 2 + iTmp;
-	case 0x44: case 0xc1: case 0xc7:
-		return 6;
-	case 0x5f: case 0x88: case 0x89:
-		return 4;
-	case 0x78:
-		return 14;
-	case 0xbb:
-		return 13;
+	case  68: case 193: case 199:
+		return 1 + 5;
+	case  73: case  95: case 136: case 137:
+		return 1 + 3;
+	case 120:
+		return 1 + 13;
+	case 187:
+		return 1 + 12;
 	default:
-		return 2;
+		return 1 + 1;
 	}
 } /* end of iGet6InfoLength */
 
+/*
+ * Fill the section information block with information
+ * from a Word 6/7 file.
+ */
+static void
+vGet6SectionInfo(const UCHAR *aucGrpprl, size_t tBytes,
+		section_block_type *pSection)
+{
+	int	iFodoOff, iInfoLen, iSize, iIndex, iTmp;
+	USHORT	usCcol;
+	UCHAR	ucTmp;
+
+	fail(aucGrpprl == NULL || pSection == NULL);
+
+	iFodoOff = 0;
+	while (tBytes >= (size_t)iFodoOff + 1) {
+		iInfoLen = 0;
+		switch (ucGetByte(iFodoOff, aucGrpprl)) {
+		case 133:	/* olstAnm */
+			iSize = (int)ucGetByte(iFodoOff + 1, aucGrpprl);
+			DBG_DEC_C(iSize != 212, iSize);
+			for (iIndex = 0, iTmp = iFodoOff + 2;
+			     iIndex < 9 && iTmp < iFodoOff + 2 + iSize - 15;
+			     iIndex++, iTmp += 16) {
+				pSection->aucNFC[iIndex] =
+						ucGetByte(iTmp, aucGrpprl);
+				DBG_DEC(pSection->aucNFC[iIndex]);
+				ucTmp = ucGetByte(iTmp + 3, aucGrpprl);
+				DBG_HEX(ucTmp);
+				if ((ucTmp & BIT(2)) != 0) {
+					pSection->usNeedPrevLvl |=
+							(USHORT)BIT(iIndex);
+				}
+				if ((ucTmp & BIT(3)) != 0) {
+					pSection->usHangingIndent |=
+							(USHORT)BIT(iIndex);
+				}
+			}
+			DBG_HEX(pSection->usNeedPrevLvl);
+			DBG_HEX(pSection->usHangingIndent);
+			break;
+		case 142:	/* bkc */
+			ucTmp = ucGetByte(iFodoOff + 1, aucGrpprl);
+			DBG_DEC(ucTmp);
+			pSection->bNewPage = ucTmp != 0 && ucTmp != 1;
+			break;
+		case 144:	/* ccolM1 */
+			usCcol = 1 + usGetWord(iFodoOff + 1, aucGrpprl);
+			DBG_DEC(usCcol);
+			break;
+		default:
+			break;
+		}
+		if (iInfoLen <= 0) {
+			iInfoLen = iGet6InfoLength(iFodoOff, aucGrpprl);
+			fail(iInfoLen <= 0);
+		}
+		iFodoOff += iInfoLen;
+	}
+} /* end of vGet6SectionInfo */
+
+/*
+ * Build the lists with Section Property Information for Word 6/7 files
+ */
+void
+vGet6SepInfo(FILE *pFile, ULONG ulStartBlock,
+	const ULONG *aulBBD, size_t tBBDLen,
+	const UCHAR *aucHeader)
+{
+	section_block_type	tSection;
+	ULONG		*aulSectPage, *aulTextOffset;
+	UCHAR	*aucBuffer, *aucFpage;
+	ULONG	ulBeginSectInfo;
+	size_t	tSectInfoLen, tOffset, tSize, tLen, tBytes;
+	int	iIndex;
+	UCHAR	aucTmp[2];
+
+	fail(pFile == NULL || aucHeader == NULL);
+	fail(ulStartBlock > MAX_BLOCKNUMBER && ulStartBlock != END_OF_CHAIN);
+	fail(aulBBD == NULL);
+
+	ulBeginSectInfo = ulGetLong(0x88, aucHeader); /* fcPlcfsed */
+	DBG_HEX(ulBeginSectInfo);
+	tSectInfoLen = (size_t)ulGetLong(0x8c, aucHeader); /* lcbPlcfsed */
+	DBG_DEC(tSectInfoLen);
+	if (tSectInfoLen < 4) {
+		DBG_DEC(tSectInfoLen);
+		return;
+	}
+
+	aucBuffer = xmalloc(tSectInfoLen);
+	if (!bReadBuffer(pFile, ulStartBlock,
+			aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+			aucBuffer, ulBeginSectInfo, tSectInfoLen)) {
+		aucBuffer = xfree(aucBuffer);
+		return;
+	}
+	NO_DBG_PRINT_BLOCK(aucBuffer, tSectInfoLen);
+
+	/* Read the Section Descriptors */
+	tLen = (tSectInfoLen - 4) / 16;
+	tSize = tLen * sizeof(ULONG);
+	/* Save the section offsets */
+	aulTextOffset = xmalloc(tSize);
+	for (iIndex = 0, tOffset = 0;
+	     iIndex < (int)tLen;
+	     iIndex++, tOffset += 4) {
+		aulTextOffset[iIndex] = ulGetLong(tOffset, aucBuffer);
+	}
+	/* Save the Sepx offsets */
+	aulSectPage = xmalloc(tSize);
+	for (iIndex = 0, tOffset = (tLen + 1) * 4;
+	     iIndex < (int)tLen;
+	     iIndex++, tOffset += 12) {
+		aulSectPage[iIndex] = ulGetLong(tOffset + 2, aucBuffer);
+		NO_DBG_HEX(aulSectPage[iIndex]); /* fcSepx */
+	}
+	aucBuffer = xfree(aucBuffer);
+
+	/* Read the Section Properties */
+	for (iIndex = 0; iIndex < (int)tLen; iIndex++) {
+		if (aulSectPage[iIndex] == FC_INVALID) {
+			vDefault2SectionInfoList(aulTextOffset[iIndex]);
+			continue;
+		}
+		/* Get the number of bytes to read */
+		if (!bReadBuffer(pFile, ulStartBlock,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+				aucTmp, aulSectPage[iIndex], 2)) {
+			continue;
+		}
+		tBytes = 2 + (size_t)usGetWord(0, aucTmp);
+		NO_DBG_DEC(tBytes);
+		/* Read the bytes */
+		aucFpage = xmalloc(tBytes);
+		if (!bReadBuffer(pFile, ulStartBlock,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+				aucFpage, aulSectPage[iIndex], tBytes)) {
+			aucFpage = xfree(aucFpage);
+			continue;
+		}
+		NO_DBG_PRINT_BLOCK(aucFpage, tBytes);
+		/* Process the bytes */
+		vGetDefaultSection(&tSection);
+		vGet6SectionInfo(aucFpage + 2, tBytes - 2, &tSection);
+		vAdd2SectionInfoList(&tSection, aulTextOffset[iIndex]);
+		aucFpage = xfree(aucFpage);
+	}
+	aulTextOffset = xfree(aulTextOffset);
+	aulSectPage = xfree(aulSectPage);
+} /* end of vGet6SepInfo */
+
 /*
  * Translate the rowinfo to a member of the row_info enumeration
  */
 row_info_enum
 eGet6RowInfo(int iFodo,
-	const unsigned char *aucGrpprl, int iBytes, row_block_type *pRow)
+	const UCHAR *aucGrpprl, int iBytes, row_block_type *pRow)
 {
 	int	iFodoOff, iInfoLen;
 	int	iIndex, iSize, iCol;
@@ -98,7 +249,7 @@ eGet6RowInfo(int iFodo,
 			}
 			break;
 		case 190:	/* cDefTable */
-			iSize = (int)ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
+			iSize = (int)usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
 			if (iSize < 6 || iBytes < iFodoOff + 7) {
 				DBG_DEC(iSize);
 				DBG_DEC(iFodoOff);
@@ -114,9 +265,10 @@ eGet6RowInfo(int iFodo,
 				break;
 			}
 			if (iCol >= (int)elementsof(pRow->asColumnWidth)) {
+				DBG_DEC(iCol);
 				werr(1, "The number of columns is corrupt");
 			}
-			pRow->ucNumberOfColumns = (unsigned char)iCol;
+			pRow->ucNumberOfColumns = (UCHAR)iCol;
 			pRow->iColumnWidthSum = 0;
 			iPosPrev = (int)(short)usGetWord(
 					iFodo + iFodoOff + 4,
@@ -161,74 +313,90 @@ eGet6RowInfo(int iFodo,
 /*
  * Fill the style information block with information
  * from a Word 6/7 file.
- * Returns TRUE when successful, otherwise FALSE
  */
-static BOOL
-bGet6StyleInfo(int iFodo, const unsigned char *aucFpage,
-		style_block_type *pStyle)
+void
+vGet6StyleInfo(int iFodo,
+	const UCHAR *aucGrpprl, int iBytes, style_block_type *pStyle)
 {
-	int	iBytes, iFodoOff, iInfoLen;
-	int	iTmp, iDel, iAdd;
+	int	iFodoOff, iInfoLen;
+	int	iTmp, iDel, iAdd, iBefore;
 	short	sTmp;
+	UCHAR	ucTmp;
 
-	fail(iFodo <= 0 || aucFpage == NULL || pStyle == NULL);
+	fail(iFodo < 0 || aucGrpprl == NULL || pStyle == NULL);
 
-	iBytes = 2 * (int)ucGetByte(iFodo, aucFpage);
-	iFodoOff = 3;
-	if (iBytes < 1) {
-		return FALSE;
-	}
-	(void)memset(pStyle, 0, sizeof(*pStyle));
-	pStyle->ucStyle = ucGetByte(iFodo + 1, aucFpage);
+	NO_DBG_DEC(pStyle->usIstd);
+
+	iFodoOff = 0;
 	while (iBytes >= iFodoOff + 1) {
 		iInfoLen = 0;
-		switch (ucGetByte(iFodo + iFodoOff, aucFpage)) {
+		switch (ucGetByte(iFodo + iFodoOff, aucGrpprl)) {
 		case   2:	/* istd */
 			sTmp = (short)ucGetByte(
-					iFodo + iFodoOff + 1, aucFpage);
+					iFodo + iFodoOff + 1, aucGrpprl);
 			NO_DBG_DEC(sTmp);
-			NO_DBG_DEC(pStyle->ucStyle);
 			break;
 		case   5:	/* jc */
 			pStyle->ucAlignment = ucGetByte(
-					iFodo + iFodoOff + 1, aucFpage);
+					iFodo + iFodoOff + 1, aucGrpprl);
 			break;
 		case  12:	/* anld */
 			iTmp = (int)ucGetByte(
-					iFodo + iFodoOff + 1, aucFpage);
-			if (iTmp < 52) {
-				DBG_DEC(iTmp);
-				break;
-			}
-			pStyle->ucListType = ucGetByte(
-					iFodo + iFodoOff + 2, aucFpage);
+					iFodo + iFodoOff + 1, aucGrpprl);
+			DBG_DEC_C(iTmp < 52, iTmp);
 			pStyle->bInList = TRUE;
-			pStyle->ucListCharacter = ucGetByte(
-					iFodo + iFodoOff + 22, aucFpage);
+			if (iTmp >= 1) {
+				pStyle->ucNFC = ucGetByte(
+					iFodo + iFodoOff + 2, aucGrpprl);
+			}
+			if (pStyle->ucNFC != LIST_BULLETS && iTmp >= 2) {
+				iBefore = (int)ucGetByte(
+					iFodo + iFodoOff + 3, aucGrpprl);
+			} else {
+				iBefore = 0;
+			}
+			if (iTmp >= 12) {
+				pStyle->usStartAt = usGetWord(
+					iFodo + iFodoOff + 12, aucGrpprl);
+			}
+			if (iTmp >= iBefore + 21) {
+				pStyle->ucListCharacter = ucGetByte(
+					iFodo + iFodoOff + iBefore + 22,
+					aucGrpprl);
+				NO_DBG_CHR(pStyle->ucListCharacter);
+			}
 			break;
 		case  13:	/* nLvlAnm */
-			iTmp = (int)ucGetByte(
-					iFodo + iFodoOff + 1, aucFpage);
-			pStyle->bInList = TRUE;
-			if (iTmp == 0x0c) {
-				pStyle->bUnmarked = TRUE;
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
+			pStyle->ucListLevel = ucTmp;
+#if 0
+			if (iTmp == 0) {
+				pStyle->bNoNumbering = TRUE;
 			}
+			if (iTmp >= 12) {
+				pStyle->bNumPause = TRUE;
+			}
+#else
+			if ((int)ucTmp == 0 || (int)ucTmp >= 12) {
+				pStyle->bNumPause = TRUE;
+			}
+#endif
 			break;
 		case  15:	/* ChgTabsPapx */
-			iTmp = (int)ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+			iTmp = (int)ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			if (iTmp < 2) {
 				iInfoLen = 1;
 				break;
 			}
 			NO_DBG_DEC(iTmp);
-			iDel = (int)ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			iDel = (int)ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			if (iTmp < 2 + 2 * iDel) {
 				iInfoLen = 1;
 				break;
 			}
 			NO_DBG_DEC(iDel);
 			iAdd = (int)ucGetByte(
-				iFodo + iFodoOff + 3 + 2 * iDel, aucFpage);
+				iFodo + iFodoOff + 3 + 2 * iDel, aucGrpprl);
 			if (iTmp < 2 + 2 * iDel + 2 * iAdd) {
 				iInfoLen = 1;
 				break;
@@ -237,16 +405,17 @@ bGet6StyleInfo(int iFodo, const unsigned char *aucFpage,
 			break;
 		case  16:	/* dxaRight */
 			pStyle->sRightIndent = (short)usGetWord(
-					iFodo + iFodoOff + 1, aucFpage);
+					iFodo + iFodoOff + 1, aucGrpprl);
 			NO_DBG_DEC(pStyle->sRightIndent);
 			break;
 		case  17:	/* dxaLeft */
 			pStyle->sLeftIndent = (short)usGetWord(
-					iFodo + iFodoOff + 1, aucFpage);
+					iFodo + iFodoOff + 1, aucGrpprl);
 			NO_DBG_DEC(pStyle->sLeftIndent);
 			break;
 		case  18:	/* Nest dxaLeft */
-			sTmp = (short)usGetWord(iFodo + iFodoOff + 1, aucFpage);
+			sTmp = (short)usGetWord(
+					iFodo + iFodoOff + 1, aucGrpprl);
 			pStyle->sLeftIndent += sTmp;
 			if (pStyle->sLeftIndent < 0) {
 				pStyle->sLeftIndent = 0;
@@ -254,44 +423,54 @@ bGet6StyleInfo(int iFodo, const unsigned char *aucFpage,
 			NO_DBG_DEC(sTmp);
 			NO_DBG_DEC(pStyle->sLeftIndent);
 			break;
+		case  21:	/* dyaBefore */
+			pStyle->usBeforeIndent = usGetWord(
+					iFodo + iFodoOff + 1, aucGrpprl);
+			NO_DBG_DEC(pStyle->usBeforeIndent);
+			break;
+		case  22:	/* dyaAfter */
+			pStyle->usAfterIndent = usGetWord(
+					iFodo + iFodoOff + 1, aucGrpprl);
+			NO_DBG_DEC(pStyle->usAfterIndent);
+			break;
 		default:
 			break;
 		}
 		if (iInfoLen <= 0) {
 			iInfoLen =
-				iGet6InfoLength(iFodo + iFodoOff, aucFpage);
+				iGet6InfoLength(iFodo + iFodoOff, aucGrpprl);
 			fail(iInfoLen <= 0);
 		}
 		iFodoOff += iInfoLen;
 	}
-	return TRUE;
-} /* end of bGet6StyleInfo */
+} /* end of vGet6StyleInfo */
 
 /*
  * Build the lists with Paragraph Information for Word 6/7 files
  */
 void
-vGet6PapInfo(FILE *pFile, long lStartBlock,
-	const long *alBBD, size_t tBBDLen,
-	const unsigned char *aucHeader)
+vGet6PapInfo(FILE *pFile, ULONG ulStartBlock,
+	const ULONG *aulBBD, size_t tBBDLen,
+	const UCHAR *aucHeader)
 {
 	row_block_type		tRow;
 	style_block_type	tStyle;
-	unsigned short	*ausParfPage;
-	unsigned char	*aucBuffer;
-	long	lParmOffset, lParmOffsetFirst, lParmOffsetLast, lBeginParfInfo;
+	USHORT	*ausParfPage;
+	UCHAR	*aucBuffer;
+	ULONG	ulCharPos, ulCharPosFirst, ulCharPosLast;
+	ULONG	ulBeginParfInfo;
 	size_t	tParfInfoLen, tParfPageNum, tOffset, tSize, tLenOld, tLen;
 	int	iIndex, iIndex2, iRun, iFodo, iLen;
 	row_info_enum	eRowInfo;
-	unsigned short	usParfFirstPage, usCount;
-	unsigned char	aucFpage[BIG_BLOCK_SIZE];
+	USHORT	usParfFirstPage, usCount, usIstd;
+	UCHAR	aucFpage[BIG_BLOCK_SIZE];
 
 	fail(pFile == NULL || aucHeader == NULL);
-	fail(lStartBlock < 0);
-	fail(alBBD == NULL);
+	fail(ulStartBlock > MAX_BLOCKNUMBER && ulStartBlock != END_OF_CHAIN);
+	fail(aulBBD == NULL);
 
-	lBeginParfInfo = (long)ulGetLong(0xc0, aucHeader); /* fcPlcfbtePapx */
-	NO_DBG_HEX(lBeginParfInfo);
+	ulBeginParfInfo = ulGetLong(0xc0, aucHeader); /* fcPlcfbtePapx */
+	NO_DBG_HEX(ulBeginParfInfo);
 	tParfInfoLen = (size_t)ulGetLong(0xc4, aucHeader); /* lcbPlcfbtePapx */
 	NO_DBG_DEC(tParfInfoLen);
 	if (tParfInfoLen < 4) {
@@ -300,16 +479,16 @@ vGet6PapInfo(FILE *pFile, long lStartBlock,
 	}
 
 	aucBuffer = xmalloc(tParfInfoLen);
-	if (!bReadBuffer(pFile, lStartBlock,
-			alBBD, tBBDLen, BIG_BLOCK_SIZE,
-			aucBuffer, lBeginParfInfo, tParfInfoLen)) {
+	if (!bReadBuffer(pFile, ulStartBlock,
+			aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+			aucBuffer, ulBeginParfInfo, tParfInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		return;
 	}
 	NO_DBG_PRINT_BLOCK(aucBuffer, tParfInfoLen);
 
 	tLen = (tParfInfoLen - 4) / 6;
-	tSize = tLen * sizeof(unsigned short);
+	tSize = tLen * sizeof(USHORT);
 	ausParfPage = xmalloc(tSize);
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
@@ -319,16 +498,15 @@ vGet6PapInfo(FILE *pFile, long lStartBlock,
 	}
 	DBG_HEX(ulGetLong(0, aucBuffer));
 	aucBuffer = xfree(aucBuffer);
-	aucBuffer = NULL;
-	tParfPageNum = (size_t)usGetWord(0x190, aucHeader);
+	tParfPageNum = (size_t)usGetWord(0x190, aucHeader); /* cpnBtePap */
 	DBG_DEC(tParfPageNum);
 	if (tLen < tParfPageNum) {
 		/* Replace ParfPage by a longer version */
 		tLenOld = tLen;
-		usParfFirstPage = usGetWord(0x18c, aucHeader);
+		usParfFirstPage = usGetWord(0x18c, aucHeader); /* pnPapFirst */
 		DBG_DEC(usParfFirstPage);
 		tLen += tParfPageNum - 1;
-		tSize = tLen * sizeof(unsigned short);
+		tSize = tLen * sizeof(USHORT);
 		ausParfPage = xrealloc(ausParfPage, tSize);
 		/* Add new values */
 		usCount = usParfFirstPage + 1;
@@ -339,14 +517,13 @@ vGet6PapInfo(FILE *pFile, long lStartBlock,
 		}
 	}
 
-	(void)memset(&tStyle, 0, sizeof(tStyle));
 	(void)memset(&tRow, 0, sizeof(tRow));
-	lParmOffsetFirst = -1;
+	ulCharPosFirst = CP_INVALID;
 	for (iIndex = 0; iIndex < (int)tLen; iIndex++) {
-		if (!bReadBuffer(pFile, lStartBlock,
-				alBBD, tBBDLen, BIG_BLOCK_SIZE,
+		if (!bReadBuffer(pFile, ulStartBlock,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
 				aucFpage,
-				(long)ausParfPage[iIndex] * BIG_BLOCK_SIZE,
+				(ULONG)ausParfPage[iIndex] * BIG_BLOCK_SIZE,
 				BIG_BLOCK_SIZE)) {
 			break;
 		}
@@ -359,45 +536,45 @@ vGet6PapInfo(FILE *pFile, long lStartBlock,
 			if (iFodo <= 0) {
 				continue;
 			}
-			if (bGet6StyleInfo(iFodo, aucFpage, &tStyle)) {
-				lParmOffset = (long)ulGetLong(
-						iIndex2 * 4, aucFpage);
-				NO_DBG_HEX(lParmOffset);
-				tStyle.lFileOffset = lTextOffset2FileOffset(
-							lParmOffset);
-				vAdd2StyleInfoList(&tStyle);
-				(void)memset(&tStyle, 0, sizeof(tStyle));
-			}
 
 			iLen = 2 * (int)ucGetByte(iFodo, aucFpage);
+			
+			usIstd = (USHORT)ucGetByte(iFodo + 1, aucFpage);
+			vFillStyleFromStylesheet(usIstd, &tStyle);
+			vGet6StyleInfo(iFodo, aucFpage + 3, iLen - 3, &tStyle);
+			ulCharPos = ulGetLong(iIndex2 * 4, aucFpage);
+			NO_DBG_HEX(ulCharPos);
+			tStyle.ulFileOffset = ulCharPos2FileOffset(ulCharPos);
+			vAdd2StyleInfoList(&tStyle);
+
 			eRowInfo = eGet6RowInfo(iFodo,
 					aucFpage + 3, iLen - 3, &tRow);
 			switch(eRowInfo) {
 			case found_a_cell:
-				if (lParmOffsetFirst >= 0) {
+				if (ulCharPosFirst != CP_INVALID) {
 					break;
 				}
-				lParmOffsetFirst = (long)ulGetLong(
+				ulCharPosFirst = ulGetLong(
 						iIndex2 * 4, aucFpage);
-				NO_DBG_HEX(lParmOffsetFirst);
-				tRow.lTextOffsetStart = lParmOffsetFirst;
-				tRow.lFileOffsetStart = lTextOffset2FileOffset(
-							lParmOffsetFirst);
-				DBG_HEX_C(tRow.lFileOffsetStart < 0,
-							lParmOffsetFirst);
+				NO_DBG_HEX(ulCharPosFirst);
+				tRow.ulCharPosStart = ulCharPosFirst;
+				tRow.ulFileOffsetStart =
+					ulCharPos2FileOffset(ulCharPosFirst);
+				DBG_HEX_C(tRow.ulFileOffsetStart == FC_INVALID,
+							ulCharPosFirst);
 				break;
 			case found_end_of_row:
-				lParmOffsetLast = (long)ulGetLong(
+				ulCharPosLast = ulGetLong(
 						iIndex2 * 4, aucFpage);
-				NO_DBG_HEX(lParmOffsetLast);
-				tRow.lTextOffsetEnd = lParmOffsetLast;
-				tRow.lFileOffsetEnd = lTextOffset2FileOffset(
-							lParmOffsetLast);
-				DBG_HEX_C(tRow.lFileOffsetEnd < 0,
-							lParmOffsetLast);
+				NO_DBG_HEX(ulCharPosLast);
+				tRow.ulCharPosEnd = ulCharPosLast;
+				tRow.ulFileOffsetEnd = ulCharPos2FileOffset(
+							ulCharPosLast);
+				DBG_HEX_C(tRow.ulFileOffsetEnd == FC_INVALID,
+							ulCharPosLast);
 				vAdd2RowInfoList(&tRow);
 				(void)memset(&tRow, 0, sizeof(tRow));
-				lParmOffsetFirst = -1;
+				ulCharPosFirst = CP_INVALID;
 				break;
 			case found_nothing:
 				break;
@@ -415,36 +592,43 @@ vGet6PapInfo(FILE *pFile, long lStartBlock,
  * from a Word 6/7 file.
  * Returns TRUE when successful, otherwise FALSE
  */
-static BOOL
-bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
-		font_block_type *pFont)
+void
+vGet6FontInfo(int iFodo, USHORT usIstd,
+	const UCHAR *aucGrpprl, int iBytes, font_block_type *pFont)
 {
-	int	iBytes, iFodoOff, iInfoLen;
-	unsigned short	usTmp;
-	short		sTmp;
-	unsigned char	ucTmp;
+	long	lTmp;
+	int	iFodoOff, iInfoLen;
+	USHORT	usTmp;
+	short	sTmp;
+	UCHAR	ucTmp;
 
-	fail(iFodo <= 0 || aucFpage == NULL || pFont == NULL);
+	fail(iFodo < 0 || aucGrpprl == NULL || pFont == NULL);
 
-	iBytes = (int)ucGetByte(iFodo, aucFpage);
-	iFodoOff = 1;
-	if (iBytes < 1) {
-		return FALSE;
-	}
-	(void)memset(pFont, 0, sizeof(*pFont));
-	pFont->sFontsize = DEFAULT_FONT_SIZE;
+	iFodoOff = 0;
 	while (iBytes >= iFodoOff + 1) {
-		switch (ucGetByte(iFodo + iFodoOff, aucFpage)) {
+		switch (ucGetByte(iFodo + iFodoOff, aucGrpprl)) {
+		case  65:	/* fRMarkDel */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
+			if (ucTmp == 0) {
+				pFont->ucFontstyle &= ~FONT_MARKDEL;
+			} else {
+				pFont->ucFontstyle |= FONT_MARKDEL;
+			}
+			break;
 		case  80:	/* cIstd */
-			sTmp = (short)usGetWord(iFodo + iFodoOff + 1, aucFpage);
-			DBG_DEC(sTmp);
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			NO_DBG_DEC(usTmp);
                         break;
 		case  82:	/* cDefault */
 			pFont->ucFontstyle &= FONT_HIDDEN;
 			pFont->ucFontcolor = FONT_COLOR_DEFAULT;
 			break;
+		case  83:	/* cPlain */
+			DBG_MSG("83: cPlain");
+			vFillFontFromStylesheet(usIstd, pFont);
+			break;
 		case  85:	/* fBold */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_BOLD;
@@ -464,7 +648,7 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case  86:	/* fItalic */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_ITALIC;
@@ -484,7 +668,7 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case  87:	/* fStrike */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_STRIKE;
@@ -504,7 +688,7 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case  90:	/* fSmallCaps */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_SMALL_CAPITALS;
@@ -524,7 +708,7 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case  91:	/* fCaps */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_CAPITALS;
@@ -544,7 +728,7 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case  92:	/* fVanish */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_HIDDEN;
@@ -564,14 +748,18 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case  93:	/* cFtc */
-			usTmp = usGetWord(iFodo + iFodoOff + 1, aucFpage);
-			if (usTmp <= (unsigned short)UCHAR_MAX) {
-				pFont->ucFontnumber = (unsigned char)usTmp;
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			if (usTmp <= (USHORT)UCHAR_MAX) {
+				pFont->ucFontnumber = (UCHAR)usTmp;
+			} else {
+				pFont->ucFontnumber = 0;
 			}
 			break;
 		case  94:	/* cKul */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
-			if (ucTmp != 0 && ucTmp != 5) {
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
+			if (ucTmp == 0 || ucTmp == 5) {
+				pFont->ucFontstyle &= ~FONT_UNDERLINE;
+			} else {
 				NO_DBG_MSG("Underline text");
 				pFont->ucFontstyle |= FONT_UNDERLINE;
 				if (ucTmp == 6) {
@@ -580,27 +768,51 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
 				}
 			}
 			break;
+		case  95:	/* cHps, cHpsPos */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
+			if (ucTmp != 0) {
+				pFont->sFontsize = (short)ucTmp;
+			}
+			DBG_DEC(ucTmp);
+			break;
 		case  98:	/* cIco */
 			pFont->ucFontcolor =
-				ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+				ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			break;
 		case  99:	/* cHps */
-			usTmp = usGetWord(iFodo + iFodoOff + 1, aucFpage);
-			if (usTmp > (unsigned short)SHRT_MAX) {
+			usTmp = usGetWord(iFodo + iFodoOff + 1, aucGrpprl);
+			if (usTmp > (USHORT)SHRT_MAX) {
 				pFont->sFontsize = SHRT_MAX;
 			} else {
 				pFont->sFontsize = (short)usTmp;
 			}
 			break;
+		case 104:	/* cIss */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
+			ucTmp &= 0x07;
+			DBG_MSG_C(ucTmp == 1, "Superscript");
+			DBG_MSG_C(ucTmp == 2, "Subscript");
+			break;
+		case 106:	/* cHps */
+			sTmp = (short)usGetWord(
+					iFodo + iFodoOff + 1, aucGrpprl);
+			lTmp = (long)pFont->sFontsize + (long)sTmp;
+			if (lTmp < 8) {
+				pFont->sFontsize = 8;
+			} else if (lTmp > 32766) {
+				pFont->sFontsize = 32766;
+			} else {
+				pFont->sFontsize = (short)lTmp;
+			}
+			break;
 		default:
 			break;
 		}
-		iInfoLen = iGet6InfoLength(iFodo + iFodoOff, aucFpage);
+		iInfoLen = iGet6InfoLength(iFodo + iFodoOff, aucGrpprl);
 		fail(iInfoLen <= 0);
 		iFodoOff += iInfoLen;
 	}
-	return TRUE;
-} /* end of bGet6FontInfo */
+} /* end of vGet6FontInfo */
 
 /*
  * Fill the picture information block with information
@@ -608,39 +820,34 @@ bGet6FontInfo(int iFodo, const unsigned char *aucFpage,
  * Returns TRUE when successful, otherwise FALSE
  */
 static BOOL
-bGet6PicInfo(int iFodo, const unsigned char *aucFpage,
-		picture_block_type *pPicture)
+bGet6PicInfo(int iFodo,
+	const UCHAR *aucGrpprl, int iBytes, picture_block_type *pPicture)
 {
-	int	iBytes, iFodoOff, iInfoLen;
+	int	iFodoOff, iInfoLen;
 	BOOL	bFound;
-	unsigned char	ucTmp;
+	UCHAR	ucTmp;
 
-	fail(iFodo <= 0 || aucFpage == NULL || pPicture == NULL);
+	fail(iFodo < 0 || aucGrpprl == NULL || pPicture == NULL);
 
-	iBytes = (int)ucGetByte(iFodo, aucFpage);
-	iFodoOff = 1;
-	if (iBytes < 1) {
-		return FALSE;
-	}
+	iFodoOff = 0;
 	bFound = FALSE;
-	(void)memset(pPicture, 0, sizeof(*pPicture));
 	while (iBytes >= iFodoOff + 1) {
-		switch (ucGetByte(iFodo + iFodoOff, aucFpage)) {
-		case 0x44:
-			pPicture->lPictureOffset = (long)ulGetLong(
-					iFodo + iFodoOff + 2, aucFpage);
+		switch (ucGetByte(iFodo + iFodoOff, aucGrpprl)) {
+		case  68:	/* fcPic */
+			pPicture->ulPictureOffset = ulGetLong(
+					iFodo + iFodoOff + 2, aucGrpprl);
 			bFound = TRUE;
 			break;
-		case 0x47:
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+		case  71:	/* dttm */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			if (ucTmp == 0x01) {
 				/* Not a picture, but a form field */
 				return FALSE;
 			}
 			DBG_DEC_C(ucTmp != 0, ucTmp);
 			break;
-		case 0x4b:
-			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucFpage);
+		case  75:	/* fOle2 */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 1, aucGrpprl);
 			if (ucTmp == 0x01) {
 				/* Not a picture, but an OLE object */
 				return FALSE;
@@ -650,7 +857,7 @@ bGet6PicInfo(int iFodo, const unsigned char *aucFpage,
 		default:
 			break;
 		}
-		iInfoLen = iGet6InfoLength(iFodo + iFodoOff, aucFpage);
+		iInfoLen = iGet6InfoLength(iFodo + iFodoOff, aucGrpprl);
 		fail(iInfoLen <= 0);
 		iFodoOff += iInfoLen;
 	}
@@ -661,24 +868,24 @@ bGet6PicInfo(int iFodo, const unsigned char *aucFpage,
  * Build the lists with Character Information for Word 6/7 files
  */
 void
-vGet6ChrInfo(FILE *pFile, const pps_info_type *pPPS,
-	const long *alBBD, size_t tBBDLen,
-	const unsigned char *aucHeader)
+vGet6ChrInfo(FILE *pFile, ULONG ulStartBlock,
+	const ULONG *aulBBD, size_t tBBDLen, const UCHAR *aucHeader)
 {
 	font_block_type		tFont;
 	picture_block_type	tPicture;
-	unsigned short	*ausCharPage;
-	unsigned char	*aucBuffer;
-	long	lFileOffset, lParmOffset, lBeginCharInfo;
+	USHORT	*ausCharPage;
+	UCHAR	*aucBuffer;
+	ULONG	ulFileOffset, ulCharPos, ulBeginCharInfo;
 	size_t	tCharInfoLen, tOffset, tSize, tLenOld, tLen, tCharPageNum;
-	int	iIndex, iIndex2, iRun, iFodo;
-	unsigned short	usCharFirstPage, usCount;
-	unsigned char	aucFpage[BIG_BLOCK_SIZE];
+	int	iIndex, iIndex2, iRun, iFodo, iLen;
+	USHORT	usCharFirstPage, usCount, usIstd;
+	UCHAR	aucFpage[BIG_BLOCK_SIZE];
 
-	fail(pFile == NULL || pPPS == NULL || aucHeader == NULL);
-	fail(alBBD == NULL);
+	fail(pFile == NULL || aucHeader == NULL);
+	fail(ulStartBlock > MAX_BLOCKNUMBER && ulStartBlock != END_OF_CHAIN);
+	fail(aulBBD == NULL);
 
-	lBeginCharInfo = (long)ulGetLong(0xb8, aucHeader); /* fcPlcfbteChpx */
+	ulBeginCharInfo = ulGetLong(0xb8, aucHeader); /* fcPlcfbteChpx */
 	NO_DBG_HEX(lBeginCharInfo);
 	tCharInfoLen = (size_t)ulGetLong(0xbc, aucHeader); /* lcbPlcfbteChpx */
 	NO_DBG_DEC(tCharInfoLen);
@@ -688,15 +895,15 @@ vGet6ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 	}
 
 	aucBuffer = xmalloc(tCharInfoLen);
-	if (!bReadBuffer(pFile, pPPS->tWordDocument.lSb,
-			alBBD, tBBDLen, BIG_BLOCK_SIZE,
-			aucBuffer, lBeginCharInfo, tCharInfoLen)) {
+	if (!bReadBuffer(pFile, ulStartBlock,
+			aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+			aucBuffer, ulBeginCharInfo, tCharInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		return;
 	}
 
 	tLen = (tCharInfoLen - 4) / 6;
-	tSize = tLen * sizeof(unsigned short);
+	tSize = tLen * sizeof(USHORT);
 	ausCharPage = xmalloc(tSize);
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
@@ -706,16 +913,15 @@ vGet6ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 	}
 	DBG_HEX(ulGetLong(0, aucBuffer));
 	aucBuffer = xfree(aucBuffer);
-	aucBuffer = NULL;
-	tCharPageNum = (size_t)usGetWord(0x18e, aucHeader);
+	tCharPageNum = (size_t)usGetWord(0x18e, aucHeader); /* cpnBteChp */
 	DBG_DEC(tCharPageNum);
 	if (tLen < tCharPageNum) {
 		/* Replace CharPage by a longer version */
 		tLenOld = tLen;
-		usCharFirstPage = usGetWord(0x18a, aucHeader);
+		usCharFirstPage = usGetWord(0x18a, aucHeader); /* pnChrFirst */
 		DBG_DEC(usCharFirstPage);
 		tLen += tCharPageNum - 1;
-		tSize = tLen * sizeof(unsigned short);
+		tSize = tLen * sizeof(USHORT);
 		ausCharPage = xrealloc(ausCharPage, tSize);
 		/* Add new values */
 		usCount = usCharFirstPage + 1;
@@ -726,44 +932,46 @@ vGet6ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 		}
 	}
 
-	(void)memset(&tFont, 0, sizeof(tFont));
-	(void)memset(&tPicture, 0, sizeof(tPicture));
 	for (iIndex = 0; iIndex < (int)tLen; iIndex++) {
-		if (!bReadBuffer(pFile, pPPS->tWordDocument.lSb,
-				alBBD, tBBDLen, BIG_BLOCK_SIZE,
+		if (!bReadBuffer(pFile, ulStartBlock,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
 				aucFpage,
-				(long)ausCharPage[iIndex] * BIG_BLOCK_SIZE,
+				(ULONG)ausCharPage[iIndex] * BIG_BLOCK_SIZE,
 				BIG_BLOCK_SIZE)) {
 			break;
 		}
 		iRun = (int)ucGetByte(0x1ff, aucFpage);
 		NO_DBG_DEC(iRun);
 		for (iIndex2 = 0; iIndex2 < iRun; iIndex2++) {
-		  	lParmOffset = (long)ulGetLong(
-						iIndex2 * 4, aucFpage);
-			lFileOffset = lTextOffset2FileOffset(lParmOffset);
+		  	ulCharPos = ulGetLong(iIndex2 * 4, aucFpage);
+			ulFileOffset = ulCharPos2FileOffset(ulCharPos);
 			iFodo = 2 * (int)ucGetByte(
 				(iRun + 1) * 4 + iIndex2, aucFpage);
-			if (iFodo == 0) {
-				NO_DBG_HEX(lParmOffset);
-				vReset2FontInfoList(lFileOffset);
-			} else {
-				if (bGet6FontInfo(iFodo, aucFpage, &tFont)) {
-					tFont.lFileOffset = lFileOffset;
-					vAdd2FontInfoList(&tFont);
-				}
+
+			iLen = (int)ucGetByte(iFodo, aucFpage);
+
+			usIstd = usGetIstd(ulFileOffset);
+			vFillFontFromStylesheet(usIstd, &tFont);
+			if (iFodo != 0) {
+				vGet6FontInfo(iFodo, usIstd,
+					aucFpage + 1, iLen - 1, &tFont);
 			}
-			(void)memset(&tFont, 0, sizeof(tFont));
+			tFont.ulFileOffset = ulFileOffset;
+			vAdd2FontInfoList(&tFont);
+
 			if (iFodo <= 0) {
 				continue;
 			}
-			if (bGet6PicInfo(iFodo, aucFpage, &tPicture)) {
-				tPicture.lFileOffset = lFileOffset;
-				tPicture.lFileOffsetPicture =
-					lDataOffset2FileOffset(tPicture.lPictureOffset);
-				vAdd2PicInfoList(&tPicture);
-			}
+
 			(void)memset(&tPicture, 0, sizeof(tPicture));
+			if (bGet6PicInfo(iFodo, aucFpage + 1,
+						iLen - 1, &tPicture)) {
+				tPicture.ulFileOffset = ulFileOffset;
+				tPicture.ulFileOffsetPicture =
+					ulDataPos2FileOffset(
+						tPicture.ulPictureOffset);
+				vAdd2PictInfoList(&tPicture);
+			}
 		}
 	}
 	ausCharPage = xfree(ausCharPage);
