@@ -1,6 +1,6 @@
 /*
  * prop8.c
- * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * Read the property information from a MS Word 8, 9 or 10 file
@@ -19,12 +19,12 @@
  * iGet8InfoLength - the length of the information for Word 8/9/10 files
  */
 static int
-iGet8InfoLength(int iByteNbr, const unsigned char *aucFpage)
+iGet8InfoLength(int iByteNbr, const UCHAR *aucGrpprl)
 {
 	int	iTmp, iDel, iAdd;
-	unsigned short	usOpCode;
+	USHORT	usOpCode;
 
-	usOpCode = usGetWord(iByteNbr, aucFpage);
+	usOpCode = usGetWord(iByteNbr, aucGrpprl);
 
 	switch (usOpCode & 0xe000) {
 	case 0x0000: case 0x2000:
@@ -36,11 +36,11 @@ iGet8InfoLength(int iByteNbr, const unsigned char *aucFpage)
 	case 0x6000:
 		return 6;
 	case 0xc000:
-		iTmp = (int)ucGetByte(iByteNbr + 2, aucFpage);
+		iTmp = (int)ucGetByte(iByteNbr + 2, aucGrpprl);
 		if (usOpCode == 0xc615 && iTmp == 255) {
-			iDel = (int)ucGetByte(iByteNbr + 3, aucFpage);
+			iDel = (int)ucGetByte(iByteNbr + 3, aucGrpprl);
 			iAdd = (int)ucGetByte(
-					iByteNbr + 4 + iDel * 4, aucFpage);
+					iByteNbr + 4 + iDel * 4, aucGrpprl);
 			iTmp = 2 + iDel * 4 + iAdd * 3;
 		}
 		return 3 + iTmp;
@@ -51,12 +51,194 @@ iGet8InfoLength(int iByteNbr, const unsigned char *aucFpage)
 	}
 } /* end of iGet8InfoLength */
 
+/*
+ * Fill the section information block with information
+ * from a Word 8/9/10 file.
+ */
+static void
+vGet8SectionInfo(const UCHAR *aucGrpprl, size_t tBytes,
+		section_block_type *pSection)
+{
+	int	iFodoOff, iInfoLen, iSize, iIndex, iTmp;
+	USHORT	usCcol;
+	UCHAR	ucTmp;
+
+	fail(aucGrpprl == NULL || pSection == NULL);
+
+	iFodoOff = 0;
+	while (tBytes >= (size_t)iFodoOff + 2) {
+		iInfoLen = 0;
+		switch (usGetWord(iFodoOff, aucGrpprl)) {
+		case 0x3009:	/* bkc */
+			ucTmp = ucGetByte(iFodoOff + 2, aucGrpprl);
+			DBG_DEC(ucTmp);
+			pSection->bNewPage = ucTmp != 0 && ucTmp != 1;
+			break;
+		case 0x500b:	/* ccolM1 */
+			usCcol = 1 + usGetWord(iFodoOff + 2, aucGrpprl);
+			DBG_DEC(usCcol);
+			break;
+		case 0xd202:	/* olstAnm */
+			iSize = (int)ucGetByte(iFodoOff + 2, aucGrpprl);
+			DBG_DEC_C(iSize != 212, iSize);
+			for (iIndex = 0, iTmp = iFodoOff + 3;
+			     iIndex < 9 && iTmp < iFodoOff + 3 + iSize - 15;
+			     iIndex++, iTmp += 16) {
+				pSection->aucNFC[iIndex] =
+						ucGetByte(iTmp, aucGrpprl);
+				DBG_DEC(pSection->aucNFC[iIndex]);
+				ucTmp = ucGetByte(iTmp + 3, aucGrpprl);
+				DBG_HEX(ucTmp);
+				if ((ucTmp & BIT(2)) != 0) {
+					pSection->usNeedPrevLvl |=
+							(USHORT)BIT(iIndex);
+				}
+				if ((ucTmp & BIT(3)) != 0) {
+					pSection->usHangingIndent |=
+							(USHORT)BIT(iIndex);
+				}
+			}
+			DBG_HEX(pSection->usNeedPrevLvl);
+			DBG_HEX(pSection->usHangingIndent);
+			break;
+		default:
+			break;
+		}
+		if (iInfoLen <= 0) {
+			iInfoLen = iGet8InfoLength(iFodoOff, aucGrpprl);
+			fail(iInfoLen <= 0);
+		}
+		iFodoOff += iInfoLen;
+	}
+} /* end of vGet8SectionInfo */
+
+/*
+ * Build the lists with Section Property Information for Word 8/9/10 files
+ */
+void
+vGet8SepInfo(FILE *pFile, const pps_info_type *pPPS,
+	const ULONG *aulBBD, size_t tBBDLen,
+	const ULONG *aulSBD, size_t tSBDLen,
+	const UCHAR *aucHeader)
+{
+	section_block_type	tSection;
+	ULONG		*aulSectPage, *aulTextOffset;
+	const ULONG	*aulBlockDepot;
+	UCHAR	*aucBuffer, *aucFpage;
+	ULONG	ulBeginSectInfo;
+	ULONG	ulTableSize, ulTableStartBlock;
+	size_t	tSectInfoLen, tBlockDepotLen;
+	size_t	tBlockSize, tOffset, tSize, tLen, tBytes;
+	int	iIndex;
+	USHORT	usDocStatus;
+	UCHAR	aucTmp[2];
+
+	fail(pFile == NULL || pPPS == NULL || aucHeader == NULL);
+	fail(aulBBD == NULL || aulSBD == NULL);
+
+	ulBeginSectInfo = ulGetLong(0xca, aucHeader); /* fcPlcfsed */
+	NO_DBG_HEX(ulBeginSectInfo);
+	tSectInfoLen = (size_t)ulGetLong(0xce, aucHeader); /* lcbPlcfsed */
+	NO_DBG_DEC(tSectInfoLen);
+	if (tSectInfoLen < 4) {
+		DBG_DEC(tSectInfoLen);
+		return;
+	}
+
+	/* Use 0Table or 1Table? */
+	usDocStatus = usGetWord(0x0a, aucHeader);
+	if (usDocStatus & BIT(9)) {
+		ulTableStartBlock = pPPS->t1Table.ulSB;
+		ulTableSize = pPPS->t1Table.ulSize;
+	} else {
+		ulTableStartBlock = pPPS->t0Table.ulSB;
+		ulTableSize = pPPS->t0Table.ulSize;
+	}
+	DBG_DEC(ulTableStartBlock);
+	if (ulTableStartBlock == 0) {
+		DBG_MSG("No section information");
+		return;
+	}
+	DBG_HEX(ulTableSize);
+	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
+	  	/* Use the Small Block Depot */
+		aulBlockDepot = aulSBD;
+		tBlockDepotLen = tSBDLen;
+		tBlockSize = SMALL_BLOCK_SIZE;
+	} else {
+	  	/* Use the Big Block Depot */
+		aulBlockDepot = aulBBD;
+		tBlockDepotLen = tBBDLen;
+		tBlockSize = BIG_BLOCK_SIZE;
+	}
+	aucBuffer = xmalloc(tSectInfoLen);
+	if (!bReadBuffer(pFile, ulTableStartBlock,
+			aulBlockDepot, tBlockDepotLen, tBlockSize,
+			aucBuffer, ulBeginSectInfo, tSectInfoLen)) {
+		aucBuffer = xfree(aucBuffer);
+		return;
+	}
+	NO_DBG_PRINT_BLOCK(aucBuffer, tSectInfoLen);
+
+	/* Read the Section Descriptors */
+	tLen = (tSectInfoLen - 4) / 16;
+	tSize = tLen * sizeof(ULONG);
+	/* Save the section offsets */
+	aulTextOffset = xmalloc(tSize);
+	for (iIndex = 0, tOffset = 0;
+	     iIndex < (int)tLen;
+	     iIndex++, tOffset += 4) {
+		aulTextOffset[iIndex] = ulGetLong(tOffset, aucBuffer);
+	}
+	/* Save the Sepx offsets */
+	aulSectPage = xmalloc(tSize);
+	for (iIndex = 0, tOffset = (tLen + 1) * 4;
+	     iIndex < (int)tLen;
+	     iIndex++, tOffset += 12) {
+		 aulSectPage[iIndex] = ulGetLong(tOffset + 2, aucBuffer);
+		 NO_DBG_HEX(aulSectPage[iIndex]); /* fcSepx */
+	}
+	aucBuffer = xfree(aucBuffer);
+
+	/* Read the Section Properties */
+	for (iIndex = 0; iIndex < (int)tLen; iIndex++) {
+		if (aulSectPage[iIndex] == FC_INVALID) {
+			vDefault2SectionInfoList(aulTextOffset[iIndex]);
+			continue;
+		}
+		/* Get the number of bytes to read */
+		if (!bReadBuffer(pFile, pPPS->tWordDocument.ulSB,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+				aucTmp, aulSectPage[iIndex], 2)) {
+			continue;
+		}
+		tBytes = 2 + (size_t)usGetWord(0, aucTmp);
+		NO_DBG_DEC(tBytes);
+		/* Read the bytes */
+		aucFpage = xmalloc(tBytes);
+		if (!bReadBuffer(pFile, pPPS->tWordDocument.ulSB,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
+				aucFpage, aulSectPage[iIndex], tBytes)) {
+			aucFpage = xfree(aucFpage);
+			continue;
+		}
+		NO_DBG_PRINT_BLOCK(aucFpage, tBytes);
+		/* Process the bytes */
+		vGetDefaultSection(&tSection);
+		vGet8SectionInfo(aucFpage + 2, tBytes - 2, &tSection);
+		vAdd2SectionInfoList(&tSection, aulTextOffset[iIndex]);
+		aucFpage = xfree(aucFpage);
+	}
+	aulTextOffset = xfree(aulTextOffset);
+	aulSectPage = xfree(aulSectPage);
+} /* end of vGet8SepInfo */
+
 /*
  * Translate the rowinfo to a member of the row_info enumeration
  */
 row_info_enum
 eGet8RowInfo(int iFodo,
-	const unsigned char *aucGrpprl, int iBytes, row_block_type *pRow)
+	const UCHAR *aucGrpprl, int iBytes, row_block_type *pRow)
 {
 	int	iFodoOff, iInfoLen;
 	int	iIndex, iSize, iCol;
@@ -90,7 +272,7 @@ eGet8RowInfo(int iFodo,
 			}
 			break;
 		case 0xd608:	/* cDefTable */
-			iSize = (int)ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
+			iSize = (int)usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
 			if (iSize < 6 || iBytes < iFodoOff + 8) {
 				DBG_DEC(iSize);
 				DBG_DEC(iFodoOff);
@@ -106,9 +288,10 @@ eGet8RowInfo(int iFodo,
 				break;
 			}
 			if (iCol >= (int)elementsof(pRow->asColumnWidth)) {
+				DBG_DEC(iCol);
 				werr(1, "The number of columns is corrupt");
 			}
-			pRow->ucNumberOfColumns = (unsigned char)iCol;
+			pRow->ucNumberOfColumns = (UCHAR)iCol;
 			pRow->iColumnWidthSum = 0;
 			iPosPrev = (int)(short)usGetWord(
 					iFodo + iFodoOff + 5,
@@ -153,57 +336,48 @@ eGet8RowInfo(int iFodo,
 /*
  * Fill the style information block with information
  * from a Word 8/9/10 file.
- * Returns TRUE when successful, otherwise FALSE
  */
-static BOOL
-bGet8StyleInfo(int iFodo, const unsigned char *aucFpage,
-		style_block_type *pStyle)
+void
+vGet8StyleInfo(int iFodo,
+	const UCHAR *aucGrpprl, int iBytes, style_block_type *pStyle)
 {
-	int	iBytes, iFodoOff, iInfoLen;
-	int	iTmp, iDel, iAdd;
+	int	iFodoOff, iInfoLen;
+	int	iTmp, iDel, iAdd, iBefore;
+	USHORT	usOpCode, usIlfo, usTmp;
 	short	sTmp;
 
-	fail(iFodo <= 0 || aucFpage == NULL || pStyle == NULL);
+	fail(iFodo < 0 || aucGrpprl == NULL || pStyle == NULL);
 
-	iBytes = 2 * (int)ucGetByte(iFodo, aucFpage);
-	iFodoOff = 3;
-	if (iBytes == 0) {
-		iFodo++;
-		iBytes = 2 * (int)ucGetByte(iFodo, aucFpage);
-	}
-	NO_DBG_DEC(iBytes);
-	if (iBytes < 2) {
-		return FALSE;
-	}
-	(void)memset(pStyle, 0, sizeof(*pStyle));
-	sTmp = (short)usGetWord(iFodo + 1, aucFpage);
-	if (sTmp >= 0 && sTmp <= (short)UCHAR_MAX) {
-		pStyle->ucStyle = (unsigned char)sTmp;
-	}
-	NO_DBG_DEC(pStyle->ucStyle);
+	NO_DBG_DEC(pStyle->usIstd);
+
+	usIlfo = 0;
+	iFodoOff = 0;
 	while (iBytes >= iFodoOff + 2) {
 		iInfoLen = 0;
-		switch (usGetWord(iFodo + iFodoOff, aucFpage)) {
+		usOpCode = usGetWord(iFodo + iFodoOff, aucGrpprl);
+		switch (usOpCode) {
 		case 0x2403:	/* jc */
 			pStyle->ucAlignment = ucGetByte(
-					iFodo + iFodoOff + 2, aucFpage);
+					iFodo + iFodoOff + 2, aucGrpprl);
 			break;
 		case 0x260a:	/* ilvl */
 			pStyle->bInList = TRUE;
 			pStyle->ucListLevel =
-				ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+				ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(pStyle->ucListLevel);
 			break;
 		case 0x4600:	/* istd */
-			sTmp = (short)usGetWord(iFodo + iFodoOff + 2, aucFpage);
-			NO_DBG_DEC(sTmp);
-			NO_DBG_DEC(pStyle->ucStyle);
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			NO_DBG_DEC(usTmp);
 			break;
 		case 0x460b:	/* ilfo */
-			NO_DBG_DEC(usGetWord(iFodo + iFodoOff + 2, aucFpage));
+			pStyle->bInList = TRUE;
+			usIlfo = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			pStyle->bNumPause = usIlfo == 0;
 			break;
 		case 0x4610: /* Nest dxaLeft */
-			sTmp = (short)usGetWord(iFodo + iFodoOff + 2, aucFpage);
+			sTmp = (short)usGetWord(
+					iFodo + iFodoOff + 2, aucGrpprl);
 			pStyle->sLeftIndent += sTmp;
 			if (pStyle->sLeftIndent < 0) {
 				pStyle->sLeftIndent = 0;
@@ -212,20 +386,20 @@ bGet8StyleInfo(int iFodo, const unsigned char *aucFpage,
 			DBG_DEC(pStyle->sLeftIndent);
 			break;
 		case 0x6c0d:	/* ChgTabsPapx */
-			iTmp = (int)ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			iTmp = (int)ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			if (iTmp < 2) {
 				iInfoLen = 1;
 				break;
 			}
 			DBG_DEC(iTmp);
-			iDel = (int)ucGetByte(iFodo + iFodoOff + 3, aucFpage);
+			iDel = (int)ucGetByte(iFodo + iFodoOff + 3, aucGrpprl);
 			if (iTmp < 2 + 2 * iDel) {
 				iInfoLen = 1;
 				break;
 			}
 			DBG_DEC(iDel);
 			iAdd = (int)ucGetByte(
-				iFodo + iFodoOff + 4 + 2 * iDel, aucFpage);
+				iFodo + iFodoOff + 4 + 2 * iDel, aucGrpprl);
 			if (iTmp < 2 + 2 * iDel + 2 * iAdd) {
 				iInfoLen = 1;
 				break;
@@ -234,67 +408,97 @@ bGet8StyleInfo(int iFodo, const unsigned char *aucFpage,
 			break;
 		case 0x840e:	/* dxaRight */
 			pStyle->sRightIndent = (short)usGetWord(
-					iFodo + iFodoOff + 2, aucFpage);
+					iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(pStyle->sRightIndent);
 			break;
 		case 0x840f:	/* dxaLeft */
 			pStyle->sLeftIndent = (short)usGetWord(
-					iFodo + iFodoOff + 2, aucFpage);
+					iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(pStyle->sLeftIndent);
 			break;
+		case 0xa413:	/* dyaBefore */
+			pStyle->usBeforeIndent = usGetWord(
+					iFodo + iFodoOff + 2, aucGrpprl);
+			NO_DBG_DEC(pStyle->usBeforeIndent);
+			break;
+		case 0xa414:	/* dyaAfter */
+			pStyle->usAfterIndent = usGetWord(
+					iFodo + iFodoOff + 2, aucGrpprl);
+			NO_DBG_DEC(pStyle->usAfterIndent);
+			break;
 		case 0xc63e:	/* anld */
-			iTmp = (int)ucGetByte(
-					iFodo + iFodoOff + 2, aucFpage);
-			if (iTmp < 84) {
-				DBG_DEC(iTmp);
+#if 0
+			if (usIlfo != 2047) {
 				break;
 			}
-			pStyle->ucListType = ucGetByte(
-					iFodo + iFodoOff + 3, aucFpage);
+#endif
+			iTmp = (int)ucGetByte(
+					iFodo + iFodoOff + 2, aucGrpprl);
+			DBG_DEC_C(iTmp < 84, iTmp);
 			pStyle->bInList = TRUE;
-			pStyle->ucListCharacter = ucGetByte(
-					iFodo + iFodoOff + 23, aucFpage);
+			if (iTmp >= 1) {
+				pStyle->ucNFC = ucGetByte(
+					iFodo + iFodoOff + 3, aucGrpprl);
+			}
+			if (pStyle->ucNFC != LIST_BULLETS && iTmp >= 2) {
+				iBefore = (int)ucGetByte(
+					iFodo + iFodoOff + 4, aucGrpprl);
+			} else {
+				iBefore = 0;
+			}
+			if (iTmp >= 12) {
+				pStyle->usStartAt = usGetWord(
+					iFodo + iFodoOff + 13, aucGrpprl);
+			}
+			if (iTmp >= iBefore + 21) {
+				pStyle->ucListCharacter = ucGetByte(
+					iFodo + iFodoOff + iBefore + 23,
+					aucGrpprl);
+				NO_DBG_CHR(pStyle->ucListCharacter);
+			}
 			break;
 		default:
+			NO_DBG_HEX(usOpCode);
 			break;
 		}
 		if (iInfoLen <= 0) {
 			iInfoLen =
-				iGet8InfoLength(iFodo + iFodoOff, aucFpage);
+				iGet8InfoLength(iFodo + iFodoOff, aucGrpprl);
 			fail(iInfoLen <= 0);
 		}
 		iFodoOff += iInfoLen;
 	}
-	return TRUE;
-} /* end of bGet8StyleInfo */
+} /* end of vGet8StyleInfo */
 
 /*
  * Build the lists with Paragraph Information for Word 8/9/10 files
  */
 void
 vGet8PapInfo(FILE *pFile, const pps_info_type *pPPS,
-	const long *alBBD, size_t tBBDLen, const long *alSBD, size_t tSBDLen,
-	const unsigned char *aucHeader)
+	const ULONG *aulBBD, size_t tBBDLen,
+	const ULONG *aulSBD, size_t tSBDLen,
+	const UCHAR *aucHeader)
 {
 	row_block_type		tRow;
 	style_block_type	tStyle;
-	long		*alParfPage;
-	const long	*alBlockDepot;
-	unsigned char	*aucBuffer;
-	long	lParmOffset, lParmOffsetFirst, lParmOffsetLast;
-	long	lBeginParfInfo, lTableSize, lTableStartBlock;
+	ULONG		*aulParfPage;
+	const ULONG	*aulBlockDepot;
+	UCHAR	*aucBuffer;
+	ULONG	ulCharPos, ulCharPosFirst, ulCharPosLast;
+	ULONG	ulBeginParfInfo;
+	ULONG	ulTableSize, ulTableStartBlock;
 	size_t	tParfInfoLen, tBlockDepotLen;
 	size_t	tBlockSize, tOffset, tSize, tLen;
 	int	iIndex, iIndex2, iRun, iFodo, iLen;
 	row_info_enum	eRowInfo;
-	unsigned short	usDocStatus;
-	unsigned char	aucFpage[BIG_BLOCK_SIZE];
+	USHORT	usDocStatus, usIstd;
+	UCHAR	aucFpage[BIG_BLOCK_SIZE];
 
 	fail(pFile == NULL || pPPS == NULL || aucHeader == NULL);
-	fail(alBBD == NULL || alSBD == NULL);
+	fail(aulBBD == NULL || aulSBD == NULL);
 
-	lBeginParfInfo = (long)ulGetLong(0x102, aucHeader); /* fcPlcfbtePapx */
-	NO_DBG_HEX(lBeginParfInfo);
+	ulBeginParfInfo = ulGetLong(0x102, aucHeader); /* fcPlcfbtePapx */
+	NO_DBG_HEX(ulBeginParfInfo);
 	tParfInfoLen = (size_t)ulGetLong(0x106, aucHeader); /* lcbPlcfbtePapx */
 	NO_DBG_DEC(tParfInfoLen);
 	if (tParfInfoLen < 4) {
@@ -305,62 +509,59 @@ vGet8PapInfo(FILE *pFile, const pps_info_type *pPPS,
 	/* Use 0Table or 1Table? */
 	usDocStatus = usGetWord(0x0a, aucHeader);
 	if (usDocStatus & BIT(9)) {
-		lTableStartBlock = pPPS->t1Table.lSb;
-		lTableSize = pPPS->t1Table.lSize;
+		ulTableStartBlock = pPPS->t1Table.ulSB;
+		ulTableSize = pPPS->t1Table.ulSize;
 	} else {
-		lTableStartBlock = pPPS->t0Table.lSb;
-		lTableSize = pPPS->t0Table.lSize;
+		ulTableStartBlock = pPPS->t0Table.ulSB;
+		ulTableSize = pPPS->t0Table.ulSize;
 	}
-	DBG_DEC(lTableStartBlock);
-	if (lTableStartBlock < 0) {
-		DBG_DEC(lTableStartBlock);
+	DBG_DEC(ulTableStartBlock);
+	if (ulTableStartBlock == 0) {
 		DBG_MSG("No paragraph information");
 		return;
 	}
-	DBG_HEX(lTableSize);
-	if (lTableSize < MIN_SIZE_FOR_BBD_USE) {
+	DBG_HEX(ulTableSize);
+	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
 	  	/* Use the Small Block Depot */
-		alBlockDepot = alSBD;
+		aulBlockDepot = aulSBD;
 		tBlockDepotLen = tSBDLen;
 		tBlockSize = SMALL_BLOCK_SIZE;
 	} else {
 	  	/* Use the Big Block Depot */
-		alBlockDepot = alBBD;
+		aulBlockDepot = aulBBD;
 		tBlockDepotLen = tBBDLen;
 		tBlockSize = BIG_BLOCK_SIZE;
 	}
 	aucBuffer = xmalloc(tParfInfoLen);
-	if (!bReadBuffer(pFile, lTableStartBlock,
-			alBlockDepot, tBlockDepotLen, tBlockSize,
-			aucBuffer, lBeginParfInfo, tParfInfoLen)) {
+	if (!bReadBuffer(pFile, ulTableStartBlock,
+			aulBlockDepot, tBlockDepotLen, tBlockSize,
+			aucBuffer, ulBeginParfInfo, tParfInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		return;
 	}
 	NO_DBG_PRINT_BLOCK(aucBuffer, tParfInfoLen);
 
 	tLen = (tParfInfoLen / 4 - 1) / 2;
-	tSize = tLen * sizeof(long);
-	alParfPage = xmalloc(tSize);
+	tSize = tLen * sizeof(ULONG);
+	aulParfPage = xmalloc(tSize);
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 4) {
-		 alParfPage[iIndex] = (long)ulGetLong(tOffset, aucBuffer);
-		 NO_DBG_DEC(alParfPage[iIndex]);
+		 aulParfPage[iIndex] = ulGetLong(tOffset, aucBuffer);
+		 NO_DBG_DEC(aulParfPage[iIndex]);
 	}
 	DBG_HEX(ulGetLong(0, aucBuffer));
 	aucBuffer = xfree(aucBuffer);
-	aucBuffer = NULL;
 	NO_DBG_PRINT_BLOCK(aucHeader, HEADER_SIZE);
 
-	(void)memset(&tStyle, 0, sizeof(tStyle));
 	(void)memset(&tRow, 0, sizeof(tRow));
-	lParmOffsetFirst = -1;
+	ulCharPosFirst = CP_INVALID;
 	for (iIndex = 0; iIndex < (int)tLen; iIndex++) {
-		fail(alParfPage[iIndex] > LONG_MAX / BIG_BLOCK_SIZE);
-		if (!bReadBuffer(pFile, pPPS->tWordDocument.lSb,
-				alBBD, tBBDLen, BIG_BLOCK_SIZE,
+		fail(aulParfPage[iIndex] > ULONG_MAX / BIG_BLOCK_SIZE);
+		if (!bReadBuffer(pFile, pPPS->tWordDocument.ulSB,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
 				aucFpage,
-				alParfPage[iIndex] * BIG_BLOCK_SIZE,
+				aulParfPage[iIndex] * BIG_BLOCK_SIZE,
 				BIG_BLOCK_SIZE)) {
 			break;
 		}
@@ -375,49 +576,49 @@ vGet8PapInfo(FILE *pFile, const pps_info_type *pPPS,
 				continue;
 			}
 
-			if (bGet8StyleInfo(iFodo, aucFpage, &tStyle)) {
-				lParmOffset = (long)ulGetLong(
-						iIndex2 * 4, aucFpage);
-				NO_DBG_HEX(lParmOffset);
-				tStyle.lFileOffset = lTextOffset2FileOffset(
-							lParmOffset);
-				vAdd2StyleInfoList(&tStyle);
-				(void)memset(&tStyle, 0, sizeof(tStyle));
-			}
-
 			iLen = 2 * (int)ucGetByte(iFodo, aucFpage);
 			if (iLen == 0) {
 				iFodo++;
 				iLen = 2 * (int)ucGetByte(iFodo, aucFpage);
 			}
+
+			usIstd = usGetWord(iFodo + 1, aucFpage);
+			vFillStyleFromStylesheet(usIstd, &tStyle);
+			vGet8StyleInfo(iFodo, aucFpage + 3, iLen - 3, &tStyle);
+			ulCharPos = ulGetLong(iIndex2 * 4, aucFpage);
+			NO_DBG_HEX(ulCharPos);
+			tStyle.ulFileOffset = ulCharPos2FileOffset(ulCharPos);
+			vAdd2StyleInfoList(&tStyle);
+
 			eRowInfo = eGet8RowInfo(iFodo,
 					aucFpage + 3, iLen - 3, &tRow);
 			switch (eRowInfo) {
 			case found_a_cell:
-				if (lParmOffsetFirst >= 0) {
+				if (ulCharPosFirst != CP_INVALID) {
 					break;
 				}
-				lParmOffsetFirst = (long)ulGetLong(
+				ulCharPosFirst = ulGetLong(
 						iIndex2 * 4, aucFpage);
-				NO_DBG_HEX(lParmOffsetFirst);
-				tRow.lTextOffsetStart = lParmOffsetFirst;
-				tRow.lFileOffsetStart = lTextOffset2FileOffset(
-							lParmOffsetFirst);
-				DBG_HEX_C(tRow.lFileOffsetStart < 0,
-							lParmOffsetFirst);
+				NO_DBG_HEX(ulCharPosFirst);
+				tRow.ulCharPosStart = ulCharPosFirst;
+				tRow.ulFileOffsetStart =
+					ulCharPos2FileOffset(ulCharPosFirst);
+				NO_DBG_HEX_C(
+					tRow.ulFileOffsetStart == FC_INVALID,
+					ulCharPosFirst);
 				break;
 			case found_end_of_row:
-				lParmOffsetLast = (long)ulGetLong(
+				ulCharPosLast = ulGetLong(
 						iIndex2 * 4, aucFpage);
-				NO_DBG_HEX(lParmOffsetLast);
-				tRow.lTextOffsetEnd = lParmOffsetLast;
-				tRow.lFileOffsetEnd = lTextOffset2FileOffset(
-							lParmOffsetLast);
-				DBG_HEX_C(tRow.lFileOffsetEnd < 0,
-							lParmOffsetLast);
+				NO_DBG_HEX(ulCharPosLast);
+				tRow.ulCharPosEnd = ulCharPosLast;
+				tRow.ulFileOffsetEnd =
+					ulCharPos2FileOffset(ulCharPosLast);
+				NO_DBG_HEX_C(tRow.ulFileOffsetEnd == FC_INVALID,
+							ulCharPosLast);
 				vAdd2RowInfoList(&tRow);
 				(void)memset(&tRow, 0, sizeof(tRow));
-				lParmOffsetFirst = -1;
+				ulCharPosFirst = CP_INVALID;
 				break;
 			case found_nothing:
 				break;
@@ -427,41 +628,38 @@ vGet8PapInfo(FILE *pFile, const pps_info_type *pPPS,
 			}
 		}
 	}
-	alParfPage = xfree(alParfPage);
+	aulParfPage = xfree(aulParfPage);
 } /* end of vGet8PapInfo */
 
 /*
  * Fill the font information block with information
  * from a Word 8/9/10 file.
- * Returns TRUE when successful, otherwise FALSE
  */
-static BOOL
-bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
-		font_block_type *pFont)
+void
+vGet8FontInfo(int iFodo, USHORT usIstd,
+	const UCHAR *aucGrpprl, int iBytes, font_block_type *pFont)
 {
-	int	iBytes, iFodoOff, iInfoLen;
-	unsigned short	usTmp;
-	short		sTmp;
-	unsigned char	ucTmp;
+	long	lTmp;
+	int	iFodoOff, iInfoLen;
+	USHORT	usTmp;
+	short	sTmp;
+	UCHAR	ucTmp;
 
-	fail(iFodo <= 0 || aucFpage == NULL || pFont == NULL);
+	fail(iFodo < 0 || aucGrpprl == NULL || pFont == NULL);
 
-	iBytes = (int)ucGetByte(iFodo, aucFpage);
-	iFodoOff = 1;
-	if (iBytes == 0) {
-		iFodo++;
-		iBytes = 2 * (int)ucGetByte(iFodo, aucFpage);
-	}
-	NO_DBG_DEC(iBytes);
-	if (iBytes < 2) {
-		return FALSE;
-	}
-	(void)memset(pFont, 0, sizeof(*pFont));
-	pFont->sFontsize = DEFAULT_FONT_SIZE;
+	iFodoOff = 0;
 	while (iBytes >= iFodoOff + 2) {
-		switch (usGetWord(iFodo + iFodoOff, aucFpage)) {
+		switch (usGetWord(iFodo + iFodoOff, aucGrpprl)) {
+		case 0x0800:	/* fRMarkDel */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
+			if (ucTmp == 0) {
+				pFont->ucFontstyle &= ~FONT_MARKDEL;
+			} else {
+				pFont->ucFontstyle |= FONT_MARKDEL;
+			}
+			break;
 		case 0x0835:	/* fBold */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_BOLD;
@@ -481,7 +679,7 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case 0x0836:	/* fItalic */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_ITALIC;
@@ -501,7 +699,7 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case 0x0837:	/* fStrike */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_STRIKE;
@@ -521,7 +719,7 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case 0x083a:	/* fSmallCaps */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_SMALL_CAPITALS;
@@ -541,7 +739,7 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case 0x083b:	/* fCaps */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_CAPITALS;
@@ -561,7 +759,7 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			}
 			break;
 		case 0x083c:	/* fVanish */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			switch (ucTmp) {
 			case   0:	/* Unset */
 				pFont->ucFontstyle &= ~FONT_HIDDEN;
@@ -584,9 +782,15 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			pFont->ucFontstyle &= FONT_HIDDEN;
 			pFont->ucFontcolor = FONT_COLOR_DEFAULT;
 			break;
+		case 0x2a33:	/* cPlain */
+			DBG_MSG("2a33: cPlain");
+			vFillFontFromStylesheet(usIstd, pFont);
+			break;
 		case 0x2a3e:	/* cKul */
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
-			if (ucTmp != 0 && ucTmp != 5) {
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
+			if (ucTmp == 0 || ucTmp == 5) {
+				pFont->ucFontstyle &= ~FONT_UNDERLINE;
+			} else {
 				NO_DBG_MSG("Underline text");
 				pFont->ucFontstyle |= FONT_UNDERLINE;
 				if (ucTmp == 6) {
@@ -597,16 +801,22 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			break;
 		case 0x2a42:	/* cIco */
 			pFont->ucFontcolor =
-				ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+				ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			NO_DBG_DEC(pFont->ucFontcolor);
 			break;
+		case 0x2a48:	/* cIss */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
+			ucTmp &= 0x07;
+			DBG_MSG_C(ucTmp == 1, "Superscript");
+			DBG_MSG_C(ucTmp == 2, "Subscript");
+			break;
 		case 0x4a30:	/* cIstd */
-			sTmp = (short)usGetWord(iFodo + iFodoOff + 2, aucFpage);
-			DBG_DEC(sTmp);
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			NO_DBG_DEC(usTmp);
 			break;
 		case 0x4a43:	/* cHps */
-			usTmp = usGetWord(iFodo + iFodoOff + 2, aucFpage);
-			if (usTmp > (unsigned short)SHRT_MAX) {
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			if (usTmp > (USHORT)SHRT_MAX) {
 				pFont->sFontsize = SHRT_MAX;
 			} else {
 				pFont->sFontsize = (short)usTmp;
@@ -614,20 +824,40 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
 			NO_DBG_DEC(pFont->sFontsize);
 			break;
 		case 0x4a51:	/* cFtc */
-			usTmp = usGetWord(iFodo + iFodoOff + 2, aucFpage);
-			if (usTmp <= (unsigned short)UCHAR_MAX) {
-				pFont->ucFontnumber = (unsigned char)usTmp;
+			usTmp = usGetWord(iFodo + iFodoOff + 2, aucGrpprl);
+			if (usTmp <= (USHORT)UCHAR_MAX) {
+				pFont->ucFontnumber = (UCHAR)usTmp;
+			} else {
+				pFont->ucFontnumber = 0;
 			}
 			break;
+		case 0xca4a:	/* cHps */
+			sTmp = (short)usGetWord(
+					iFodo + iFodoOff + 2, aucGrpprl);
+			lTmp = (long)pFont->sFontsize + (long)sTmp;
+			if (lTmp < 8) {
+				pFont->sFontsize = 8;
+			} else if (lTmp > 32766) {
+				pFont->sFontsize = 32766;
+			} else {
+				pFont->sFontsize = (short)lTmp;
+			}
+			break;
+		case 0xea3f:	/* cHps, cHpsPos */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
+			if (ucTmp != 0) {
+				pFont->sFontsize = (short)ucTmp;
+			}
+			DBG_DEC(ucTmp);
+			break;
 		default:
 			break;
 		}
-		iInfoLen = iGet8InfoLength(iFodo + iFodoOff, aucFpage);
+		iInfoLen = iGet8InfoLength(iFodo + iFodoOff, aucGrpprl);
 		fail(iInfoLen <= 0);
 		iFodoOff += iInfoLen;
 	}
-	return TRUE;
-} /* end of bGet8FontInfo */
+} /* end of vGet8FontInfo */
 
 /*
  * Fill the picture information block with information
@@ -635,54 +865,44 @@ bGet8FontInfo(int iFodo, const unsigned char *aucFpage,
  * Returns TRUE when successful, otherwise FALSE
  */
 static BOOL
-bGet8PicInfo(int iFodo, const unsigned char *aucFpage,
-		picture_block_type *pPicture)
+bGet8PicInfo(int iFodo,
+	const UCHAR *aucGrpprl, int iBytes, picture_block_type *pPicture)
 {
-	int	iBytes, iFodoOff, iInfoLen;
+	int	iFodoOff, iInfoLen;
 	BOOL	bFound;
-	unsigned char	ucTmp;
+	UCHAR	ucTmp;
 
-	fail(iFodo <= 0 || aucFpage == NULL || pPicture == NULL);
+	fail(iFodo <= 0 || aucGrpprl == NULL || pPicture == NULL);
 
-	iBytes = (int)ucGetByte(iFodo, aucFpage);
-	iFodoOff = 1;
-	if (iBytes == 0) {
-		iFodo++;
-		iBytes = 2 * (int)ucGetByte(iFodo, aucFpage);
-	}
-	NO_DBG_DEC(iBytes);
-	if (iBytes < 2) {
-		return FALSE;
-	}
+	iFodoOff = 0;
 	bFound = FALSE;
-	(void)memset(pPicture, 0, sizeof(*pPicture));
 	while (iBytes >= iFodoOff + 2) {
-		switch (usGetWord(iFodo + iFodoOff, aucFpage)) {
-		case 0x0806:
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+		switch (usGetWord(iFodo + iFodoOff, aucGrpprl)) {
+		case 0x0806:	/* fData */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			if (ucTmp == 0x01) {
 				/* Not a picture, but a form field */
 				return FALSE;
 			}
 			DBG_DEC_C(ucTmp != 0, ucTmp);
 			break;
-		case 0x080a:
-			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucFpage);
+		case 0x080a:	/* fOle2 */
+			ucTmp = ucGetByte(iFodo + iFodoOff + 2, aucGrpprl);
 			if (ucTmp == 0x01) {
 				/* Not a picture, but an OLE object */
 				return FALSE;
 			}
 			DBG_DEC_C(ucTmp != 0, ucTmp);
 			break;
-		case 0x6a03:
-			pPicture->lPictureOffset = (long)ulGetLong(
-					iFodo + iFodoOff + 2, aucFpage);
+		case 0x6a03:	/* fcPic */
+			pPicture->ulPictureOffset = ulGetLong(
+					iFodo + iFodoOff + 2, aucGrpprl);
 			bFound = TRUE;
 			break;
 		default:
 			break;
 		}
-		iInfoLen = iGet8InfoLength(iFodo + iFodoOff, aucFpage);
+		iInfoLen = iGet8InfoLength(iFodo + iFodoOff, aucGrpprl);
 		fail(iInfoLen <= 0);
 		iFodoOff += iInfoLen;
 	}
@@ -694,27 +914,28 @@ bGet8PicInfo(int iFodo, const unsigned char *aucFpage,
  */
 void
 vGet8ChrInfo(FILE *pFile, const pps_info_type *pPPS,
-	const long *alBBD, size_t tBBDLen, const long *alSBD, size_t tSBDLen,
-	const unsigned char *aucHeader)
+	const ULONG *aulBBD, size_t tBBDLen,
+	const ULONG *aulSBD, size_t tSBDLen,
+	const UCHAR *aucHeader)
 {
 	font_block_type		tFont;
 	picture_block_type	tPicture;
-	long		*alCharPage;
-	const long	*alBlockDepot;
-	unsigned char	*aucBuffer;
-	long	lFileOffset, lParmOffset, lBeginCharInfo;
-	long	lTableSize, lTableStartBlock;
+	ULONG		*aulCharPage;
+	const ULONG	*aulBlockDepot;
+	UCHAR	*aucBuffer;
+	ULONG	ulFileOffset, ulCharPos, ulBeginCharInfo;
+	ULONG	ulTableSize, ulTableStartBlock;
 	size_t	tCharInfoLen, tBlockDepotLen;
 	size_t	tOffset, tBlockSize, tSize, tLen;
-	int	iIndex, iIndex2, iRun, iFodo;
-	unsigned short	usDocStatus;
-	unsigned char	aucFpage[BIG_BLOCK_SIZE];
+	int	iIndex, iIndex2, iRun, iFodo, iLen;
+	USHORT	usDocStatus, usIstd;
+	UCHAR	aucFpage[BIG_BLOCK_SIZE];
 
 	fail(pFile == NULL || pPPS == NULL || aucHeader == NULL);
-	fail(alBBD == NULL || alSBD == NULL);
+	fail(aulBBD == NULL || aulSBD == NULL);
 
-	lBeginCharInfo = (long)ulGetLong(0xfa, aucHeader); /* fcPlcfbteChpx */
-	NO_DBG_HEX(lBeginCharInfo);
+	ulBeginCharInfo = ulGetLong(0xfa, aucHeader); /* fcPlcfbteChpx */
+	NO_DBG_HEX(ulBeginCharInfo);
 	tCharInfoLen = (size_t)ulGetLong(0xfe, aucHeader); /* lcbPlcfbteChpx */
 	NO_DBG_DEC(tCharInfoLen);
 	if (tCharInfoLen < 4) {
@@ -725,60 +946,57 @@ vGet8ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 	/* Use 0Table or 1Table? */
 	usDocStatus = usGetWord(0x0a, aucHeader);
 	if (usDocStatus & BIT(9)) {
-		lTableStartBlock = pPPS->t1Table.lSb;
-		lTableSize = pPPS->t1Table.lSize;
+		ulTableStartBlock = pPPS->t1Table.ulSB;
+		ulTableSize = pPPS->t1Table.ulSize;
 	} else {
-		lTableStartBlock = pPPS->t0Table.lSb;
-		lTableSize = pPPS->t0Table.lSize;
+		ulTableStartBlock = pPPS->t0Table.ulSB;
+		ulTableSize = pPPS->t0Table.ulSize;
 	}
-	DBG_DEC(lTableStartBlock);
-	if (lTableStartBlock < 0) {
-		DBG_DEC(lTableStartBlock);
+	DBG_DEC(ulTableStartBlock);
+	if (ulTableStartBlock == 0) {
 		DBG_MSG("No character information");
 		return;
 	}
-	DBG_HEX(lTableSize);
-	if (lTableSize < MIN_SIZE_FOR_BBD_USE) {
+	DBG_HEX(ulTableSize);
+	if (ulTableSize < MIN_SIZE_FOR_BBD_USE) {
 	  	/* Use the Small Block Depot */
-		alBlockDepot = alSBD;
+		aulBlockDepot = aulSBD;
 		tBlockDepotLen = tSBDLen;
 		tBlockSize = SMALL_BLOCK_SIZE;
 	} else {
 	  	/* Use the Big Block Depot */
-		alBlockDepot = alBBD;
+		aulBlockDepot = aulBBD;
 		tBlockDepotLen = tBBDLen;
 		tBlockSize = BIG_BLOCK_SIZE;
 	}
 	aucBuffer = xmalloc(tCharInfoLen);
-	if (!bReadBuffer(pFile, lTableStartBlock,
-			alBlockDepot, tBlockDepotLen, tBlockSize,
-			aucBuffer, lBeginCharInfo, tCharInfoLen)) {
+	if (!bReadBuffer(pFile, ulTableStartBlock,
+			aulBlockDepot, tBlockDepotLen, tBlockSize,
+			aucBuffer, ulBeginCharInfo, tCharInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		return;
 	}
 	NO_DBG_PRINT_BLOCK(aucBuffer, tCharInfoLen);
 
 	tLen = (tCharInfoLen / 4 - 1) / 2;
-	tSize = tLen * sizeof(long);
-	alCharPage = xmalloc(tSize);
+	tSize = tLen * sizeof(ULONG);
+	aulCharPage = xmalloc(tSize);
 	for (iIndex = 0, tOffset = (tLen + 1) * 4;
 	     iIndex < (int)tLen;
 	     iIndex++, tOffset += 4) {
-		 alCharPage[iIndex] = (long)ulGetLong(tOffset, aucBuffer);
-		 NO_DBG_DEC(alCharPage[iIndex]);
+		 aulCharPage[iIndex] = ulGetLong(tOffset, aucBuffer);
+		 NO_DBG_DEC(aulCharPage[iIndex]);
 	}
 	DBG_HEX(ulGetLong(0, aucBuffer));
 	aucBuffer = xfree(aucBuffer);
-	aucBuffer = NULL;
 	NO_DBG_PRINT_BLOCK(aucHeader, HEADER_SIZE);
 
-	(void)memset(&tFont, 0, sizeof(tFont));
 	for (iIndex = 0; iIndex < (int)tLen; iIndex++) {
-		fail(alCharPage[iIndex] > LONG_MAX / BIG_BLOCK_SIZE);
-		if (!bReadBuffer(pFile, pPPS->tWordDocument.lSb,
-				alBBD, tBBDLen, BIG_BLOCK_SIZE,
+		fail(aulCharPage[iIndex] > ULONG_MAX / BIG_BLOCK_SIZE);
+		if (!bReadBuffer(pFile, pPPS->tWordDocument.ulSB,
+				aulBBD, tBBDLen, BIG_BLOCK_SIZE,
 				aucFpage,
-				alCharPage[iIndex] * BIG_BLOCK_SIZE,
+				aulCharPage[iIndex] * BIG_BLOCK_SIZE,
 				BIG_BLOCK_SIZE)) {
 			break;
 		}
@@ -786,33 +1004,36 @@ vGet8ChrInfo(FILE *pFile, const pps_info_type *pPPS,
 		iRun = (int)ucGetByte(0x1ff, aucFpage);
 		NO_DBG_DEC(iRun);
 		for (iIndex2 = 0; iIndex2 < iRun; iIndex2++) {
-			lParmOffset = (long)ulGetLong(
-						iIndex2 * 4, aucFpage);
-			lFileOffset = lTextOffset2FileOffset(lParmOffset);
+			ulCharPos = ulGetLong(iIndex2 * 4, aucFpage);
+			ulFileOffset = ulCharPos2FileOffset(ulCharPos);
 			iFodo = 2 * (int)ucGetByte(
 				(iRun + 1) * 4 + iIndex2, aucFpage);
-			if (iFodo == 0) {
-				NO_DBG_HEX(lParmOffset);
-				vReset2FontInfoList(lFileOffset);
-			} else {
-				if (bGet8FontInfo(iFodo, aucFpage, &tFont)) {
-					tFont.lFileOffset = lFileOffset;
-					vAdd2FontInfoList(&tFont);
-				}
+
+			iLen = (int)ucGetByte(iFodo, aucFpage);
+
+			usIstd = usGetIstd(ulFileOffset);
+			vFillFontFromStylesheet(usIstd, &tFont);
+			if (iFodo != 0) {
+				vGet8FontInfo(iFodo, usIstd,
+					aucFpage + 1, iLen - 1, &tFont);
 			}
-			(void)memset(&tFont, 0, sizeof(tFont));
+			tFont.ulFileOffset = ulFileOffset;
+			vAdd2FontInfoList(&tFont);
+
 			if (iFodo <= 0) {
 				continue;
 			}
-			if (bGet8PicInfo(iFodo, aucFpage, &tPicture)) {
-				tPicture.lFileOffset = lFileOffset;
-				tPicture.lFileOffsetPicture =
-					lDataOffset2FileOffset(
-						tPicture.lPictureOffset);
-				vAdd2PicInfoList(&tPicture);
-			}
+
 			(void)memset(&tPicture, 0, sizeof(tPicture));
+			if (bGet8PicInfo(iFodo, aucFpage + 1,
+						iLen - 1, &tPicture)) {
+				tPicture.ulFileOffset = ulFileOffset;
+				tPicture.ulFileOffsetPicture =
+					ulDataPos2FileOffset(
+						tPicture.ulPictureOffset);
+				vAdd2PictInfoList(&tPicture);
+			}
 		}
 	}
-	alCharPage = xfree(alCharPage);
+	aulCharPage = xfree(aulCharPage);
 } /* end of vGet8ChrInfo */
