@@ -1,6 +1,6 @@
 /*
  * word2text.c
- * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * MS Word to text functions
@@ -45,6 +45,9 @@ static int	iCurrPct, iPrevPct;
 static int	iWordVersion = -1;
 /* Special treatment for files from Word 6 on an Apple Macintosh */
 static BOOL	bWord6MacFile = FALSE;
+/* Section Information */
+static const section_block_type	*pSection = NULL;
+static const section_block_type	*pSectionNext = NULL;
 /* All the (command line) options */
 static options_type	tOptions;
 /* Needed for reading a complete table row */
@@ -53,14 +56,20 @@ static BOOL	bStartRow = FALSE;
 static BOOL	bEndRowNorm = FALSE;
 static BOOL	bEndRowFast = FALSE;
 static BOOL	bIsTableRow = FALSE;
+/* Index of the next style and font information */
+static USHORT	usIstdNext = ISTD_NORMAL;
 /* Needed for finding the start of a style */
 static const style_block_type	*pStyleInfo = NULL;
+static style_block_type		tStyleNext;
 static BOOL	bStartStyle = FALSE;
+static BOOL	bStartStyleNext = FALSE;
 /* Needed for finding the start of a font */
 static const font_block_type	*pFontInfo = NULL;
+static font_block_type		tFontNext;
 static BOOL	bStartFont = FALSE;
+static BOOL	bStartFontNext = FALSE;
 /* Needed for finding an image */
-static long	lFileOffsetImage = -1;
+static ULONG	ulFileOffsetImage = FC_INVALID;
 
 
 /*
@@ -85,19 +94,19 @@ vUpdateCounters(void)
 static BOOL
 bOutputContainsText(output_type *pAnchor)
 {
-	output_type	*pTmp;
+	output_type	*pCurr;
 	int	iIndex;
 
 	fail(pAnchor == NULL);
 
-	for (pTmp = pAnchor; pTmp != NULL; pTmp = pTmp->pNext) {
-		fail(pTmp->lStringWidth < 0);
-		for (iIndex = 0; iIndex < pTmp->iNextFree; iIndex++) {
-			if (isspace((int)pTmp->szStorage[iIndex])) {
+	for (pCurr = pAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
+		fail(pCurr->lStringWidth < 0);
+		for (iIndex = 0; iIndex < pCurr->iNextFree; iIndex++) {
+			if (isspace((int)pCurr->szStorage[iIndex])) {
 				continue;
 			}
 #if defined(DEBUG)
-			if (pTmp->szStorage[iIndex] == FILLER_CHAR) {
+			if (pCurr->szStorage[iIndex] == FILLER_CHAR) {
 				continue;
 			}
 #endif /* DEBUG */
@@ -113,13 +122,14 @@ bOutputContainsText(output_type *pAnchor)
 static long
 lTotalStringWidth(output_type *pAnchor)
 {
-	output_type	*pTmp;
+	output_type	*pCurr;
 	long		lTotal;
 
 	lTotal = 0;
-	for (pTmp = pAnchor; pTmp != NULL; pTmp = pTmp->pNext) {
-		fail(pTmp->lStringWidth < 0);
-		lTotal += pTmp->lStringWidth;
+	for (pCurr = pAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
+		DBG_DEC_C(pCurr->lStringWidth < 0, pCurr->lStringWidth);
+		fail(pCurr->lStringWidth < 0);
+		lTotal += pCurr->lStringWidth;
 	}
 	return lTotal;
 } /* end of lTotalStringWidth */
@@ -128,7 +138,7 @@ lTotalStringWidth(output_type *pAnchor)
  * vStoreByte - store one byte
  */
 static void
-vStoreByte(unsigned char ucChar, output_type *pOutput)
+vStoreByte(UCHAR ucChar, output_type *pOutput)
 {
 	fail(pOutput == NULL);
 
@@ -136,6 +146,7 @@ vStoreByte(unsigned char ucChar, output_type *pOutput)
 		pOutput->szStorage[pOutput->iNextFree] = '\0';
 		return;
 	}
+
 	while (pOutput->iNextFree + 2 > (int)pOutput->tStorageSize) {
 		pOutput->tStorageSize += EXTENTION_SIZE;
 		pOutput->szStorage = xrealloc(pOutput->szStorage,
@@ -150,35 +161,37 @@ vStoreByte(unsigned char ucChar, output_type *pOutput)
  * vStoreCharacter - store one character
  */
 static void
-vStoreCharacter(unsigned long ulChar, output_type *pOutput)
+vStoreCharacter(ULONG ulChar, output_type *pOutput)
 {
 	int	iLen;
 
 	fail(pOutput == NULL);
 
 	if (tOptions.eEncoding == encoding_utf8) {
+		DBG_HEX_C(ulChar > 0xffff, ulChar);
 		fail(ulChar > 0xffff);
 		if (ulChar < 0x80) {
-			vStoreByte((unsigned char)ulChar, pOutput);
+			vStoreByte((UCHAR)ulChar, pOutput);
 			iLen = 1;
 		} else if (ulChar < 0x800) {
-			vStoreByte((unsigned char)(0xc0 | ulChar >> 6),
+			vStoreByte((UCHAR)(0xc0 | ulChar >> 6),
 								pOutput);
-			vStoreByte((unsigned char)(0x80 | (ulChar & 0x3f)),
+			vStoreByte((UCHAR)(0x80 | (ulChar & 0x3f)),
 								pOutput);
 			iLen = 2;
 		} else {
-			vStoreByte((unsigned char)(0xe0 | ulChar >> 12),
+			vStoreByte((UCHAR)(0xe0 | ulChar >> 12),
 								pOutput);
-			vStoreByte((unsigned char)(0x80 | (ulChar >> 6 & 0x3f)),
+			vStoreByte((UCHAR)(0x80 | (ulChar >> 6 & 0x3f)),
 								pOutput);
-			vStoreByte((unsigned char)(0x80 | (ulChar & 0x3f)),
+			vStoreByte((UCHAR)(0x80 | (ulChar & 0x3f)),
 								pOutput);
 			iLen = 3;
 		}
 	} else {
+		DBG_HEX_C(ulChar > 0xff, ulChar);
 		fail(ulChar > 0xff);
-		vStoreByte((unsigned char)ulChar, pOutput);
+		vStoreByte((UCHAR)ulChar, pOutput);
 		iLen = 1;
 	}
 	pOutput->lStringWidth += lComputeStringWidth(
@@ -204,47 +217,50 @@ vStoreString(const char *szString, int iStringLength, output_type *pOutput)
 } /* end of vStoreString */
 
 /*
- * vStoreIntegerAsDecimal - store an integer as a decimal number
+ * vStoreNumberAsDecimal - store a number as a decimal number
  */
 static void
-vStoreIntegerAsDecimal(int iNumber, output_type *pOutput)
+vStoreNumberAsDecimal(UINT uiNumber, output_type *pOutput)
 {
 	int	iLen;
 	char	szString[15];
 
+	fail(uiNumber == 0);
 	fail(pOutput == NULL);
 
-	iLen = sprintf(szString, "%d", iNumber);
+	iLen = sprintf(szString, "%u", uiNumber);
 	vStoreString(szString, iLen, pOutput);
-} /* end of vStoreIntegerAsDecimal */
+} /* end of vStoreNumberAsDecimal */
 
 /*
- * vStoreIntegerAsRoman - store an integer as a roman numerical
+ * vStoreNumberAsRoman - store a number as a roman numerical
  */
 static void
-vStoreIntegerAsRoman(int iNumber, output_type *pOutput)
+vStoreNumberAsRoman(UINT uiNumber, output_type *pOutput)
 {
 	int	iLen;
 	char	szString[15];
 
-	fail(iNumber <= 0);
+	fail(uiNumber == 0);
 	fail(pOutput == NULL);
 
-	iLen = iInteger2Roman(iNumber, FALSE, szString);
+	iLen = iNumber2Roman(uiNumber, FALSE, szString);
 	vStoreString(szString, iLen, pOutput);
-} /* end of vStoreIntegerAsRoman */
+} /* end of vStoreNumberAsRoman */
 
 /*
  * vStoreStyle - store a style
  */
 static void
-vStoreStyle(output_type *pOutput)
+vStoreStyle(output_type *pOutput, const style_block_type *pStyle)
 {
 	int	iLen;
 	char	szString[120];
 
 	fail(pOutput == NULL);
-	iLen = iStyle2Window(szString, pStyleInfo);
+	fail(pStyle == NULL);
+
+	iLen = iStyle2Window(szString, pStyle, pSection);
 	vStoreString(szString, iLen, pOutput);
 } /* end of vStoreStyle */
 
@@ -252,78 +268,75 @@ vStoreStyle(output_type *pOutput)
  * Create an empty line by adding a extra "newline"
  */
 static void
-vEmptyLine2Diagram(diagram_type *pDiag, draw_fontref tFontRef, int iFontsize)
+vEmptyLine2Diagram(diagram_type *pDiag, draw_fontref tFontRef, short sFontsize)
 {
 	fail(pDiag == NULL);
-	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
+	fail(sFontsize < MIN_FONT_SIZE || sFontsize > MAX_FONT_SIZE);
 
 	if (pDiag->lXleft > 0) {
 		/* To the start of the line */
-		vMove2NextLine(pDiag, tFontRef, iFontsize);
+		vMove2NextLine(pDiag, tFontRef, sFontsize);
 	}
 	/* Empty line */
-	vMove2NextLine(pDiag, tFontRef, iFontsize);
+	vMove2NextLine(pDiag, tFontRef, sFontsize);
 } /* end of vEmptyLine2Diagram */
 
 /*
  * vPutIndentation - output the given amount of indentation
  */
 static void
-vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
-	int iListNumber, unsigned char ucListType, char cListCharacter,
-	int iLeftIndent)
+vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bNumPause,
+	UINT uiListNumber, UCHAR ucNFC, char cListCharacter,
+	long lLeftIndentation)
 {
-	long	lWidth, lLeftIndentation;
+	long	lWidth;
 	int	iNextFree;
 	char	szLine[30];
 
 	fail(pDiag == NULL || pOutput == NULL);
-	fail(iListNumber < 0);
-	fail(iLeftIndent < 0);
+	fail(lLeftIndentation < 0);
 
-	if (iLeftIndent <= 0) {
+	if (lLeftIndentation <= 0) {
 		return;
 	}
-	lLeftIndentation = lTwips2MilliPoints(iLeftIndent);
-	if (bUnmarked) {
+	if (bNumPause) {
 		vSetLeftIndentation(pDiag, lLeftIndentation);
 		return;
 	}
-	fail(iListNumber <= 0);
 	fail(iscntrl((int)cListCharacter));
 
-	switch (ucListType) {
+	switch (ucNFC) {
 	case LIST_ARABIC_NUM:
-		iNextFree = sprintf(szLine, "%d", iListNumber);
+		iNextFree = sprintf(szLine, "%u", uiListNumber);
 		break;
 	case LIST_ROMAN_NUM_UPPER:
 	case LIST_ROMAN_NUM_LOWER:
-		iNextFree = iInteger2Roman(iListNumber,
-			ucListType == LIST_ROMAN_NUM_UPPER, szLine);
+		iNextFree = iNumber2Roman(uiListNumber,
+				ucNFC == LIST_ROMAN_NUM_UPPER, szLine);
 		break;
 	case LIST_UPPER_ALPHA:
 	case LIST_LOWER_ALPHA:
-		iNextFree = iInteger2Alpha(iListNumber,
-			ucListType == LIST_UPPER_ALPHA, szLine);
+		iNextFree = iNumber2Alpha(uiListNumber,
+				ucNFC == LIST_UPPER_ALPHA, szLine);
 		break;
 	case LIST_ORDINAL_NUM:
-		if (iListNumber % 10 == 1 && iListNumber != 11) {
-			iNextFree = sprintf(szLine, "%dst", iListNumber);
-		} else if (iListNumber % 10 == 2 && iListNumber != 12) {
-			iNextFree = sprintf(szLine, "%dnd", iListNumber);
-		} else if (iListNumber % 10 == 3 && iListNumber != 13) {
-			iNextFree = sprintf(szLine, "%drd", iListNumber);
+		if (uiListNumber % 10 == 1 && uiListNumber != 11) {
+			iNextFree = sprintf(szLine, "%ust", uiListNumber);
+		} else if (uiListNumber % 10 == 2 && uiListNumber != 12) {
+			iNextFree = sprintf(szLine, "%und", uiListNumber);
+		} else if (uiListNumber % 10 == 3 && uiListNumber != 13) {
+			iNextFree = sprintf(szLine, "%urd", uiListNumber);
 		} else {
-			iNextFree = sprintf(szLine, "%dth", iListNumber);
+			iNextFree = sprintf(szLine, "%uth", uiListNumber);
 		}
 		break;
 	case LIST_BULLETS:
 		iNextFree = 0;
 		break;
 	default:
-		DBG_DEC(ucListType);
+		DBG_DEC(ucNFC);
 		DBG_FIXME();
-		iNextFree = sprintf(szLine, "%d", iListNumber);
+		iNextFree = sprintf(szLine, "%u", uiListNumber);
 		break;
 	}
 	szLine[iNextFree++] = cListCharacter;
@@ -406,7 +419,7 @@ pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 	int		iColor;
 	short		sFontsize;
 	draw_fontref	tFontRef;
-	unsigned char	ucFontstyle;
+	UCHAR	ucFontstyle;
 
 	iColor = FONT_COLOR_DEFAULT;
 	ucFontstyle = FONT_REGULAR;
@@ -450,15 +463,14 @@ pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
  *
  * returns the next character of EOF
  */
-static unsigned long
+static ULONG
 ulGetChar(FILE *pFile, list_id_enum eListID)
 {
 	const font_block_type	*pCurr;
-	unsigned long		ulChar;
-	long		lFileOffset, lTextOffset;
+	ULONG		ulChar, ulFileOffset, ulTextOffset;
 	row_info_enum	eRowInfo;
-	unsigned short	usChar, usPropMod;
-	BOOL	bSkip;
+	USHORT		usChar, usPropMod;
+	BOOL		bSkip;
 
 	fail(pFile == NULL);
 
@@ -466,9 +478,9 @@ ulGetChar(FILE *pFile, list_id_enum eListID)
 	bSkip = FALSE;
 	for (;;) {
 		usChar = usNextChar(pFile, eListID,
-					&lFileOffset, &lTextOffset, &usPropMod);
-		if (usChar == (unsigned short)EOF) {
-			return (unsigned long)EOF;
+				&ulFileOffset, &ulTextOffset, &usPropMod);
+		if (usChar == (USHORT)EOF) {
+			return (ULONG)EOF;
 		}
 
 		vUpdateCounters();
@@ -478,39 +490,39 @@ ulGetChar(FILE *pFile, list_id_enum eListID)
 #if 0
 			bStartRow = eRowInfo == found_a_cell ||
 				(pRowInfo != NULL &&
-				 lFileOffset == pRowInfo->lFileOffsetStart &&
+				 ulFileOffset == pRowInfo->ulFileOffsetStart &&
 				 eRowInfo != found_not_a_cell);
 #else
 			bStartRow = pRowInfo != NULL &&
-				lFileOffset == pRowInfo->lFileOffsetStart;
+				ulFileOffset == pRowInfo->ulFileOffsetStart;
 #endif
-			NO_DBG_HEX_C(bStartRow, pRowInfo->lFileOffsetStart);
+			NO_DBG_HEX_C(bStartRow, pRowInfo->ulFileOffsetStart);
 		}
 		if (!bEndRowNorm) {
 #if 0
 			bEndRow = eRowInfo == found_end_of_row ||
 				(pRowInfo != NULL &&
-				 lFileOffset == pRowInfo->lFileOffsetEnd &&
+				 ulFileOffset == pRowInfo->ulFileOffsetEnd &&
 				 eRowInfo != found_not_end_of_row);
 #else
 			bEndRowNorm = pRowInfo != NULL &&
-				lFileOffset == pRowInfo->lFileOffsetEnd;
+				ulFileOffset == pRowInfo->ulFileOffsetEnd;
 #endif
-			NO_DBG_HEX_C(bEndRowNorm, pRowInfo->lFileOffsetEnd);
+			NO_DBG_HEX_C(bEndRowNorm, pRowInfo->ulFileOffsetEnd);
 		}
 		if (!bEndRowFast) {
 			bEndRowFast = eRowInfo == found_end_of_row;
-			NO_DBG_HEX_C(bEndRowFast, pRowInfo->lFileOffsetEnd);
+			NO_DBG_HEX_C(bEndRowFast, pRowInfo->ulFileOffsetEnd);
 		}
 
 		if (!bStartStyle) {
 			bStartStyle = pStyleInfo != NULL &&
-				lFileOffset == pStyleInfo->lFileOffset;
-			NO_DBG_HEX_C(bStartStyle, lFileOffset);
+				ulFileOffset == pStyleInfo->ulFileOffset;
+			NO_DBG_HEX_C(bStartStyle, ulFileOffset);
 		}
-		if (pCurr != NULL && lFileOffset == pCurr->lFileOffset) {
+		if (pCurr != NULL && ulFileOffset == pCurr->ulFileOffset) {
 			bStartFont = TRUE;
-			NO_DBG_HEX(lFileOffset);
+			NO_DBG_HEX(ulFileOffset);
 			pFontInfo = pCurr;
 			pCurr = pGetNextFontInfoListItem(pCurr);
 		}
@@ -528,51 +540,70 @@ ulGetChar(FILE *pFile, list_id_enum eListID)
 			continue;
 		}
 		ulChar = ulTranslateCharacters(usChar,
-					lFileOffset,
-					tOptions.eEncoding == encoding_utf8,
+					ulFileOffset,
+					iWordVersion,
+					tOptions.eEncoding,
 					bWord6MacFile);
 		if (ulChar == IGNORE_CHARACTER) {
 			continue;
 		}
 		if (ulChar == PICTURE) {
-			lFileOffsetImage = lGetPicInfoListItem(lFileOffset);
+			ulFileOffsetImage = ulGetPictInfoListItem(ulFileOffset);
 		} else {
-			lFileOffsetImage = -1;
+			ulFileOffsetImage = FC_INVALID;
+		}
+		if (ulChar == PAR_END) {
+			/* End of paragraph seen, prepare for the next */
+			vFillStyleFromStylesheet(usIstdNext, &tStyleNext);
+			vCorrectStyleValues(&tStyleNext);
+			bStartStyleNext = TRUE;
+			vFillFontFromStylesheet(usIstdNext, &tFontNext);
+			vCorrectFontValues(&tFontNext);
+			bStartFontNext = TRUE;
+		}
+		if (ulChar == PAGE_BREAK) {
+			/* Might be the start of a new section */
+			pSectionNext = pGetSectionInfo(pSection, ulTextOffset);
 		}
 		return ulChar;
 	}
 } /* end of ulGetChar */
 
 /*
- * vWord2Text
+ * bWord2Text - translate Word to text or Postcript
+ *
+ * returns TRUE when succesful, otherwise FALSE
  */
-void
-vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
+BOOL
+bWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 {
 	imagedata_type	tImage;
+	const style_block_type	*pStyleTmp;
+	const font_block_type	*pFontTmp;
 	output_type	*pAnchor, *pOutput, *pLeftOver;
-	unsigned long	ulChar;
-	long	lWidthCurr, lWidthMax, lRightIndentation;
-	long	lDefaultTabWidth, lTmp;
+	ULONG	ulChar;
+	long	lBeforeIndentation, lAfterIndentation;
+	long	lLeftIndentation, lRightIndentation;
+	long	lWidthCurr, lWidthMax, lDefaultTabWidth, lTmp;
 	list_id_enum 	eListID;
 	image_info_enum	eRes;
-	int	iFootnoteNumber, iEndnoteNumber, iLeftIndent;
-	int	iListNumber;
-	BOOL	bWasTableRow, bTableFontClosed, bUnmarked;
-	BOOL	bAllCapitals, bHiddenText, bSuccess;
+	UINT	uiListNumber, uiFootnoteNumber, uiEndnoteNumber;
+	BOOL	bWasTableRow, bTableFontClosed, bWasInList, bWasEndOfParagraph;
+	BOOL	bNumPause, bAllCapitals, bHiddenText, bSuccess;
 	short	sFontsize;
-	unsigned char	ucFontnumber, ucFontcolor, ucTmp;
-	unsigned char	ucFontstyle, ucFontstyleMinimal;
-	unsigned char	ucListType, ucAlignment;
+	UCHAR	ucFontnumber, ucFontcolor, ucTmp;
+	UCHAR	ucFontstyle, ucFontstyleMinimal;
+	UCHAR	ucNFC, ucAlignment;
 	char	cListChar;
 
 	fail(pFile == NULL || lFilesize <= 0 || pDiag == NULL);
 
-	DBG_MSG("vWord2Text");
+	DBG_MSG("bWord2Text");
 
 	iWordVersion = iInitDocument(pFile, lFilesize);
-	if (iWordVersion < 6) {
-		return;
+	if (iWordVersion < 0) {
+		DBG_DEC(iWordVersion);
+		return FALSE;
 	}
 	vAddFonts2Diagram(pDiag);
 
@@ -581,14 +612,16 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 	lCharCounter = 0;
 	iCurrPct = 0;
 	iPrevPct = -1;
-	lDocumentLength = lGetDocumentLength();
+	lDocumentLength = (long)ulGetDocumentLength();
 #endif /* __riscos */
 	bWord6MacFile = bIsWord6MacFile();
+	pSection = pGetSectionInfo(NULL, 0);
+	pSectionNext = pSection;
 	lDefaultTabWidth = lGetDefaultTabWidth();
 	DBG_DEC_C(lDefaultTabWidth != 36000, lDefaultTabWidth);
 	pRowInfo = pGetNextRowInfoListItem();
-	DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetStart);
-	DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetEnd);
+	DBG_HEX_C(pRowInfo != NULL, pRowInfo->ulFileOffsetStart);
+	DBG_HEX_C(pRowInfo != NULL, pRowInfo->ulFileOffsetEnd);
 	DBG_MSG_C(pRowInfo == NULL, "No rows at all");
 	bStartRow = FALSE;
 	bEndRowNorm = FALSE;
@@ -596,11 +629,13 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 	bIsTableRow = FALSE;
 	bWasTableRow = FALSE;
 	vResetStyles();
-	pStyleInfo = pGetNextStyleInfoListItem();
+	pStyleInfo = pGetNextStyleInfoListItem(NULL);
 	bStartStyle = FALSE;
+	bWasInList = FALSE;
+	usIstdNext = ISTD_NORMAL;
 	pAnchor = NULL;
 	pFontInfo = pGetNextFontInfoListItem(NULL);
-	DBG_HEX_C(pFontInfo != NULL, pFontInfo->lFileOffset);
+	DBG_HEX_C(pFontInfo != NULL, pFontInfo->ulFileOffset);
 	DBG_MSG_C(pFontInfo == NULL, "No fonts at all");
 	bStartFont = FALSE;
 	ucFontnumber = 0;
@@ -610,17 +645,20 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 	ucFontcolor = FONT_COLOR_DEFAULT;
 	pAnchor = pStartNewOutput(pAnchor, NULL);
 	pOutput = pAnchor;
-	pOutput->iColor = ucFontcolor;
+	pOutput->iColor = (int)ucFontcolor;
 	pOutput->ucFontstyle = ucFontstyle;
 	pOutput->tFontRef = tOpenFont(ucFontnumber, ucFontstyle, sFontsize);
 	pOutput->sFontsize = sFontsize;
 	bTableFontClosed = TRUE;
-	iLeftIndent = 0;
+	lBeforeIndentation = 0;
+	lAfterIndentation = 0;
+	lLeftIndentation = 0;
 	lRightIndentation = 0;
-	bUnmarked = TRUE;
-	ucListType = LIST_BULLETS;
+	bWasEndOfParagraph = TRUE;
+	bNumPause = TRUE;
+	ucNFC = LIST_BULLETS;
 	cListChar = OUR_BULLET;
-	iListNumber = 0;
+	uiListNumber = 0;
 	ucAlignment = ALIGNMENT_LEFT;
 	bAllCapitals = FALSE;
 	bHiddenText = FALSE;
@@ -639,12 +677,12 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 
 	visdelay_begin();
 
-	iFootnoteNumber = 0;
-	iEndnoteNumber = 0;
+	uiFootnoteNumber = 0;
+	uiEndnoteNumber = 0;
 	eListID = text_list;
 	for(;;) {
 		ulChar = ulGetChar(pFile, eListID);
-		if (ulChar == (unsigned long)EOF) {
+		if (ulChar == (ULONG)EOF) {
 			if (bOutputContainsText(pAnchor)) {
 				OUTPUT_LINE();
 			} else {
@@ -653,18 +691,18 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			switch (eListID) {
 			case text_list:
 				eListID = footnote_list;
-				if (iFootnoteNumber > 0) {
+				if (uiFootnoteNumber != 0) {
 					vPutNoteSeparator(pAnchor);
 					OUTPUT_LINE();
-					iFootnoteNumber = 0;
+					uiFootnoteNumber = 0;
 				}
 				break;
 			case footnote_list:
 				eListID = endnote_list;
-				if (iEndnoteNumber > 0) {
+				if (uiEndnoteNumber != 0) {
 					vPutNoteSeparator(pAnchor);
 					OUTPUT_LINE();
-					iEndnoteNumber = 0;
+					uiEndnoteNumber = 0;
 				}
 				break;
 			case endnote_list:
@@ -725,7 +763,7 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 		    !bIsTableRow &&
 		    ulChar != PAR_END &&
 		    ulChar != HARD_RETURN &&
-		    ulChar != FORM_FEED &&
+		    ulChar != PAGE_BREAK &&
 		    ulChar != COLUMN_FEED) {
 			/*
 			 * The end of a table should be followed by an
@@ -733,15 +771,16 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			 */
 			OUTPUT_LINE();
 			vEndOfParagraph2Diagram(pDiag,
-						pOutput->tFontRef,
-						pOutput->sFontsize);
+					pOutput->tFontRef,
+					pOutput->sFontsize,
+					(long)pOutput->sFontsize * 600);
 		}
 
 		switch (ulChar) {
-		case FORM_FEED:
+		case PAGE_BREAK:
 		case COLUMN_FEED:
 			if (bIsTableRow) {
-				vStoreCharacter('\n', pOutput);
+				vStoreCharacter((ULONG)'\n', pOutput);
 				break;
 			}
 			if (bOutputContainsText(pAnchor)) {
@@ -749,94 +788,140 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			} else {
 				RESET_LINE();
 			}
-			if (ulChar == FORM_FEED) {
+			if (ulChar == PAGE_BREAK) {
 				vEndOfPage2Diagram(pDiag,
 						pOutput->tFontRef,
-						pOutput->sFontsize);
+						pOutput->sFontsize,
+						lAfterIndentation);
 			} else {
 				vEndOfParagraph2Diagram(pDiag,
 						pOutput->tFontRef,
-						pOutput->sFontsize);
+						pOutput->sFontsize,
+						lAfterIndentation);
 			}
 			break;
 		default:
 			break;
 		}
 
-		if (bStartFont) {
+		if (bStartFont || (bStartFontNext && ulChar != PAR_END)) {
 			/* Begin of a font found */
-			fail(pFontInfo == NULL);
-			bAllCapitals = bIsCapitals(pFontInfo->ucFontstyle);
-			bHiddenText = bIsHidden(pFontInfo->ucFontstyle);
-			ucTmp = pFontInfo->ucFontstyle &
-			(FONT_BOLD|FONT_ITALIC|FONT_UNDERLINE|FONT_STRIKE);
+			if (bStartFont) {
+				/* bStartFont takes priority */
+				fail(pFontInfo == NULL);
+				pFontTmp = pFontInfo;
+			} else {
+				pFontTmp = &tFontNext;
+			}
+			bAllCapitals = bIsCapitals(pFontTmp->ucFontstyle);
+			bHiddenText = bIsHidden(pFontTmp->ucFontstyle);
+			ucTmp = pFontTmp->ucFontstyle &
+				(FONT_BOLD|FONT_ITALIC|FONT_UNDERLINE|
+				 FONT_STRIKE|FONT_MARKDEL);
 			if (!bIsTableRow &&
-			    (sFontsize != pFontInfo->sFontsize ||
-			    ucFontnumber != pFontInfo->ucFontnumber ||
+			    (sFontsize != pFontTmp->sFontsize ||
+			    ucFontnumber != pFontTmp->ucFontnumber ||
 			    ucFontstyleMinimal != ucTmp ||
-			    ucFontcolor != pFontInfo->ucFontcolor)) {
+			    ucFontcolor != pFontTmp->ucFontcolor)) {
 				pOutput = pStartNextOutput(pOutput);
 				vCloseFont();
-				pOutput->iColor = pFontInfo->ucFontcolor;
-				pOutput->ucFontstyle = pFontInfo->ucFontstyle;
-				pOutput->sFontsize = pFontInfo->sFontsize;
+				pOutput->iColor = (int)pFontTmp->ucFontcolor;
+				pOutput->ucFontstyle = pFontTmp->ucFontstyle;
+				pOutput->sFontsize = pFontTmp->sFontsize;
 				pOutput->tFontRef = tOpenFont(
-						pFontInfo->ucFontnumber,
-						pFontInfo->ucFontstyle,
-						pFontInfo->sFontsize);
+						pFontTmp->ucFontnumber,
+						pFontTmp->ucFontstyle,
+						pFontTmp->sFontsize);
 				fail(!bCheckDoubleLinkedList(pAnchor));
 			}
-			ucFontnumber = pFontInfo->ucFontnumber;
-			sFontsize = pFontInfo->sFontsize;
-			ucFontcolor = pFontInfo->ucFontcolor;
-			ucFontstyle = pFontInfo->ucFontstyle;
+			ucFontnumber = pFontTmp->ucFontnumber;
+			sFontsize = pFontTmp->sFontsize;
+			ucFontcolor = pFontTmp->ucFontcolor;
+			ucFontstyle = pFontTmp->ucFontstyle;
 			ucFontstyleMinimal = ucTmp;
-			pFontInfo = pGetNextFontInfoListItem(pFontInfo);
-			NO_DBG_HEX_C(pFontInfo != NULL, pFontInfo->lFileOffset);
-			DBG_MSG_C(pFontInfo == NULL, "No more fonts");
+			if (bStartFont) {
+				/* Get the next font info */
+				pFontInfo = pGetNextFontInfoListItem(pFontInfo);
+				NO_DBG_HEX_C(pFontInfo != NULL,
+						pFontInfo->ulFileOffset);
+				DBG_MSG_C(pFontInfo == NULL, "No more fonts");
+			}
 			bStartFont = FALSE;
+			bStartFontNext = FALSE;
 		}
 
-		if (bStartStyle) {
+		if (bStartStyle || (bStartStyleNext && ulChar != PAR_END)) {
 			/* Begin of a style found */
-			fail(pStyleInfo == NULL);
-			if (!bIsTableRow) {
-				vStoreStyle(pOutput);
+			if (bStartStyle) {
+				/* bStartStyle takes priority */
+				fail(pStyleInfo == NULL);
+				pStyleTmp = pStyleInfo;
+			} else {
+				pStyleTmp = &tStyleNext;
 			}
-			iLeftIndent = pStyleInfo->sLeftIndent;
+			if (!bIsTableRow) {
+				vStoreStyle(pOutput, pStyleTmp);
+			}
+			usIstdNext = pStyleTmp->usIstdNext;
+			lBeforeIndentation =
+				lTwips2MilliPoints(pStyleTmp->usBeforeIndent);
+			lAfterIndentation =
+				lTwips2MilliPoints(pStyleTmp->usAfterIndent);
+			lLeftIndentation =
+				lTwips2MilliPoints(pStyleTmp->sLeftIndent);
 			lRightIndentation =
-				lTwips2MilliPoints(pStyleInfo->sRightIndent);
-			bUnmarked = !pStyleInfo->bInList ||
-						pStyleInfo->bUnmarked;
-			ucListType = pStyleInfo->ucListType;
-			cListChar = (char)pStyleInfo->ucListCharacter;
-			ucAlignment = pStyleInfo->ucAlignment;
-			if (pStyleInfo->bInList) {
-				if (!pStyleInfo->bUnmarked) {
-					iListNumber++;
+				lTwips2MilliPoints(pStyleTmp->sRightIndent);
+			bNumPause = !pStyleTmp->bInList ||
+						pStyleTmp->bNumPause;
+			ucNFC = pStyleTmp->ucNFC;
+			cListChar = (char)pStyleTmp->ucListCharacter;
+			ucAlignment = pStyleTmp->ucAlignment;
+			if (pStyleTmp->bInList) {
+				if (bWasInList) {
+					if (!pStyleTmp->bNumPause) {
+						uiListNumber++;
+					}
+				} else {
+					uiListNumber =
+						(UINT)pStyleTmp->usStartAt;
 				}
 			} else {
-				iListNumber = 0;
+				uiListNumber = 0;
+			}
+			bWasInList = pStyleTmp->bInList;
+			if (bStartStyle) {
+				pStyleInfo =
+					pGetNextStyleInfoListItem(pStyleInfo);
+				NO_DBG_HEX_C(pStyleInfo != NULL,
+						pStyleInfo->ulFileOffset);
+				NO_DBG_MSG_C(pStyleInfo == NULL,
+						"No more styles");
 			}
-			pStyleInfo = pGetNextStyleInfoListItem();
-			NO_DBG_HEX_C(pStyleInfo != NULL,
-						pStyleInfo->lFileOffset);
 			bStartStyle = FALSE;
+			bStartStyleNext = FALSE;
+		}
+
+		if (bWasEndOfParagraph) {
+			vStartOfParagraph2Diagram(pDiag,
+						pOutput->tFontRef,
+						pOutput->sFontsize,
+						lBeforeIndentation);
+			bWasEndOfParagraph = FALSE;
 		}
 
 		if (!bIsTableRow &&
 		    lTotalStringWidth(pAnchor) == 0) {
-			vPutIndentation(pDiag, pAnchor, bUnmarked,
-					iListNumber, ucListType, cListChar,
-					iLeftIndent);
-			/* One mark per paragraph will do */
-			bUnmarked = TRUE;
+			vPutIndentation(pDiag, pAnchor, bNumPause,
+					uiListNumber, ucNFC, cListChar,
+					lLeftIndentation);
+			/* One number or mark per paragraph will do */
+			bNumPause = TRUE;
 		}
 
 		switch (ulChar) {
 		case PICTURE:
 			(void)memset(&tImage, 0, sizeof(tImage));
-			eRes = eExamineImage(pFile, lFileOffsetImage, &tImage);
+			eRes = eExamineImage(pFile, ulFileOffsetImage, &tImage);
 			switch (eRes) {
 			case image_no_information:
 				bSuccess = FALSE;
@@ -852,7 +937,7 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 #endif
 				bSuccess = bTranslateImage(pDiag, pFile,
 					eRes == image_minimal_information,
-					lFileOffsetImage, &tImage);
+					ulFileOffsetImage, &tImage);
 				break;
 			default:
 				DBG_DEC(eRes);
@@ -864,45 +949,35 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			}
 			break;
 		case FOOTNOTE_CHAR:
-			iFootnoteNumber++;
-			vStoreCharacter('[', pOutput);
-			vStoreIntegerAsDecimal(iFootnoteNumber, pOutput);
-			vStoreCharacter(']', pOutput);
+			uiFootnoteNumber++;
+			vStoreCharacter((ULONG)'[', pOutput);
+			vStoreNumberAsDecimal(uiFootnoteNumber, pOutput);
+			vStoreCharacter((ULONG)']', pOutput);
 			break;
 		case ENDNOTE_CHAR:
-			iEndnoteNumber++;
-			vStoreCharacter('[', pOutput);
-			vStoreIntegerAsRoman(iEndnoteNumber, pOutput);
-			vStoreCharacter(']', pOutput);
+			uiEndnoteNumber++;
+			vStoreCharacter((ULONG)'[', pOutput);
+			vStoreNumberAsRoman(uiEndnoteNumber, pOutput);
+			vStoreCharacter((ULONG)']', pOutput);
 			break;
 		case UNKNOWN_NOTE_CHAR:
 			vStoreString("[?]", 3, pOutput);
 			break;
 		case PAR_END:
 			if (bIsTableRow) {
-				vStoreCharacter('\n', pOutput);
+				vStoreCharacter((ULONG)'\n', pOutput);
 				break;
 			}
-			if (bOutputContainsText(pAnchor)) {
-				OUTPUT_LINE();
-			} else {
-				RESET_LINE();
-			}
+			OUTPUT_LINE();
 			vEndOfParagraph2Diagram(pDiag,
 						pOutput->tFontRef,
-						pOutput->sFontsize);
-			/*
-			 * End of paragraph seen, reset indentation,
-			 * marking and alignment
-			 */
-			iLeftIndent = 0;
-			lRightIndentation = 0;
-			bUnmarked = TRUE;
-			ucAlignment = ALIGNMENT_LEFT;
+						pOutput->sFontsize,
+						lAfterIndentation);
+			bWasEndOfParagraph = TRUE;
 			break;
 		case HARD_RETURN:
 			if (bIsTableRow) {
-				vStoreCharacter('\n', pOutput);
+				vStoreCharacter((ULONG)'\n', pOutput);
 				break;
 			}
 			if (bOutputContainsText(pAnchor)) {
@@ -912,21 +987,21 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 					pOutput->tFontRef,
 					pOutput->sFontsize);
 			break;
-		case FORM_FEED:
+		case PAGE_BREAK:
 		case COLUMN_FEED:
-			/* Already dealt with */
+			pSection = pSectionNext;
 			break;
 		case TABLE_SEPARATOR:
 			if (bIsTableRow) {
 				vStoreCharacter(ulChar, pOutput);
 				break;
 			}
-			vStoreCharacter(' ', pOutput);
+			vStoreCharacter((ULONG)' ', pOutput);
 			vStoreCharacter(TABLE_SEPARATOR_CHAR, pOutput);
 			break;
 		case TAB:
 			if (bIsTableRow) {
-				vStoreCharacter(' ', pOutput);
+				vStoreCharacter((ULONG)' ', pOutput);
 				break;
 			}
 			if (tOptions.iParagraphBreak == 0 &&
@@ -991,8 +1066,10 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			bIsTableRow = FALSE;
 			bEndRowNorm = FALSE;
 			bEndRowFast = FALSE;
-			DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetStart);
-			DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetEnd);
+			NO_DBG_HEX_C(pRowInfo != NULL,
+						pRowInfo->ulFileOffsetStart);
+			NO_DBG_HEX_C(pRowInfo != NULL,
+						pRowInfo->ulFileOffsetEnd);
 			continue;
 		}
 		lWidthCurr = lTotalStringWidth(pAnchor);
@@ -1010,8 +1087,7 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 			;	/* EMPTY */
 		fail(pOutput == NULL);
 		if (lTotalStringWidth(pAnchor) > 0) {
-			vSetLeftIndentation(pDiag,
-					lTwips2MilliPoints(iLeftIndent));
+			vSetLeftIndentation(pDiag, lLeftIndentation);
 		}
 	}
 
@@ -1021,4 +1097,5 @@ vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 	vCloseFont();
 	vFreeDocument();
 	visdelay_end();
-} /* end of vWord2Text */
+	return TRUE;
+} /* end of bWord2Text */
