@@ -1,6 +1,6 @@
 /*
  * wordtypes.h
- * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2002 A.J. van Os; Released under GPL
  *
  * Description:
  * Typedefs for the interpretation of MS Word files
@@ -9,6 +9,11 @@
 #if !defined(__wordtypes_h)
 #define __wordtypes_h 1
 
+typedef unsigned char	UCHAR;
+typedef unsigned short	USHORT;
+typedef unsigned int	UINT;
+typedef unsigned long	ULONG;
+
 #if defined(__riscos)
 typedef struct diagram_tag {
 	draw_diag	tInfo;
@@ -27,18 +32,18 @@ typedef struct diagram_tag {
 	long		lXleft;			/* In DrawUnits */
 	long		lYtop;			/* In DrawUnits */
 } diagram_type;
-typedef unsigned char	draw_fontref;
+typedef UCHAR	draw_fontref;
 #endif /* __riscos */
 
 typedef struct output_tag {
-  	char		*szStorage;
-	long		lStringWidth;		/* In millipoints */
-	size_t		tStorageSize;
-	int		iNextFree;
-	int		iColor;
-	short		sFontsize;
-	unsigned char	ucFontstyle;
-	draw_fontref	tFontRef;
+  	char	*szStorage;
+	long	lStringWidth;		/* In millipoints */
+	size_t	tStorageSize;
+	int	iNextFree;
+	int	iColor;
+	short	sFontsize;
+	UCHAR	ucFontstyle;
+	draw_fontref		tFontRef;
 	struct output_tag	*pPrev;
 	struct output_tag	*pNext;
 } output_type;
@@ -53,11 +58,11 @@ typedef enum encoding_tag {
 
 /* Font translation table entry */
 typedef struct font_table_tag {
-	unsigned char	ucWordFontnumber;
-	unsigned char	ucFontstyle;
-	unsigned char	ucInUse;
-	char		szWordFontname[65];
-	char		szOurFontname[33];
+	UCHAR	ucWordFontnumber;
+	UCHAR	ucFontstyle;
+	UCHAR	ucInUse;
+	char	szWordFontname[65];
+	char	szOurFontname[33];
 } font_table_type;
 
 /* Options */
@@ -86,8 +91,8 @@ typedef struct options_tag {
 
 /* Property Set Storage */
 typedef struct pps_tag {
-	long	lSize;
-	long	lSb;
+	ULONG	ulSB;
+	ULONG	ulSize;
 } pps_type;
 typedef struct pps_info_tag {
 	pps_type	tWordDocument;
@@ -98,56 +103,65 @@ typedef struct pps_info_tag {
 
 /* Record of data block information */
 typedef struct data_block_tag {
-	long	lFileOffset;
-	long	lDataOffset;
-	long	lLength;
+	ULONG	ulFileOffset;
+	ULONG	ulDataPos;
+	ULONG	ulLength;
 } data_block_type;
 
 /* Record of text block information */
 typedef struct text_block_tag {
-	long	lFileOffset;
-	long	lTextOffset;
-	long	lLength;
+	ULONG	ulFileOffset;
+	ULONG	ulCharPos;
+	ULONG	ulLength;
 	BOOL	bUsesUnicode;	/* This block uses 16 bits per character */
-	unsigned short	usPropMod;
+	USHORT	usPropMod;
 } text_block_type;
 
 /* Record of table-row block information */
 typedef struct row_block_tag {
-	long	lFileOffsetStart;
-	long	lFileOffsetEnd;
-	long	lTextOffsetStart;
-	long	lTextOffsetEnd;
+	ULONG	ulFileOffsetStart;
+	ULONG	ulFileOffsetEnd;
+	ULONG	ulCharPosStart;
+	ULONG	ulCharPosEnd;
 	int	iColumnWidthSum;			/* In twips */
 	short	asColumnWidth[TABLE_COLUMN_MAX+1];	/* In twips */
-	unsigned char	ucNumberOfColumns;
+	UCHAR	ucNumberOfColumns;
 } row_block_type;
 
+/* Various level types */
+typedef enum level_type_tag {
+	level_type_none = 0,
+	level_type_outline,
+	level_type_numbering,
+	level_type_sequence,
+	level_type_pause
+} level_type_enum;
+
 /* Linked list of style description information */
 typedef struct style_block_tag {
-	long	lFileOffset;
+	ULONG	ulFileOffset;
 	BOOL	bInList;
-	BOOL	bUnmarked;
+	BOOL	bNumPause;
+	USHORT	usIstd;		/* Current style */
+	USHORT	usIstdNext;	/* Next style unless overruled */
+	USHORT	usStartAt;
+	USHORT	usBeforeIndent;	/* Vertical indent before paragraph in twips */
+	USHORT	usAfterIndent;	/* Vertical indent after paragraph in twips */
 	short	sLeftIndent;	/* Left indentation in twips */
 	short	sRightIndent;	/* Right indentation in twips */
-	unsigned char	ucStyle;
-	unsigned char	ucAlignment;
-	unsigned char	ucListType;
-	unsigned char	ucListLevel;
-	unsigned char	ucListCharacter;
+	UCHAR	ucAlignment;
+	UCHAR	ucNFC;		/* Number format code */
+	UCHAR	ucListLevel;
+	UCHAR	ucListCharacter;
 } style_block_type;
-typedef struct style_desc_tag {
-	style_block_type	tInfo;
-	struct style_desc_tag	*pNext;
-} style_desc_type;
 
 /* Linked list of font description information */
 typedef struct font_block_tag {
-	long	lFileOffset;
-	short		sFontsize;
-	unsigned char	ucFontnumber;
-	unsigned char	ucFontcolor;
-	unsigned char	ucFontstyle;
+	ULONG	ulFileOffset;
+	short	sFontsize;
+	UCHAR	ucFontnumber;
+	UCHAR	ucFontcolor;
+	UCHAR	ucFontstyle;
 } font_block_type;
 typedef struct font_desc_tag {
 	font_block_type	tInfo;
@@ -156,15 +170,23 @@ typedef struct font_desc_tag {
 
 /* Linked list of picture description information */
 typedef struct picture_block_tag {
-	long	lFileOffset;
-	long	lFileOffsetPicture;
-	long	lPictureOffset;
+	ULONG	ulFileOffset;
+	ULONG	ulFileOffsetPicture;
+	ULONG	ulPictureOffset;
 } picture_block_type;
 typedef struct picture_desc_tag {
 	picture_block_type	tInfo;
 	struct picture_desc_tag	*pNext;
 } picture_desc_type;
 
+/* Section description information */
+typedef struct section_block_tag {
+	BOOL	bNewPage;
+	UCHAR	aucNFC[9];		/* Number format code */
+	USHORT	usNeedPrevLvl;		/* Print previous level numbers */
+	USHORT	usHangingIndent;
+} section_block_type;
+
 /* Types of images */
 typedef enum imagetype_tag {
 	imagetype_is_unknown = 0,
@@ -205,7 +227,7 @@ typedef struct imagedata_tag {
 	compression_enum	eCompression;	/* Type of compression */
 	BOOL	bColorImage;	/* Is color image */
 	int	iColorsUsed;	/* 0 = uses the maximum number of colors */
-	unsigned char aucPalette[256][3];	/* RGB palette */
+	UCHAR 	aucPalette[256][3];	/* RGB palette */
 } imagedata_type;
 
 typedef enum row_info_tag {
